@@ -1,12 +1,17 @@
 #ifndef UDPFROMTO_H
 #define UDPFROMTO_H
 /*
- * Version:	$Id$
+ * Version:	$Id: udpfromto.h,v 1.3 2007/04/05 10:52:14 aland Exp $
  *
  */
 
-#include <sys/socket.h>
+#include <freeradius-devel/ident.h>
+RCSIDH(udpfromtoh, "$Id: udpfromto.h,v 1.3 2007/04/05 10:52:14 aland Exp $")
 
+#include <freeradius-devel/autoconf.h>
+#include <freeradius-devel/libradius.h>
+
+#ifdef WITH_UDPFROMTO
 int udpfromto_init(int s);
 int recvfromto(int s, void *buf, size_t len, int flags,
 	       struct sockaddr *from, socklen_t *fromlen,
@@ -14,5 +19,6 @@ int recvfromto(int s, void *buf, size_t len, int flags,
 int sendfromto(int s, void *buf, size_t len, int flags,
 	       struct sockaddr *from, socklen_t fromlen,
 	       struct sockaddr *to, socklen_t tolen);
+#endif
 
 #endif
