@@ -2,7 +2,7 @@
  * missing.c	Replacements for functions that are or can be
  *		missing on some platforms.
  *
- * Version:	$Id$
+ * Version:	$Id: missing.c,v 1.22 2008/04/20 14:51:25 aland Exp $
  *
  *   This library is free software; you can redistribute it and/or
  *   modify it under the terms of the GNU Lesser General Public
@@ -18,29 +18,21 @@
  *   License along with this library; if not, write to the Free Software
  *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
  *
- * Copyright 2000  The FreeRADIUS server project
+ * Copyright 2000,2006  The FreeRADIUS server project
  */
 
-static const char rcsid[] = "$Id$";
+#include	<freeradius-devel/ident.h>
+RCSID("$Id: missing.c,v 1.22 2008/04/20 14:51:25 aland Exp $")
 
-#include	"autoconf.h"
+#include	<freeradius-devel/libradius.h>
 
-#include	<stdio.h>
-#include	<stdlib.h>
-#include	<sys/types.h>
-#include	<sys/socket.h>
-#include	<netinet/in.h>
-#include	<arpa/inet.h>
 #include	<ctype.h>
 
-#include	"libradius.h"
-#include	"missing.h"
-
 #ifndef HAVE_CRYPT
-char *crypt(char *key, char *salt)
+char *crypt(UNUSED char *key, char *salt)
 {
 	/*log(L_ERR, "crypt() called but not implemented");*/
-	return "____fnord____";
+	return salt;
 }
 #endif
 
@@ -88,7 +80,7 @@ int strcasecmp(char *s1, char *s2)
 #endif
 
 #ifndef HAVE_INET_ATON
-int inet_aton(char *cp, struct in_addr *inp)
+int inet_aton(const char *cp, struct in_addr *inp)
 {
 	int	a1, a2, a3, a4;
 
@@ -100,19 +92,6 @@ int inet_aton(char *cp, struct in_addr *inp)
 }
 #endif
 
-#ifndef HAVE_GETHOSTNAME
-int gethostname(char *name, int len)
-{
-	char		*h;
-
-	h = getenv("HOSTNAME");
-	if (!h || (strlen(h) + 1 > len))
-		return -1;
-	strcpy(name, h);
-	return 0;
-}
-#endif
-
 #ifndef HAVE_STRSEP
 /*
  *	Get next token from string *stringp, where tokens are
@@ -212,3 +191,85 @@ struct tm *gmtime_r(const time_t *l_clock, struct tm *result)
   return result;
 }
 #endif
+
+#ifndef HAVE_GETTIMEOFDAY
+#ifdef WIN32
+/*
+ * Number of micro-seconds between the beginning of the Windows epoch
+ * (Jan. 1, 1601) and the Unix epoch (Jan. 1, 1970).
+ *
+ * This assumes all Win32 compilers have 64-bit support.
+ */
+#if defined(_MSC_VER) || defined(_MSC_EXTENSIONS) || defined(__WATCOMC__)
+#define DELTA_EPOCH_IN_USEC  11644473600000000Ui64
+#else
+#define DELTA_EPOCH_IN_USEC  11644473600000000ULL
+#endif
+
+static uint64_t filetime_to_unix_epoch (const FILETIME *ft)
+{
+	uint64_t res = (uint64_t) ft->dwHighDateTime << 32;
+
+	res |= ft->dwLowDateTime;
+	res /= 10;                   /* from 100 nano-sec periods to usec */
+	res -= DELTA_EPOCH_IN_USEC;  /* from Win epoch to Unix epoch */
+	return (res);
+}
+
+int gettimeofday (struct timeval *tv, UNUSED void *tz)
+{
+	FILETIME  ft;
+	uint64_t tim;
+
+	if (!tv) {
+		errno = EINVAL;
+		return (-1);
+	}
+        GetSystemTimeAsFileTime (&ft);
+        tim = filetime_to_unix_epoch (&ft);
+        tv->tv_sec  = (long) (tim / 1000000L);
+        tv->tv_usec = (long) (tim % 1000000L);
+        return (0);
+}
+#endif
+#endif
+
+#define NTP_EPOCH_OFFSET	2208988800ULL
+
+/*
+ *	Convert 'struct timeval' into NTP format (32-bit integer
+ *	of seconds, 32-bit integer of fractional seconds)
+ */
+void
+timeval2ntp(const struct timeval *tv, uint8_t *ntp)
+{
+	uint32_t sec, usec;
+
+	sec = tv->tv_sec + NTP_EPOCH_OFFSET;
+	usec = tv->tv_usec * 4295; /* close enough to 2^32 / USEC */
+	usec -= ((tv->tv_usec * 2143) >> 16); /*  */
+
+	sec = htonl(sec);
+	usec = htonl(usec);
+
+	memcpy(ntp, &sec, sizeof(sec));
+	memcpy(ntp + sizeof(sec), &usec, sizeof(usec));
+}
+
+/*
+ *	Inverse of timeval2ntp
+ */
+void
+ntp2timeval(struct timeval *tv, const char *ntp)
+{
+	uint32_t sec, usec;
+
+	memcpy(&sec, ntp, sizeof(sec));
+	memcpy(&usec, ntp + sizeof(sec), sizeof(usec));
+
+	sec = ntohl(sec);
+	usec = ntohl(usec);
+
+	tv->tv_sec = sec - NTP_EPOCH_OFFSET;
+	tv->tv_usec = usec / 4295; /* close enough */
+}
