@@ -1,7 +1,7 @@
 /*
  * auth.c	User authentication.
  *
- * Version:	$Id$
+ * Version:	$Id: auth.c,v 1.178 2008/04/18 09:29:49 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -15,34 +15,28 @@
  *
  *   You should have received a copy of the GNU General Public License
  *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
  *
- * Copyright 2000  The FreeRADIUS server project
+ * Copyright 2000,2006  The FreeRADIUS server project
  * Copyright 2000  Miquel van Smoorenburg <miquels@cistron.nl>
  * Copyright 2000  Jeff Carneal <jeff@apex.net>
  */
 
-static const char rcsid[] = "$Id$";
+#include <freeradius-devel/ident.h>
+RCSID("$Id: auth.c,v 1.178 2008/04/18 09:29:49 aland Exp $")
 
-#include "autoconf.h"
-#include "libradius.h"
+#include <freeradius-devel/radiusd.h>
+#include <freeradius-devel/modules.h>
+#include <freeradius-devel/rad_assert.h>
 
-#include <stdlib.h>
-#include <string.h>
 #include <ctype.h>
 
-#ifdef HAVE_NETINET_IN_H
-#	include <netinet/in.h>
-#endif
-
-#include "radiusd.h"
-#include "modules.h"
-
 /*
  *	Return a short string showing the terminal server, port
  *	and calling station ID.
  */
-char *auth_name(char *buf, size_t buflen, REQUEST *request, int do_cli) {
+char *auth_name(char *buf, size_t buflen, REQUEST *request, int do_cli)
+{
 	VALUE_PAIR	*cli;
 	VALUE_PAIR	*pair;
 	int		port = 0;
@@ -50,62 +44,17 @@ char *auth_name(char *buf, size_t buflen, REQUEST *request, int do_cli) {
 	if ((cli = pairfind(request->packet->vps, PW_CALLING_STATION_ID)) == NULL)
 		do_cli = 0;
 	if ((pair = pairfind(request->packet->vps, PW_NAS_PORT)) != NULL)
-		port = pair->lvalue;
+		port = pair->vp_integer;
 
-	snprintf(buf, buflen, "from client %.128s port %u%s%.128s",
-			client_name(request->packet->src_ipaddr), port,
-			(do_cli ? " cli " : ""), (do_cli ? (char *)cli->strvalue : ""));
+	snprintf(buf, buflen, "from client %.128s port %u%s%.128s%s",
+			request->client->shortname, port,
+		 (do_cli ? " cli " : ""), (do_cli ? (char *)cli->vp_strvalue : ""),
+		 (request->packet->dst_port == 0) ? " via TLS tunnel" : "");
 
 	return buf;
 }
 
 
-/*
- *	Check if account has expired, and if user may login now.
- */
-static int check_expiration(REQUEST *request)
-{
-	VALUE_PAIR *check_item;
-	VALUE_PAIR *vp;
-
-	check_item = pairfind(request->config_items, PW_EXPIRATION);
-
-	if (!check_item)  return 0;
-
-	/*
-	 *	Has this user's password expired?
-	 *
-	 *	If so, remove ALL reply attributes,
-	 *	and add our own Reply-Message, saying
-	 *	why they're being rejected.
-	 */
-	if (((time_t) check_item->lvalue) <= request->timestamp) {
-		vp = pairmake("Reply-Message",
-			      "Password Has Expired\r\n",
-			      T_OP_ADD);
-		pairfree(&request->reply->vps);
-		request->reply->vps = vp;
-		return -1;
-	}
-
-#define EXP_TIMEOUT ((uint32_t) (((time_t) check_item->lvalue) - request->timestamp))
-	/*
-	 *	Otherwise, set the Session-Timeout based on expiration.
-	 */
-	vp = pairfind(request->reply->vps, PW_SESSION_TIMEOUT);
-	if (!vp) {
-		vp = pairmake("Session-Timeout", "0", T_OP_SET);
-		if (!vp) return -1; /* out of memory */
-		
-		vp->lvalue = EXP_TIMEOUT;
-		pairadd(&request->reply->vps, vp);
-	} else if (vp->lvalue > EXP_TIMEOUT) {
-		vp->lvalue = EXP_TIMEOUT;
-	} /* else Session-Timeout is smaller than Expiration, leave it alone */
-
-	return 0;
-}
-
 
 /*
  * Make sure user/pass are clean
@@ -118,7 +67,7 @@ static int rad_authlog(const char *msg, REQUEST *request, int goodpass) {
 	char buf[1024];
 	VALUE_PAIR *username = NULL;
 
-	if (!mainconfig.log_auth) {
+	if (!request->root->log_auth) {
 		return 0;
 	}
 
@@ -137,7 +86,7 @@ static int rad_authlog(const char *msg, REQUEST *request, int goodpass) {
 	if (username == NULL) {
 		strcpy(clean_username, "<no User-Name attribute>");
 	} else {
-		librad_safeprint((char *)username->strvalue,
+		librad_safeprint((char *)username->vp_strvalue,
 				username->length,
 				clean_username, sizeof(clean_username));
 	}
@@ -145,13 +94,23 @@ static int rad_authlog(const char *msg, REQUEST *request, int goodpass) {
 	/*
 	 *	Clean up the password
 	 */
-	if (mainconfig.log_auth_badpass || mainconfig.log_auth_goodpass) {
+	if (request->root->log_auth_badpass || request->root->log_auth_goodpass) {
 		if (!request->password) {
-			strcpy(clean_password, "<no User-Password attribute>");
-		} else if (request->password->attribute == PW_CHAP_PASSWORD) {
+			VALUE_PAIR *auth_type;
+
+			auth_type = pairfind(request->config_items,
+					     PW_AUTH_TYPE);
+			if (auth_type && (auth_type->vp_strvalue[0] != '\0')) {
+				snprintf(clean_password, sizeof(clean_password),
+					 "<via Auth-Type = %s>",
+					 auth_type->vp_strvalue);
+			} else {
+				strcpy(clean_password, "<no User-Password attribute>");
+			}
+		} else if (pairfind(request->packet->vps, PW_CHAP_PASSWORD)) {
 			strcpy(clean_password, "<CHAP-Password>");
 		} else {
-			librad_safeprint((char *)request->password->strvalue,
+			librad_safeprint((char *)request->password->vp_strvalue,
 					 request->password->length,
 					 clean_password, sizeof(clean_password));
 		}
@@ -161,15 +120,15 @@ static int rad_authlog(const char *msg, REQUEST *request, int goodpass) {
 		radlog(L_AUTH, "%s: [%s%s%s] (%s)",
 				msg,
 				clean_username,
-				mainconfig.log_auth_goodpass ? "/" : "",
-				mainconfig.log_auth_goodpass ? clean_password : "",
+				request->root->log_auth_goodpass ? "/" : "",
+				request->root->log_auth_goodpass ? clean_password : "",
 				auth_name(buf, sizeof(buf), request, 1));
 	} else {
 		radlog(L_AUTH, "%s: [%s%s%s] (%s)",
 				msg,
 				clean_username,
-				mainconfig.log_auth_badpass ? "/" : "",
-				mainconfig.log_auth_badpass ? clean_password : "",
+				request->root->log_auth_badpass ? "/" : "",
+				request->root->log_auth_badpass ? clean_password : "",
 				auth_name(buf, sizeof(buf), request, 1));
 	}
 
@@ -186,13 +145,13 @@ static int rad_authlog(const char *msg, REQUEST *request, int goodpass) {
  *
  *	NOTE: NOT the same as the RLM_ values !
  */
-int rad_check_password(REQUEST *request)
+static int rad_check_password(REQUEST *request)
 {
 	VALUE_PAIR *auth_type_pair;
 	VALUE_PAIR *cur_config_item;
 	VALUE_PAIR *password_pair;
 	VALUE_PAIR *auth_item;
-	char string[MAX_STRING_LEN];
+	uint8_t my_chap[MAX_STRING_LEN];
 	int auth_type = -1;
 	int result;
 	int auth_type_count = 0;
@@ -205,11 +164,11 @@ int rad_check_password(REQUEST *request)
 	 */
 	cur_config_item = request->config_items;
 	while(((auth_type_pair = pairfind(cur_config_item, PW_AUTH_TYPE))) != NULL) {
-		auth_type = auth_type_pair->lvalue;
+		auth_type = auth_type_pair->vp_integer;
 		auth_type_count++;
 
 		DEBUG2("  rad_check_password:  Found Auth-Type %s",
-				auth_type_pair->strvalue);
+				auth_type_pair->vp_strvalue);
 		cur_config_item = auth_type_pair->next;
 
 		if (auth_type == PW_AUTHTYPE_REJECT) {
@@ -220,7 +179,7 @@ int rad_check_password(REQUEST *request)
 
 	if (( auth_type_count > 1) && (debug_flag)) {
 		radlog(L_ERR, "Warning:  Found %d auth-types on request for user '%s'",
-			auth_type_count, request->username->strvalue);
+			auth_type_count, request->username->vp_strvalue);
 	}
 
 	/*
@@ -234,8 +193,37 @@ int rad_check_password(REQUEST *request)
 		return 0;
 	}
 
+	password_pair =  pairfind(request->config_items, PW_USER_PASSWORD);
+	if (password_pair &&
+	    pairfind(request->config_items, PW_CLEARTEXT_PASSWORD)) {
+		pairdelete(&request->config_items, PW_USER_PASSWORD);
+		password_pair = NULL;
+	}
+
+	if (password_pair) {
+		DICT_ATTR *da;
+
+		DEBUG("!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!");
+		DEBUG("!!!    Replacing User-Password in config items with Cleartext-Password.     !!!");
+		DEBUG("!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!");
+		DEBUG("!!! Please update your configuration so that the \"known good\"               !!!");
+		DEBUG("!!! clear text password is in Cleartext-Password, and not in User-Password. !!!");
+		DEBUG("!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!");
+		password_pair->attribute = PW_CLEARTEXT_PASSWORD;
+		da = dict_attrbyvalue(PW_CLEARTEXT_PASSWORD);
+		if (!da) {
+			radlog(L_ERR, "FATAL: You broke the dictionaries.  Please use the default dictionaries!");
+			_exit(1);
+		}
+
+		password_pair->name = da->name;
+	}
+
 	/*
-	 *	Find the password from the users file.
+	 *	Find the "known good" password.
+	 *
+	 *	FIXME: We should get rid of these hacks, and replace
+	 *	them with a module.
 	 */
 	if ((password_pair = pairfind(request->config_items, PW_CRYPT_PASSWORD)) != NULL) {
 		/*
@@ -244,20 +232,7 @@ int rad_check_password(REQUEST *request)
 		 */
 		if (auth_type == -1) auth_type = PW_AUTHTYPE_CRYPT;
 	} else {
-		password_pair = pairfind(request->config_items, PW_PASSWORD);
-		if (!password_pair) {
-			password_pair = pairfind(request->config_items,
-						 PW_CLEARTEXT_PASSWORD);
-			if (password_pair) {
-				VALUE_PAIR *old;
-
-				old = pairmake("User-Password",
-					       password_pair->strvalue,
-					       T_OP_SET);
-				if (old) pairadd(&request->config_items, old);
-			}
-		}
-		
+		password_pair = pairfind(request->config_items, PW_CLEARTEXT_PASSWORD);
 	}
 
 	if (auth_type < 0) {
@@ -298,8 +273,8 @@ int rad_check_password(REQUEST *request)
 				return -1;
 			}
 
-			switch (lrad_crypt_check((char *)auth_item->strvalue,
-									 (char *)password_pair->strvalue)) {
+			switch (fr_crypt_check((char *)auth_item->vp_strvalue,
+									 (char *)password_pair->vp_strvalue)) {
 			case -1:
 			  rad_authlog("Login incorrect "
 						  "(system failed to supply an encrypted password for comparison)", request, 0);
@@ -316,7 +291,10 @@ int rad_check_password(REQUEST *request)
 			 *	authentication fails.
 			 */
 			auth_item = request->password;
-			if (auth_item == NULL) {
+			if (!auth_item)
+				auth_item = pairfind(request->packet->vps,
+						     PW_CHAP_PASSWORD);
+			if (!auth_item) {
 				DEBUG2("auth: No User-Password or CHAP-Password attribute in the request");
 				return -1;
 			}
@@ -334,9 +312,9 @@ int rad_check_password(REQUEST *request)
 			/*
 			 *	Local password is just plain text.
 	 		 */
-			if (auth_item->attribute == PW_PASSWORD) {
-				if (strcmp((char *)password_pair->strvalue,
-					   (char *)auth_item->strvalue) != 0) {
+			if (auth_item->attribute == PW_USER_PASSWORD) {
+				if (strcmp((char *)password_pair->vp_strvalue,
+					   (char *)auth_item->vp_strvalue) != 0) {
 					DEBUG2("auth: user supplied User-Password does NOT match local User-Password");
 					return -1;
 				}
@@ -350,13 +328,13 @@ int rad_check_password(REQUEST *request)
 				return -1;
 			}
 
-			rad_chap_encode(request->packet, string,
-					auth_item->strvalue[0], password_pair);
+			rad_chap_encode(request->packet, my_chap,
+					auth_item->vp_octets[0], password_pair);
 
 			/*
 			 *	Compare them
 			 */
-			if (memcmp(string + 1, auth_item->strvalue + 1,
+			if (memcmp(my_chap + 1, auth_item->vp_strvalue + 1,
 				   CHAP_VALUE_LENGTH) != 0) {
 				DEBUG2("auth: user supplied CHAP-Password does NOT match local User-Password");
 				return -1;
@@ -424,8 +402,8 @@ int rad_postauth(REQUEST *request)
 	 */
 	vp = pairfind(request->config_items, PW_POST_AUTH_TYPE);
 	if (vp) {
-		DEBUG2("  Found Post-Auth-Type %s", vp->strvalue);
-		postauth_type = vp->lvalue;
+		DEBUG2("  Found Post-Auth-Type %s", vp->vp_strvalue);
+		postauth_type = vp->vp_integer;
 	}
 	result = module_post_auth(postauth_type, request);
 	switch (result) {
@@ -445,7 +423,6 @@ int rad_postauth(REQUEST *request)
 		 */
 		case RLM_MODULE_HANDLED:
 			/* FIXME */
-			result = RLM_MODULE_OK;
 			break;
 		/*
 		 *	The module had a number of OK return codes.
@@ -460,31 +437,6 @@ int rad_postauth(REQUEST *request)
 	return result;
 }
 
-/*
- *	Before sending an Access-Reject, call the modules in the
- *	Post-Auth-Type REJECT stanza.
- */
-static int rad_postauth_reject(REQUEST *request)
-{
-	int		result;
-	VALUE_PAIR	*tmp;
-	DICT_VALUE	*dval;
-
-	dval = dict_valbyname(PW_POST_AUTH_TYPE, "REJECT");
-	if (dval) {
-		/* Overwrite the Post-Auth-Type with the value REJECT */
-		pairdelete(&request->config_items, PW_POST_AUTH_TYPE);
-		tmp = paircreate(PW_POST_AUTH_TYPE, PW_TYPE_INTEGER);
-		tmp->lvalue = dval->value;
-		pairadd(&request->config_items, tmp);
-		result = rad_postauth(request);
-	} else {
-		/* No REJECT stanza */
-		result = RLM_MODULE_OK;
-	}
-	return result;
-}
-
 /*
  *	Process and reply to an authentication request
  *
@@ -495,18 +447,14 @@ int rad_authenticate(REQUEST *request)
 {
 	VALUE_PAIR	*namepair;
 	VALUE_PAIR	*check_item;
-	VALUE_PAIR	*reply_item;
 	VALUE_PAIR	*auth_item;
 	VALUE_PAIR	*module_msg;
 	VALUE_PAIR	*tmp = NULL;
-	int		result, r;
+	int		result;
 	char		umsg[MAX_STRING_LEN + 1];
 	const char	*user_msg = NULL;
 	const char	*password;
-	char		*exec_program;
-	int		exec_wait;
-	int		seen_callback_id;
-	char		buf[1024], logstr[1024];
+	char		logstr[1024];
 	char		autz_retry = 0;
 	int		autz_type = 0;
 
@@ -523,13 +471,10 @@ int rad_authenticate(REQUEST *request)
 		 *	accordingly.
 		 */
 		case PW_AUTHENTICATION_ACK:
-			tmp = paircreate(PW_AUTH_TYPE, PW_TYPE_INTEGER);
-			if (tmp == NULL) {
-				radlog(L_ERR|L_CONS, "Not enough memory");
-				exit(1);
-			}
-			tmp->lvalue = PW_AUTHTYPE_ACCEPT;
-			pairadd(&request->config_items, tmp);
+			tmp = radius_paircreate(request,
+						&request->config_items,
+						PW_AUTH_TYPE, PW_TYPE_INTEGER);
+			if (tmp) tmp->vp_integer = PW_AUTHTYPE_ACCEPT;
 			break;
 		/*
 		 *	Challenges are punted back to the NAS without any
@@ -550,7 +495,6 @@ int rad_authenticate(REQUEST *request)
 			rad_authlog("Login incorrect (Home Server says so)",
 				    request, 0);
 			request->reply->code = PW_AUTHENTICATION_REJECT;
-			rad_postauth_reject(request);
 			return RLM_MODULE_REJECT;
 		}
 	}
@@ -568,7 +512,7 @@ int rad_authenticate(REQUEST *request)
 	 */
 	if (!request->password) {
 		request->password = pairfind(request->packet->vps,
-					     PW_PASSWORD);
+					     PW_USER_PASSWORD);
 	}
 
 	/*
@@ -576,14 +520,14 @@ int rad_authenticate(REQUEST *request)
 	 */
 	auth_item = request->password;
 	if (auth_item) {
-		password = (const char *)auth_item->strvalue;
+		password = (const char *)auth_item->vp_strvalue;
 
 	} else {
 		/*
 		 *	Maybe there's a CHAP-Password?
 		 */
 		if ((auth_item = pairfind(request->packet->vps,
-				PW_CHAP_PASSWORD)) != NULL) {
+					  PW_CHAP_PASSWORD)) != NULL) {
 			password = "<CHAP-PASSWORD>";
 
 		} else {
@@ -606,9 +550,9 @@ autz_redo:
 		case RLM_MODULE_OK:
 		case RLM_MODULE_UPDATED:
 			break;
-		case RLM_MODULE_FAIL:
 		case RLM_MODULE_HANDLED:
 			return result;
+		case RLM_MODULE_FAIL:
 		case RLM_MODULE_INVALID:
 		case RLM_MODULE_REJECT:
 		case RLM_MODULE_USERLOCK:
@@ -617,7 +561,7 @@ autz_redo:
 					PW_MODULE_FAILURE_MESSAGE)) != NULL) {
 				char msg[MAX_STRING_LEN + 16];
 				snprintf(msg, sizeof(msg), "Invalid user (%s)",
-					 module_msg->strvalue);
+					 module_msg->vp_strvalue);
 				rad_authlog(msg,request,0);
 			} else {
 				rad_authlog("Invalid user", request, 0);
@@ -628,8 +572,8 @@ autz_redo:
 	if (!autz_retry) {
 		tmp = pairfind(request->config_items, PW_AUTZ_TYPE);
 		if (tmp) {
-			DEBUG2("  Found Autz-Type %s", tmp->strvalue);
-			autz_type = tmp->lvalue;
+			DEBUG2("  Found Autz-Type %s", tmp->vp_strvalue);
+			autz_type = tmp->vp_integer;
 			autz_retry = 1;
 			goto autz_redo;
 		}
@@ -645,19 +589,27 @@ autz_redo:
 	    ((tmp = pairfind(request->config_items, PW_PROXY_TO_REALM)) != NULL)) {
 		REALM *realm;
 
+		realm = realm_find2(tmp->vp_strvalue);
+
+		/*
+		 *	Don't authenticate, as the request is going to
+		 *	be proxied.
+		 */
+		if (realm && realm->auth_pool) {
+			return RLM_MODULE_OK;
+		}
+
 		/*
 		 *	Catch users who set Proxy-To-Realm to a LOCAL
-		 *	realm (sigh).
+		 *	realm (sigh).  But don't complain if it is
+		 *	*the* LOCAL realm.
 		 */
-		realm = realm_find(tmp->strvalue, 0);
-		if (realm && (realm->ipaddr == htonl(INADDR_NONE))) {
-			DEBUG2("  WARNING: You set Proxy-To-Realm = %s, but it is a LOCAL realm!  Cancelling invalid proxy request.", realm->realm);
-		} else {
-			/*
-			 *	Don't authenticate, as the request is
-			 *	proxied.
-			 */
-			return RLM_MODULE_OK;
+		if (realm &&(strcmp(realm->name, "LOCAL") != 0)) {
+			DEBUG2("  WARNING: You set Proxy-To-Realm = %s, but it is a LOCAL realm!  Cancelling invalid proxy request.", realm->name);
+		}
+
+		if (!realm) {
+			DEBUG2("  WARNING: You set Proxy-To-Realm = %s, but the realm does not exist!  Cancelling invalid proxy request.", tmp->vp_strvalue);
 		}
 	}
 
@@ -670,8 +622,6 @@ autz_redo:
 	 *	Validate the user
 	 */
 	do {
-		if ((result = check_expiration(request)) < 0)
-				break;
 		result = rad_check_password(request);
 		if (result > 0) {
 			/* don't reply! */
@@ -694,7 +644,7 @@ autz_redo:
 			char msg[MAX_STRING_LEN+19];
 
 			snprintf(msg, sizeof(msg), "Login incorrect (%s)",
-				 module_msg->strvalue);
+				 module_msg->vp_strvalue);
 			rad_authlog(msg, request, 0);
 		} else {
 			rad_authlog("Login incorrect", request, 0);
@@ -702,10 +652,10 @@ autz_redo:
 
 		/* double check: maybe the secret is wrong? */
 		if ((debug_flag > 1) && (auth_item != NULL) &&
-				(auth_item->attribute == PW_PASSWORD)) {
-			u_char *p;
+				(auth_item->attribute == PW_USER_PASSWORD)) {
+			char *p;
 
-			p = auth_item->strvalue;
+			p = auth_item->vp_strvalue;
 			while (*p != '\0') {
 				if (!isprint((int) *p)) {
 					log_debug("  WARNING: Unprintable characters in the password.\n\t  Double-check the shared secret on the server and the NAS!");
@@ -718,12 +668,12 @@ autz_redo:
 
 	if (result >= 0 &&
 	    (check_item = pairfind(request->config_items, PW_SIMULTANEOUS_USE)) != NULL) {
-		int session_type = 0;
+		int r, session_type = 0;
 
 		tmp = pairfind(request->config_items, PW_SESSION_TYPE);
 		if (tmp) {
-			DEBUG2("  Found Session-Type %s", tmp->strvalue);
-			session_type = tmp->lvalue;
+			DEBUG2("  Found Session-Type %s", tmp->vp_strvalue);
+			session_type = tmp->vp_integer;
 		}
 
 		/*
@@ -731,7 +681,7 @@ autz_redo:
 		 *	for the Simultaneous-Use parameter.
 		 */
 		if (namepair &&
-		    (r = module_checksimul(session_type, request, check_item->lvalue)) != 0) {
+		    (r = module_checksimul(session_type, request, check_item->vp_integer)) != 0) {
 			char mpp_ok = 0;
 
 			if (r == 2){
@@ -739,16 +689,16 @@ autz_redo:
 				VALUE_PAIR *port_limit;
 
 				if ((port_limit = pairfind(request->reply->vps, PW_PORT_LIMIT)) != NULL &&
-					port_limit->lvalue > check_item->lvalue){
+					port_limit->vp_integer > check_item->vp_integer){
 					DEBUG2("main auth: MPP is OK");
 					mpp_ok = 1;
 				}
 			}
 			if (!mpp_ok){
-				if (check_item->lvalue > 1) {
+				if (check_item->vp_integer > 1) {
 		  		snprintf(umsg, sizeof(umsg),
 							"\r\nYou are already logged in %d times  - access denied\r\n\n",
-							(int)check_item->lvalue);
+							(int)check_item->vp_integer);
 					user_msg = umsg;
 				} else {
 					user_msg = "\r\nYou are already logged in - access denied\r\n\n";
@@ -761,11 +711,12 @@ autz_redo:
 				 *	Remove ALL reply attributes.
 				 */
 				pairfree(&request->reply->vps);
-				tmp = pairmake("Reply-Message", user_msg, T_OP_SET);
-				request->reply->vps = tmp;
+				radius_pairmake(request, &request->reply->vps,
+						"Reply-Message",
+						user_msg, T_OP_SET);
 
 				snprintf(logstr, sizeof(logstr), "Multiple logins (max %d) %s",
-					check_item->lvalue,
+					check_item->vp_integer,
 					r == 2 ? "[MPP attempt]" : "");
 				rad_authlog(logstr, request, 1);
 
@@ -774,71 +725,11 @@ autz_redo:
 		}
 	}
 
-	if (result >= 0 &&
-	    (check_item = pairfind(request->config_items, PW_LOGIN_TIME)) != NULL) {
-
-		/*
-		 *	Authentication is OK. Now see if this
-		 *	user may login at this time of the day.
-		 */
-		r = timestr_match((char *)check_item->strvalue,
-				  request->timestamp);
-
-		if (r == 0) {	/* unlimited */
-			/*
-			 *	Do nothing: login-time is OK.
-			 */
-
-			/*
-			 *	Session-Timeout needs to be at least
-			 *	60 seconds, some terminal servers
-			 *	ignore smaller values.
-			 */
-		} else if (r < 60) {
-			/*
-			 *	User called outside allowed time interval.
-			 */
-			result = -1;
-			user_msg = "You are calling outside your allowed timespan\r\n";
-
-			request->reply->code = PW_AUTHENTICATION_REJECT;
-			pairfree(&request->reply->vps);
-
-			tmp = pairmake("Reply-Message", user_msg, T_OP_SET);
-			request->reply->vps = tmp;
-
-			snprintf(logstr, sizeof(logstr), "Outside allowed timespan (time allowed %s)",
-				 check_item->strvalue);
-			rad_authlog(logstr, request, 1);
-
-		} else if (r > 0) {
-
-			/*
-			 *	User is allowed, but set Session-Timeout.
-			 */
-			if ((reply_item = pairfind(request->reply->vps,
-					PW_SESSION_TIMEOUT)) != NULL) {
-				if (reply_item->lvalue > (unsigned) r)
-					reply_item->lvalue = r;
-			} else {
-				if ((reply_item = paircreate(
-						PW_SESSION_TIMEOUT,
-						PW_TYPE_INTEGER)) == NULL) {
-					radlog(L_ERR|L_CONS, "no memory");
-					exit(1);
-				}
-				reply_item->lvalue = r;
-				pairadd(&request->reply->vps, reply_item);
-			}
-		}
-	}
-
 	/*
 	 *	Result should be >= 0 here - if not, it means the user
 	 *	is rejected, so we just process post-auth and return.
 	 */
 	if (result < 0) {
-		rad_postauth_reject(request);
 		return RLM_MODULE_REJECT;
 	}
 
@@ -865,139 +756,16 @@ autz_redo:
 		 */
 		if ((vpPortId = pairfind(request->packet->vps,
 					 PW_NAS_PORT)) != NULL) {
-		  unsigned long tvalue = ntohl(tmp->lvalue);
-		  tmp->lvalue = htonl(tvalue + vpPortId->lvalue);
+		  unsigned long tvalue = ntohl(tmp->vp_integer);
+		  tmp->vp_integer = htonl(tvalue + vpPortId->vp_integer);
 		  tmp->flags.addport = 0;
-		  ip_ntoa(tmp->strvalue, tmp->lvalue);
+		  ip_ntoa(tmp->vp_strvalue, tmp->vp_integer);
 		} else {
 			DEBUG2("WARNING: No NAS-Port attribute in request.  CANNOT return a Framed-IP-Address + NAS-Port.\n");
 			pairdelete(&request->reply->vps, PW_FRAMED_IP_ADDRESS);
 		}
 	}
 
-	/*
-	 *	See if we need to execute a program.
-	 *	FIXME: somehow cache this info, and only execute the
-	 *	program when we receive an Accounting-START packet.
-	 *	Only at that time we know dynamic IP etc.
-	 */
-	exec_program = NULL;
-	exec_wait = 0;
-	if ((auth_item = pairfind(request->reply->vps, PW_EXEC_PROGRAM)) != NULL) {
-		exec_wait = 0;
-		exec_program = strdup((char *)auth_item->strvalue);
-		pairdelete(&request->reply->vps, PW_EXEC_PROGRAM);
-	}
-	if ((auth_item = pairfind(request->reply->vps, PW_EXEC_PROGRAM_WAIT)) != NULL) {
-		free(exec_program);
-		exec_wait = 1;
-		exec_program = strdup((char *)auth_item->strvalue);
-		pairdelete(&request->reply->vps, PW_EXEC_PROGRAM_WAIT);
-	}
-
-	/*
-	 *	Hack - allow % expansion in certain value strings.
-	 *	This is nice for certain Exec-Program programs.
-	 */
-	seen_callback_id = 0;
-	if ((auth_item = pairfind(request->reply->vps, PW_CALLBACK_ID)) != NULL) {
-		seen_callback_id = 1;
-		radius_xlat(buf, sizeof(auth_item->strvalue),
-			    (char *)auth_item->strvalue, request, NULL);
-		strNcpy((char *)auth_item->strvalue, buf,
-			sizeof(auth_item->strvalue));
-		auth_item->length = strlen((char *)auth_item->strvalue);
-	}
-
-
-	/*
-	 *	If we want to exec a program, but wait for it,
-	 *	do it first before sending the reply.
-	 */
-	if (exec_program && exec_wait) {
-		r = radius_exec_program(exec_program, request,
-					exec_wait,
-					NULL, 0,
-					request->packet->vps, &tmp);
-		free(exec_program);
-		exec_program = NULL;
-
-		/*
-		 *	Always add the value-pairs to the reply.
-		 */
-		pairmove(&request->reply->vps, &tmp);
-		pairfree(&tmp);
-
-		if (r < 0) {
-			/*
-			 *	Error. radius_exec_program() returns -1 on
-			 *	fork/exec errors.
-			 */
-			user_msg = "Access denied (external check failed)";
-			tmp = pairmake("Reply-Message", user_msg, T_OP_SET);
-			pairadd(&request->reply->vps, tmp);
-
-			request->reply->code = PW_AUTHENTICATION_REJECT;
-			rad_authlog("Login incorrect (external check failed)",
-				    request, 0);
-			rad_postauth_reject(request);
-
-			return RLM_MODULE_REJECT;
-		}
-		if (r > 0) {
-			/*
-			 *	Reject. radius_exec_program() returns >0
-			 *	if the exec'ed program had a non-zero
-			 *	exit status.
-			 */
-			request->reply->code = PW_AUTHENTICATION_REJECT;
-			rad_authlog("Login incorrect (external check said so)",
-				    request, 0);
-			rad_postauth_reject(request);
-
-			return RLM_MODULE_REJECT;
-		}
-	}
-
-	/*
-	 *	Delete "normal" A/V pairs when using callback.
-	 *
-	 *	FIXME: This is stupid. The portmaster should accept
-	 *	these settings instead of insisting on using a
-	 *	dialout location.
-	 *
-	 *	FIXME2: Move this into the above exec thingy?
-	 *	(if you knew how I use the exec_wait, you'd understand).
-	 */
-	if (seen_callback_id) {
-		pairdelete(&request->reply->vps, PW_FRAMED_PROTOCOL);
-		pairdelete(&request->reply->vps, PW_FRAMED_IP_ADDRESS);
-		pairdelete(&request->reply->vps, PW_FRAMED_IP_NETMASK);
-		pairdelete(&request->reply->vps, PW_FRAMED_ROUTE);
-		pairdelete(&request->reply->vps, PW_FRAMED_MTU);
-		pairdelete(&request->reply->vps, PW_FRAMED_COMPRESSION);
-		pairdelete(&request->reply->vps, PW_FILTER_ID);
-		pairdelete(&request->reply->vps, PW_PORT_LIMIT);
-		pairdelete(&request->reply->vps, PW_CALLBACK_NUMBER);
-	}
-
-	/*
-	 *	Filter (possibly multiple) Reply-Message attributes
-	 *	through radius_xlat, modifying them in place.
-	 */
-	if (user_msg == NULL) {
-		reply_item = pairfind(request->reply->vps, PW_REPLY_MESSAGE);
-		while (reply_item) {
-			radius_xlat(buf, sizeof(reply_item->strvalue),
-				    (char *)reply_item->strvalue, request, NULL);
-			strNcpy((char *)reply_item->strvalue, buf,
-				sizeof(reply_item->strvalue));
-			reply_item->length = strlen((char *)reply_item->strvalue);
-			user_msg = NULL;
-			reply_item = pairfind(reply_item->next, PW_REPLY_MESSAGE);
-		}
-	}
-
 	/*
 	 *	Set the reply to Access-Accept, if it hasn't already
 	 *	been set to something.  (i.e. Access-Challenge)
@@ -1009,23 +777,15 @@ autz_redo:
 		char msg[MAX_STRING_LEN+12];
 
 		snprintf(msg, sizeof(msg), "Login OK (%s)",
-			 module_msg->strvalue);
+			 module_msg->vp_strvalue);
 		rad_authlog(msg, request, 1);
 	} else {
 		rad_authlog("Login OK", request, 1);
 	}
 
-	if (exec_program && !exec_wait) {
-		/*
-		 *	No need to check the exit status here.
-		 */
-		radius_exec_program(exec_program, request, exec_wait,
-				    NULL, 0, request->packet->vps, NULL);
-	}
-
-	if (exec_program)
-		free(exec_program);
-
+	/*
+	 *	Run the modules in the 'post-auth' section.
+	 */
 	result = rad_postauth(request);
 
 	return result;
