@@ -1,7 +1,7 @@
 /*
  * smux.c	SNMP support
  *
- * Version:	$Id$
+ * Version:	$Id: smux.c,v 1.26 2007/12/02 16:37:16 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -15,46 +15,34 @@
  *
  *   You should have received a copy of the GNU General Public License
  *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
  *
- * Copyright 2000  The FreeRADIUS server project
+ * Copyright 2000,2006  The FreeRADIUS server project
  * Copyright 1999  Jochen Friedrich <jochen@scram.de>
  * Copyright 1999  Kunihiro Ishiguro <kunihiro@zebra.org>
  */
 
-static const char rcsid[] =
-"$Id$";
+#include <freeradius-devel/ident.h>
+RCSID("$Id: smux.c,v 1.26 2007/12/02 16:37:16 aland Exp $")
 
-#include "autoconf.h"
+#include <freeradius-devel/radiusd.h>
 
 #ifdef WITH_SNMP
 
-#include "libradius.h"
+#include <freeradius-devel/radius_snmp.h>
+#include <freeradius-devel/smux.h>
 
-#include <sys/socket.h>
 #include <sys/file.h>
 
-#ifdef HAVE_NETINET_IN_H
-#include <netinet/in.h>
-#endif
-
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
-#include <netdb.h>
 #include <fcntl.h>
 #include <ctype.h>
 
-#include "radiusd.h"
-#include "radius_snmp.h"
-#include "smux.h"
-
 #define min(A,B) ((A) < (B) ? (A) : (B))
 
 
 
 /* internal prototypes */
-static int oid_compare (oid *, int, oid *, int);
+static int oid_compare (oid *, size_t, oid *, size_t);
 
 /* SMUX subtree vector. */
 static struct list *treelist = NULL;
@@ -102,9 +90,9 @@ oid_copy_addr (oid my_oid[], struct in_addr *addr, int len)
 #endif /* NOT USED */
 
 static int
-oid_compare (oid *o1, int o1_len, oid *o2, int o2_len)
+oid_compare (oid *o1, size_t o1_len, oid *o2, size_t o2_len)
 {
-	int i;
+	size_t i;
 
 	for (i = 0; i < min (o1_len, o2_len); i++) {
 			if (o1[i] < o2[i])
@@ -121,9 +109,9 @@ oid_compare (oid *o1, int o1_len, oid *o2, int o2_len)
 }
 
 static int
-oid_compare_part (oid *o1, int o1_len, oid *o2, int o2_len)
+oid_compare_part (oid *o1, size_t o1_len, oid *o2, size_t o2_len)
 {
-	int i;
+	size_t i;
 
 	for (i = 0; i < min (o1_len, o2_len); i++) {
 			if (o1[i] < o2[i])
@@ -140,7 +128,7 @@ oid_compare_part (oid *o1, int o1_len, oid *o2, int o2_len)
 static void
 smux_oid_dump(const char *prefix, oid *my_oid, size_t oid_len)
 {
-	int i;
+	size_t i;
 	int first = 1;
 	char buf[MAX_OID_LEN * 3];
 
@@ -150,7 +138,7 @@ smux_oid_dump(const char *prefix, oid *my_oid, size_t oid_len)
 			sprintf (buf + strlen (buf), "%s%d", first ? "" : ".", (int) my_oid[i]);
 			first = 0;
 	}
-	DEBUG2 ("%s: %s", prefix, buf);
+	DEBUG2 ("SMUX %s: %s", prefix, buf);
 }
 
 static int
@@ -238,19 +226,19 @@ smux_sock (void)
 
 static void
 smux_getresp_send (oid objid[], size_t objid_len, long reqid, long errstat,
-		long errindex, u_char val_type, const void *arg, size_t arg_len)
+		long errindex, u_char val_type, void *arg, size_t arg_len)
 {
-	int ret;
+	size_t ret;
 	u_char buf[BUFSIZ];
 	u_char *ptr, *h1, *h1e, *h2, *h2e;
-	int len, length;
+	size_t len, length;
 
 	ptr = buf;
 	len = BUFSIZ;
 	length = len;
 
-	DEBUG2("SMUX GETRSP send");
-	DEBUG2("SMUX GETRSP reqid: %ld", reqid);
+	DEBUG3("SMUX GETRSP send");
+	DEBUG3("SMUX GETRSP reqid: %ld", reqid);
 
 	h1 = ptr;
 	/* Place holder h1 for complete sequence */
@@ -261,12 +249,12 @@ smux_getresp_send (oid objid[], size_t objid_len, long reqid, long errstat,
 			(u_char) (ASN_UNIVERSAL | ASN_PRIMITIVE | ASN_INTEGER),
 			&reqid, sizeof (reqid));
 
-	DEBUG2("SMUX GETRSP errstat: %ld", errstat);
+	DEBUG3("SMUX GETRSP errstat: %ld", errstat);
 
 	ptr = asn_build_int (ptr, &len,
 			(u_char) (ASN_UNIVERSAL | ASN_PRIMITIVE | ASN_INTEGER),
 			&errstat, sizeof (errstat));
-	DEBUG2("SMUX GETRSP errindex: %ld", errindex);
+	DEBUG3("SMUX GETRSP errindex: %ld", errindex);
 
 	ptr = asn_build_int (ptr, &len,
 			(u_char) (ASN_UNIVERSAL | ASN_PRIMITIVE | ASN_INTEGER),
@@ -280,7 +268,7 @@ smux_getresp_send (oid objid[], size_t objid_len, long reqid, long errstat,
 	h2e = ptr;
 
 	ptr = snmp_build_var_op (ptr, objid, &objid_len,
-			val_type, arg_len, arg, &len);
+				 val_type, arg_len, arg, &len);
 
 	/* Now variable size is known, fill in size */
 	asn_build_sequence(h2,&length,(u_char)(ASN_SEQUENCE|ASN_CONSTRUCTOR),ptr-h2e);
@@ -293,8 +281,8 @@ smux_getresp_send (oid objid[], size_t objid_len, long reqid, long errstat,
 	ret = send (rad_snmp.smux_fd, buf, (ptr - buf), 0);
 }
 
-static char *
-smux_var (char *ptr, int len, oid objid[], size_t *objid_len,
+static u_char *
+smux_var (u_char *ptr, size_t len, oid objid[], size_t *objid_len,
 		size_t *var_val_len,
 		u_char *var_val_type,
 		void **var_value)
@@ -304,13 +292,13 @@ smux_var (char *ptr, int len, oid objid[], size_t *objid_len,
 	size_t val_len;
 	u_char *val;
 
-	DEBUG2("SMUX var parse: len %d", len);
+	DEBUG3("SMUX var parse: len %d", len);
 
 	/* Parse header. */
 	ptr = asn_parse_header (ptr, &len, &type);
 
-	DEBUG2("SMUX var parse: type %d len %d", type, len);
-	DEBUG2("SMUX var parse: type must be %d", (ASN_SEQUENCE | ASN_CONSTRUCTOR));
+	DEBUG3("SMUX var parse: type %d len %d", type, len);
+	DEBUG3("SMUX var parse: type must be %d", (ASN_SEQUENCE | ASN_CONSTRUCTOR));
 
 	/* Parse var option. */
 	*objid_len = MAX_OID_LEN;
@@ -329,53 +317,53 @@ smux_var (char *ptr, int len, oid objid[], size_t *objid_len,
 	/* Requested object id length is objid_len. */
 	smux_oid_dump ("Request OID", objid, *objid_len);
 
-	DEBUG2 ("SMUX val_type: %d", val_type);
+	DEBUG3 ("SMUX val_type: %d", val_type);
 
 	/* Check request value type. */
 	switch (val_type) {
 		case ASN_NULL:
 			/* In case of SMUX_GET or SMUX_GET_NEXT val_type is set to
 				 ASN_NULL. */
-			DEBUG2 ("ASN_NULL");
+			DEBUG3 ("ASN_NULL");
 			break;
 
 		case ASN_INTEGER:
-			DEBUG2 ("ASN_INTEGER");
+			DEBUG3 ("ASN_INTEGER");
 			break;
 		case ASN_COUNTER:
 		case ASN_GAUGE:
 		case ASN_TIMETICKS:
 		case ASN_UINTEGER:
-			DEBUG2 ("ASN_COUNTER");
+			DEBUG3 ("ASN_COUNTER");
 			break;
 		case ASN_COUNTER64:
-			DEBUG2 ("ASN_COUNTER64");
+			DEBUG3 ("ASN_COUNTER64");
 			break;
 		case ASN_IPADDRESS:
-			DEBUG2 ("ASN_IPADDRESS");
+			DEBUG3 ("ASN_IPADDRESS");
 			break;
 		case ASN_OCTET_STR:
-			DEBUG2 ("ASN_OCTET_STR");
+			DEBUG3 ("ASN_OCTET_STR");
 			break;
 		case ASN_OPAQUE:
 		case ASN_NSAP:
 		case ASN_OBJECT_ID:
-			DEBUG2 ("ASN_OPAQUE");
+			DEBUG3 ("ASN_OPAQUE");
 			break;
 		case SNMP_NOSUCHOBJECT:
-			DEBUG2 ("SNMP_NOSUCHOBJECT");
+			DEBUG3 ("SNMP_NOSUCHOBJECT");
 			break;
 		case SNMP_NOSUCHINSTANCE:
-			DEBUG2 ("SNMP_NOSUCHINSTANCE");
+			DEBUG3 ("SNMP_NOSUCHINSTANCE");
 			break;
 		case SNMP_ENDOFMIBVIEW:
-			DEBUG2 ("SNMP_ENDOFMIBVIEW");
+			DEBUG3 ("SNMP_ENDOFMIBVIEW");
 			break;
 		case ASN_BIT_STR:
-			DEBUG2 ("ASN_BIT_STR");
+			DEBUG3 ("ASN_BIT_STR");
 			break;
 		default:
-			DEBUG2 ("Unknown type");
+			DEBUG3 ("Unknown type");
 			break;
 	}
 	return ptr;
@@ -395,7 +383,7 @@ smux_set (oid *reqid, size_t *reqid_len,
 	struct list *l;
 	int subresult;
 	oid *suffix;
-	int suffix_len;
+	size_t suffix_len;
 	int result;
 	const unsigned char *statP = NULL;
 	WriteMethod *write_method = NULL;
@@ -426,7 +414,7 @@ smux_set (oid *reqid, size_t *reqid_len,
 
 				/* This is exact match so result must be zero. */
 				if (result == 0) {
-					DEBUG2 ("SMUX function call index is %d", v->magic);
+					DEBUG3 ("SMUX function call index is %d", v->magic);
 
 					statP = (*v->findVar) (v, suffix, &suffix_len, 1,
 					&val_len, &write_method);
@@ -451,7 +439,7 @@ smux_set (oid *reqid, size_t *reqid_len,
 
 static int
 smux_get (oid *reqid, size_t *reqid_len, int exact,
-		u_char *val_type,const void **val, size_t *val_len)
+		u_char *val_type,u_char **val, size_t *val_len)
 {
 	int j;
 	struct subtree *subtree;
@@ -459,7 +447,7 @@ smux_get (oid *reqid, size_t *reqid_len, int exact,
 	struct list *l;
 	int subresult;
 	oid *suffix;
-	int suffix_len;
+	size_t suffix_len;
 	int result;
 	WriteMethod *write_method=NULL;
 
@@ -486,11 +474,10 @@ smux_get (oid *reqid, size_t *reqid_len, int exact,
 
 				/* This is exact match so result must be zero. */
 				if (result == 0) {
-					DEBUG2 ("SMUX function call index is %d", v->magic);
+					DEBUG3 ("SMUX function call index is %d", v->magic);
 
 					*val = (*v->findVar) (v, suffix, &suffix_len, exact,
-							val_len, &write_method);
-
+							      val_len, &write_method);
 					/* There is no instance. */
 					if (*val == NULL)
 						return SNMP_ERR_NOSUCHNAME;
@@ -513,7 +500,7 @@ smux_get (oid *reqid, size_t *reqid_len, int exact,
 
 static int
 smux_getnext (oid *reqid, size_t *reqid_len, int exact,
-		u_char *val_type, const void **val, size_t *val_len)
+		u_char *val_type, u_char **val, size_t *val_len)
 {
 	int j;
 	oid save[MAX_OID_LEN];
@@ -523,7 +510,7 @@ smux_getnext (oid *reqid, size_t *reqid_len, int exact,
 	struct list *l;
 	int subresult;
 	oid *suffix;
-	int suffix_len;
+	size_t suffix_len;
 	int result;
 	WriteMethod *write_method=NULL;
 
@@ -568,13 +555,13 @@ smux_getnext (oid *reqid, size_t *reqid_len, int exact,
 							v->name, v->namelen);
 
 				if (result <= 0) {
-					DEBUG2 ("SMUX function call index is %d", v->magic);
+					DEBUG3 ("SMUX function call index is %d", v->magic);
 					if(result<0) {
 						oid_copy(suffix, v->name, v->namelen);
 						suffix_len = v->namelen;
 					}
 					*val = (*v->findVar) (v, suffix, &suffix_len, exact,
-					val_len, &write_method);
+							      val_len, &write_method);
 					*reqid_len = suffix_len + subtree->name_len;
 					if (*val) {
 						*val_type = v->type;
@@ -591,8 +578,8 @@ smux_getnext (oid *reqid, size_t *reqid_len, int exact,
 }
 
 /* GET message header. */
-static char *
-smux_parse_get_header (char *ptr, size_t *len, long *reqid)
+static u_char *
+smux_parse_get_header (u_char *ptr, size_t *len, long *reqid)
 {
 	u_char type;
 	long errstat;
@@ -601,23 +588,23 @@ smux_parse_get_header (char *ptr, size_t *len, long *reqid)
 	/* Request ID. */
 	ptr = asn_parse_int (ptr, len, &type, reqid, sizeof (*reqid));
 
-	DEBUG2 ("SMUX GET reqid: %ld len: %d", *reqid, (int) *len);
+	DEBUG3 ("SMUX GET reqid: %ld len: %d", *reqid, (int) *len);
 
 	/* Error status. */
 	ptr = asn_parse_int (ptr, len, &type, &errstat, sizeof (errstat));
 
-	DEBUG2 ("SMUX GET errstat %ld len: %d", errstat, *len);
+	DEBUG3 ("SMUX GET errstat %ld len: %d", errstat, *len);
 
 	/* Error index. */
 	ptr = asn_parse_int (ptr, len, &type, &errindex, sizeof (errindex));
 
-	DEBUG2 ("SMUX GET errindex %ld len: %d", errindex, *len);
+	DEBUG3 ("SMUX GET errindex %ld len: %d", errindex, *len);
 
 	return ptr;
 }
 
 static void
-smux_parse_set (char *ptr, size_t len, int action)
+smux_parse_set (u_char *ptr, size_t len, int action)
 {
 	long reqid;
 	oid my_oid[MAX_OID_LEN];
@@ -627,7 +614,7 @@ smux_parse_set (char *ptr, size_t len, int action)
 	size_t val_len;
 	int ret;
 
-	DEBUG2 ("SMUX SET(%s) message parse: len %d",
+	DEBUG3 ("SMUX SET(%s) message parse: len %d",
 			(RESERVE1 == action) ? "RESERVE1" : ((FREE == action) ? "FREE" : "COMMIT"),
 			len);
 
@@ -646,17 +633,17 @@ smux_parse_set (char *ptr, size_t len, int action)
 }
 
 static void
-smux_parse_get (char *ptr, size_t len, int exact)
+smux_parse_get (u_char *ptr, size_t len, int exact)
 {
 	long reqid;
 	oid my_oid[MAX_OID_LEN];
 	size_t oid_len;
 	u_char val_type;
-	const void *val;
+	u_char *val;
 	size_t val_len;
 	int ret;
 
-	DEBUG2 ("SMUX GET message parse: len %d", len);
+	DEBUG3 ("SMUX GET message parse: len %d", len);
 
 	/* Parse GET message header. */
 	ptr = smux_parse_get_header (ptr, &len, &reqid);
@@ -679,7 +666,7 @@ smux_parse_get (char *ptr, size_t len, int exact)
 
 /* Parse SMUX_CLOSE message. */
 static void
-smux_parse_close (char *ptr, int len)
+smux_parse_close (u_char *ptr, size_t len)
 {
 	long reason = 0;
 
@@ -692,24 +679,24 @@ smux_parse_close (char *ptr, int len)
 
 /* SMUX_RRSP message. */
 static void
-smux_parse_rrsp (char *ptr, int len)
+smux_parse_rrsp (u_char *ptr, size_t len)
 {
-	char val;
+	u_char val;
 	long errstat;
 
 	ptr = asn_parse_int (ptr, &len, &val, &errstat, sizeof (errstat));
 
-	DEBUG2 ("SMUX_RRSP value: %d errstat: %ld", val, errstat);
+	DEBUG3 ("SMUX_RRSP value: %d errstat: %ld", val, errstat);
 }
 
 /* Parse SMUX message. */
 static int
-smux_parse (char *ptr, int len)
+smux_parse (u_char *ptr, size_t len)
 {
 	/* this buffer we'll use for SOUT message. We could allocate it with malloc and
 		 save only static pointer/lenght, but IMHO static buffer is a faster solusion */
 	static u_char sout_save_buff[SMUXMAXPKTSIZE];
-	static int sout_save_len = 0;
+	static size_t sout_save_len = 0;
 
 	int len_income = len; /* see note below: YYY */
 	u_char type;
@@ -768,35 +755,35 @@ process_rest: /* see note below: YYY */
 			break;
 		case SMUX_CLOSE:
 			/* Close SMUX connection. */
-			DEBUG2 ("SMUX_CLOSE");
+			DEBUG3 ("SMUX_CLOSE");
 			smux_parse_close (ptr, len);
 			return -1;
 			break;
 		case SMUX_RRSP:
 			/* This is response for register message. */
-			DEBUG2 ("SMUX_RRSP");
+			DEBUG3 ("SMUX_RRSP");
 			smux_parse_rrsp (ptr, len);
 			break;
 		case SMUX_GET:
 			/* Exact request for object id. */
-			DEBUG2 ("SMUX_GET");
+			DEBUG3 ("SMUX_GET");
 			smux_parse_get (ptr, len, 1);
 			break;
 		case SMUX_GETNEXT:
 			/* Next request for object id. */
-			DEBUG2 ("SMUX_GETNEXT");
+			DEBUG3 ("SMUX_GETNEXT");
 			smux_parse_get (ptr, len, 0);
 			break;
 		case SMUX_SET:
 			/* SMUX_SET is supported with some limitations. */
-			DEBUG2 ("SMUX_SET");
+			DEBUG3 ("SMUX_SET");
 			/* save the data for future SMUX_SOUT */
 			memcpy (sout_save_buff, ptr, len);
 			sout_save_len = len;
 			smux_parse_set (ptr, len, RESERVE1);
 			break;
 		default:
-			DEBUG ("Unknown type: %d", type);
+			DEBUG ("SMUX Unknown type: %d", type);
 			break;
 	}
 	return 0;
@@ -811,7 +798,7 @@ smux_read ()
 	int ret;
 
 	rad_snmp.smux_event=SMUX_NONE;
-	DEBUG2 ("SMUX read start");
+	DEBUG3 ("SMUX read start");
 
 	/* Read message from SMUX socket. */
 	len = recv (rad_snmp.smux_fd, buf, SMUXMAXPKTSIZE, 0);
@@ -832,7 +819,7 @@ smux_read ()
 		return -1;
 	}
 
-	DEBUG2 ("SMUX read len: %d", len);
+	DEBUG3 ("SMUX read len: %d", len);
 
 	/* Parse the message. */
 	ret = smux_parse (buf, len);
@@ -854,9 +841,9 @@ smux_open(void)
 {
 	u_char buf[BUFSIZ];
 	u_char *ptr;
-	int len;
-	u_long smux_proto_version;
-	u_char rad_progname[] = "radiusd";
+	size_t len;
+	long smux_proto_version;
+	char rad_progname[] = "radiusd";
 
 	smux_oid_dump ("SMUX open oid", smux_oid, smux_oid_len);
 	DEBUG2 ("SMUX open progname: %s", rad_progname);
@@ -871,26 +858,26 @@ smux_open(void)
 	/* SMUX Open. */
 	smux_proto_version = 0;
 	ptr = asn_build_int (ptr, &len,
-			(u_char)(ASN_UNIVERSAL | ASN_PRIMITIVE | ASN_INTEGER),
-			&smux_proto_version, sizeof (u_long));
+			     (u_char)(ASN_UNIVERSAL | ASN_PRIMITIVE | ASN_INTEGER),
+			     &smux_proto_version, sizeof (u_long));
 
 	/* SMUX connection oid. */
 	ptr = asn_build_objid (ptr, &len,
-			(u_char)
-			(ASN_UNIVERSAL | ASN_PRIMITIVE | ASN_OBJECT_ID),
-			smux_oid, smux_oid_len);
+			       (u_char)
+			       (ASN_UNIVERSAL | ASN_PRIMITIVE | ASN_OBJECT_ID),
+			       smux_oid, smux_oid_len);
 
 	/* SMUX connection description. */
 	ptr = asn_build_string (ptr, &len,
-			(u_char)
-			(ASN_UNIVERSAL | ASN_PRIMITIVE | ASN_OCTET_STR),
-			rad_progname, strlen (rad_progname));
+				(u_char)
+				(ASN_UNIVERSAL | ASN_PRIMITIVE | ASN_OCTET_STR),
+				(u_char *) rad_progname, strlen(rad_progname));
 
 	/* SMUX connection password. */
 	ptr = asn_build_string (ptr, &len,
-			(u_char)
-			(ASN_UNIVERSAL | ASN_PRIMITIVE | ASN_OCTET_STR),
-			rad_snmp.smux_password, strlen(rad_snmp.smux_password));
+				(u_char)
+				(ASN_UNIVERSAL | ASN_PRIMITIVE | ASN_OCTET_STR),
+				(const u_char *) rad_snmp.smux_password, strlen(rad_snmp.smux_password));
 
 	/* Fill in real SMUX header.	We exclude ASN header size (2). */
 	len = BUFSIZ;
@@ -904,7 +891,8 @@ smux_register(void)
 {
 	u_char buf[BUFSIZ];
 	u_char *ptr;
-	int len, ret;
+	size_t len;
+	int ret;
 	long priority;
 	long operation;
 	struct subtree *subtree;
@@ -946,9 +934,10 @@ smux_register(void)
 		len = BUFSIZ;
 		asn_build_header (buf, &len, (u_char) SMUX_RREQ, (ptr - buf) - 2);
 		ret = send (rad_snmp.smux_fd, buf, (ptr - buf), 0);
-		if (ret < 0)
+		if (ret < 0) {
 			return ret;
 		}
+	}
 	return ret;
 }
 
@@ -1093,7 +1082,7 @@ smux_init (oid defoid[], size_t defoid_len)
 
 /* Register subtree to smux master tree. */
 void
-smux_register_mib(const char *descr, struct variable *var, size_t width,
+smux_register_mib(UNUSED const char *descr, struct variable *var, size_t width,
 		int num, oid name[], size_t namelen)
 {
 	struct subtree *tree, *tt;
