@@ -1,7 +1,7 @@
 /*
  * eap.c    rfc2284 & rfc2869 implementation
  *
- * Version:     $Id$
+ * Version:     $Id: eap.c,v 1.75 2007/12/27 05:41:41 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -15,9 +15,9 @@
  *
  *   You should have received a copy of the GNU General Public License
  *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
  *
- * Copyright 2000-2003  The FreeRADIUS server project
+ * Copyright 2000-2003,2006  The FreeRADIUS server project
  * Copyright 2001  hereUare Communications, Inc. <raghud@hereuare.com>
  * Copyright 2003  Alan DeKok <aland@freeradius.org>
  */
@@ -54,9 +54,10 @@
  *
  */
 
-#include "rlm_eap.h"
+#include <freeradius-devel/ident.h>
+RCSID("$Id: eap.c,v 1.75 2007/12/27 05:41:41 aland Exp $")
 
-static const char rcsid[] = "$Id$";
+#include "rlm_eap.h"
 
 static const char *eap_codes[] = {
   "",				/* 0 is invalid */
@@ -119,6 +120,10 @@ int eaptype_load(EAP_TYPES **type, int eap_type, CONF_SECTION *cs)
 		free(node);
 		return -1;
 	}
+	cf_log_module(cs, "Linked to sub-module %s", buffer);
+
+	cf_log_module(cs, "Instantiating eap-%s", eaptype_name);
+
 	if ((node->type->attach) &&
 	    ((node->type->attach)(node->cs, &(node->type_data)) < 0)) {
 
@@ -129,7 +134,6 @@ int eaptype_load(EAP_TYPES **type, int eap_type, CONF_SECTION *cs)
 		return -1;
 	}
 
-	DEBUG("rlm_eap: Loaded and initialized type %s", eaptype_name);
 	*type = node;
 	return 0;
 }
@@ -186,6 +190,7 @@ static int eaptype_call(EAP_TYPES *atype, EAP_HANDLER *handler)
  */
 int eaptype_select(rlm_eap_t *inst, EAP_HANDLER *handler)
 {
+	size_t		i;
 	unsigned int	default_eap_type = inst->default_eap_type;
 	eaptype_t	*eaptype;
 	VALUE_PAIR	*vp;
@@ -203,6 +208,14 @@ int eaptype_select(rlm_eap_t *inst, EAP_HANDLER *handler)
 		return EAP_INVALID;
 	}
 
+	/*
+	 *	Multiple levels of nesting are invalid.
+	 */
+	if (handler->request->parent && handler->request->parent->parent) {
+		DEBUG2(" rlm_eap: Multiple levels of TLS nesting is invalid.");
+		return EAP_INVALID;
+	}
+
 	/*
 	 *	Figure out what to do.
 	 */
@@ -215,7 +228,7 @@ int eaptype_select(rlm_eap_t *inst, EAP_HANDLER *handler)
 		 */
 		vp = pairfind(handler->request->config_items,
 			      PW_EAP_TYPE);
-		if (vp) default_eap_type = vp->lvalue;
+		if (vp) default_eap_type = vp->vp_integer;
 
 	do_initiate:
 		/*
@@ -246,14 +259,9 @@ int eaptype_select(rlm_eap_t *inst, EAP_HANDLER *handler)
 			default_eap_type = PW_EAP_TLS;
 		}
 
-
-		/*
-		 *	We don't do TLS inside of TLS, as it's a bad
-		 *	idea...
-		 */
-		if (((handler->request->options & RAD_REQUEST_OPTION_FAKE_REQUEST) != 0) &&
-		    (default_eap_type == PW_EAP_TLS)) {
-			DEBUG2(" rlm_eap: Unable to tunnel TLS inside of TLS");
+		if ((default_eap_type == PW_EAP_TNC) &&
+		    !handler->request->parent) {
+			DEBUG2(" rlm_eap: ERROR: EAP-TNC must be run inside of a TLS method.");
 			return EAP_INVALID;
 		}
 
@@ -286,55 +294,82 @@ int eaptype_select(rlm_eap_t *inst, EAP_HANDLER *handler)
 			handler->opaque = NULL;
 		}
 
-		/*
-		 *	It is invalid to request identity,
-		 *	notification & nak in nak
-		 */
 		if (eaptype->data == NULL) {
 			DEBUG2(" rlm_eap: Empty NAK packet, cannot decide what EAP type the client wants.");
 			return EAP_INVALID;
 		}
 
 		/*
-		 *	FIXME: Pick one type out of the one they asked
-		 *	for, as they may have asked for many.
+		 *	Pick one type out of the one they asked for,
+		 *	as they may have asked for many.
 		 */
-		if ((eaptype->data[0] < PW_EAP_MD5) ||
-		    (eaptype->data[0] > PW_EAP_MAX_TYPES)) {
-			DEBUG2(" rlm_eap: NAK asked for bad type %d",
-			       eaptype->data[0]);
-			return EAP_INVALID;
+		default_eap_type = 0;
+		vp = pairfind(handler->request->config_items,
+			      PW_EAP_TYPE);
+		for (i = 0; i < eaptype->length; i++) {
+			/*
+			 *	It is invalid to request identity,
+			 *	notification & nak in nak.
+			 *
+			 *	Type 0 is valid, and means there are no
+			 *	common choices.
+			 */
+			if (eaptype->data[i] < PW_EAP_MD5) {
+				DEBUG2(" rlm_eap: NAK asked for bad type %d",
+				       eaptype->data[i]);
+				return EAP_INVALID;
+			}
+
+			if ((eaptype->data[i] > PW_EAP_MAX_TYPES) ||
+			    !inst->types[eaptype->data[i]]) {
+				DEBUG2(" rlm_eap: NAK asked for unsupported type %d",
+				       eaptype->data[i]);
+				continue;
+			}
+
+			eaptype_name = eaptype_type2name(eaptype->data[i],
+							 namebuf,
+							 sizeof(namebuf));
+			
+			/*
+			 *	Prevent a firestorm if the client is confused.
+			 */
+			if (handler->eap_type == eaptype->data[i]) {
+				DEBUG2(" rlm_eap: ERROR! Our request for %s was NAK'd with a request for %s.  Skipping the requested type.",
+				       eaptype_name, eaptype_name);
+				continue;
+			}
+
+			/*
+			 *	Enforce per-user configuration of EAP
+			 *	types.
+			 */
+			if (vp && (vp->vp_integer != eaptype->data[i])) {
+				char	mynamebuf[64];
+				DEBUG2("  rlm_eap: Client wants %s, while we require %s.  Skipping the requested type.",
+				       eaptype_name,
+				       eaptype_type2name(vp->vp_integer,
+							 mynamebuf,
+							 sizeof(mynamebuf)));
+				continue;
+			}
+
+			default_eap_type = eaptype->data[i];
+			break;
 		}
 
-		default_eap_type = eaptype->data[0];
+		/*
+		 *	We probably want to return 'fail' here...
+		 */
+		if (!default_eap_type) {
+			DEBUG2(" rlm_eap: No common EAP types found.");
+			return EAP_INVALID;
+		}
 		eaptype_name = eaptype_type2name(default_eap_type,
 						 namebuf, sizeof(namebuf));
 		DEBUG2(" rlm_eap: EAP-NAK asked for EAP-Type/%s",
 		       eaptype_name);
 
-		/*
-		 *	Prevent a firestorm if the client is confused.
-		 */
-		if (handler->eap_type == default_eap_type) {
-			DEBUG2(" rlm_eap: ERROR! Our request for %s was NAK'd with a request for %s, what is the client thinking?",
-			       eaptype_name, eaptype_name);
-			return EAP_INVALID;
-		}
-
-		/*
-		 *	Enforce per-user configuration of EAP types.
-		 */
-		vp = pairfind(handler->request->config_items,
-			      PW_EAP_TYPE);
-		if (vp && (vp->lvalue != default_eap_type)) {
-			char	mynamebuf[64];
-			DEBUG2("  rlm_eap: Client wants %s, while we require %s, rejecting the user.",
-			       eaptype_name,
-			       eaptype_type2name(vp->lvalue,
-						 mynamebuf,
-						 sizeof(mynamebuf)));
-			return EAP_INVALID;
-		}
 		goto do_initiate;
 		break;
 
@@ -371,64 +406,6 @@ int eaptype_select(rlm_eap_t *inst, EAP_HANDLER *handler)
 }
 
 
-/*
- *	EAP packet format to be sent over the wire
- *
- *	i.e. code+id+length+data where data = null/type+typedata
- *	based on code.
- */
-static int eap_wireformat(EAP_PACKET *reply)
-{
-	eap_packet_t	*hdr;
-	uint16_t total_length = 0;
-
-	if (reply == NULL) return EAP_INVALID;
-
-	total_length = EAP_HEADER_LEN;
-	if (reply->code < 3) {
-		total_length += 1/*EAPtype*/;
-		if (reply->type.data && reply->type.length > 0) {
-			total_length += reply->type.length;
-		}
-	}
-
-	reply->packet = (unsigned char *)malloc(total_length);
-	hdr = (eap_packet_t *)reply->packet;
-	if (!hdr) {
-		radlog(L_ERR, "rlm_eap: out of memory");
-		return EAP_INVALID;
-	}
-
-	hdr->code = (reply->code & 0xFF);
-	hdr->id = (reply->id & 0xFF);
-	total_length = htons(total_length);
-	memcpy(hdr->length, &total_length, sizeof(uint16_t));
-
-	/*
-	 *	Request and Response packets are special.
-	 */
-	if ((reply->code == PW_EAP_REQUEST) ||
-	    (reply->code == PW_EAP_RESPONSE)) {
-		hdr->data[0] = (reply->type.type & 0xFF);
-
-		/*
-		 * Here since we cannot know the typedata format and length
-		 *
-		 * Type_data is expected to be wired by each EAP-Type
-		 *
-		 * Zero length/No typedata is supported as long as
-		 * type is defined
-		 */
-		if (reply->type.data && reply->type.length > 0) {
-			memcpy(&hdr->data[1], reply->type.data, reply->type.length);
-			free(reply->type.data);
-			reply->type.data = reply->packet + EAP_HEADER_LEN + 1/*EAPtype*/;
-		}
-	}
-
-	return EAP_VALID;
-}
-
 /*
  *	compose EAP reply packet in EAP-Message attr of RADIUS.  If
  *	EAP exceeds 253, frame it in multiple EAP-Message attrs.
@@ -438,11 +415,8 @@ static int eap_wireformat(EAP_PACKET *reply)
  */
 int eap_compose(EAP_HANDLER *handler)
 {
-	uint16_t eap_len, len;
-	VALUE_PAIR *eap_msg;
 	VALUE_PAIR *vp;
 	eap_packet_t *eap_packet;
-	unsigned char 	*ptr;
 	REQUEST *request = handler->request;
 	EAP_DS *eap_ds = handler->eap_ds;
 	EAP_PACKET *reply = eap_ds->request;
@@ -515,36 +489,19 @@ int eap_compose(EAP_HANDLER *handler)
 		eap_ds->request->type.type = handler->eap_type;
 	}
 
-
+	/*
+	 *	FIXME: We malloc memory for the eap packet, and then
+	 *	immediately copy that data into VALUE_PAIRs.  This
+	 *	could be done more efficiently...
+	 */
 	if (eap_wireformat(reply) == EAP_INVALID) {
 		return RLM_MODULE_INVALID;
 	}
 	eap_packet = (eap_packet_t *)reply->packet;
 
-	memcpy(&eap_len, &(eap_packet->length), sizeof(uint16_t));
-	len = eap_len = ntohs(eap_len);
-	ptr = (unsigned char *)eap_packet;
-
-	do {
-		if (eap_len > 253) {
-			len = 253;
-			eap_len -= 253;
-		} else {
-			len = eap_len;
-			eap_len = 0;
-		}
-
-		/*
-		 * create a value pair & append it to the request reply list
-		 * This memory gets freed up when request is freed up
-		 */
-		eap_msg = paircreate(PW_EAP_MESSAGE, PW_TYPE_OCTETS);
-		memcpy(eap_msg->strvalue, ptr, len);
-		eap_msg->length = len;
-		pairadd(&(request->reply->vps), eap_msg);
-		ptr += len;
-		eap_msg = NULL;
-	} while (eap_len);
+	vp = eap_packet2vp(eap_packet);
+	if (!vp) return RLM_MODULE_INVALID;
+	pairadd(&(request->reply->vps), vp);
 
 	/*
 	 *	EAP-Message is always associated with
@@ -556,7 +513,7 @@ int eap_compose(EAP_HANDLER *handler)
 	vp = pairfind(request->reply->vps, PW_MESSAGE_AUTHENTICATOR);
 	if (!vp) {
 		vp = paircreate(PW_MESSAGE_AUTHENTICATOR, PW_TYPE_OCTETS);
-		memset(vp->strvalue, 0, AUTH_VECTOR_LEN);
+		memset(vp->vp_octets, 0, AUTH_VECTOR_LEN);
 		vp->length = AUTH_VECTOR_LEN;
 		pairadd(&(request->reply->vps), vp);
 	}
@@ -621,7 +578,7 @@ int eap_start(rlm_eap_t *inst, REQUEST *request)
 	 *	this allows you to NOT do EAP for some users.
 	 */
 	vp = pairfind(request->packet->vps, PW_EAP_TYPE);
-	if (vp && vp->lvalue == 0) {
+	if (vp && vp->vp_integer == 0) {
 		DEBUG2("  rlm_eap: Found EAP-Message, but EAP-Type = None, so we're not doing EAP.");
 		return EAP_NOOP;
 	}
@@ -644,8 +601,8 @@ int eap_start(rlm_eap_t *inst, REQUEST *request)
 		 *	If it's a LOCAL realm, then we're not proxying
 		 *	to it.
 		 */
-		realm = realm_find(proxy->strvalue, 0);
-		if (realm && (realm->ipaddr == htonl(INADDR_NONE))) {
+		realm = realm_find(proxy->vp_strvalue);
+		if (realm && (realm->auth_pool == NULL)) {
 			proxy = NULL;
 		}
 	}
@@ -670,16 +627,16 @@ int eap_start(rlm_eap_t *inst, REQUEST *request)
 		 */
 		if (proxy) {
 		do_proxy:
-			DEBUG2("  rlm_eap: Request is supposed to be proxied to Realm %s.  Not doing EAP.", proxy->strvalue);
+			DEBUG2("  rlm_eap: Request is supposed to be proxied to Realm %s.  Not doing EAP.", proxy->vp_strvalue);
 			return EAP_NOOP;
 		}
-		
+
 		DEBUG2("  rlm_eap: Got EAP_START message");
 		if ((eap_ds = eap_ds_alloc()) == NULL) {
 			DEBUG2("  rlm_eap: EAP Start failed in allocation");
 			return EAP_FAIL;
 		}
-		
+
 		/*
 		 *	It's an EAP-Start packet.  Tell them to stop wasting
 		 *	our time, and give us an EAP-Identity packet.
@@ -689,7 +646,7 @@ int eap_start(rlm_eap_t *inst, REQUEST *request)
 		 */
 		eap_ds->request->code = PW_EAP_REQUEST;
 		eap_ds->request->type.type = PW_EAP_IDENTITY;
-		
+
 		/*
 		 *	We don't have a handler, but eap_compose needs one,
 		 *	(for various reasons), so we fake it out here.
@@ -697,9 +654,9 @@ int eap_start(rlm_eap_t *inst, REQUEST *request)
 		memset(&handler, 0, sizeof(handler));
 		handler.request = request;
 		handler.eap_ds = eap_ds;
-		
+
 		eap_compose(&handler);
-		
+
 		eap_ds_free(&eap_ds);
 		return EAP_FOUND;
 	} /* end of handling EAP-Start */
@@ -722,7 +679,7 @@ int eap_start(rlm_eap_t *inst, REQUEST *request)
 	 */
 	vp = paircreate(PW_EAP_TYPE, PW_TYPE_INTEGER);
 	if (vp) {
-		vp->lvalue = eap_msg->strvalue[4];
+		vp->vp_integer = eap_msg->vp_octets[4];
 		pairadd(&(request->packet->vps), vp);
 	}
 
@@ -744,13 +701,13 @@ int eap_start(rlm_eap_t *inst, REQUEST *request)
 	 *	We're allowed only a few codes.  Request, Response,
 	 *	Success, or Failure.
 	 */
-	if ((eap_msg->strvalue[0] == 0) ||
-	    (eap_msg->strvalue[0] > PW_EAP_MAX_CODES)) {
+	if ((eap_msg->vp_octets[0] == 0) ||
+	    (eap_msg->vp_octets[0] > PW_EAP_MAX_CODES)) {
 		DEBUG2("  rlm_eap: Unknown EAP packet");
 	} else {
 		DEBUG2("  rlm_eap: EAP packet type %s id %d length %d",
-		       eap_codes[eap_msg->strvalue[0]],
-		       eap_msg->strvalue[1],
+		       eap_codes[eap_msg->vp_octets[0]],
+		       eap_msg->vp_octets[1],
 		       eap_msg->length);
 	}
 
@@ -760,8 +717,8 @@ int eap_start(rlm_eap_t *inst, REQUEST *request)
 	 *	sending success/fail packets to us, as it doesn't make
 	 *	sense.
 	 */
-	if ((eap_msg->strvalue[0] != PW_EAP_REQUEST) &&
-	    (eap_msg->strvalue[0] != PW_EAP_RESPONSE)) {
+	if ((eap_msg->vp_octets[0] != PW_EAP_REQUEST) &&
+	    (eap_msg->vp_octets[0] != PW_EAP_RESPONSE)) {
 		DEBUG2("  rlm_eap: Ignoring EAP packet which we don't know how to handle.");
 		return EAP_FAIL;
 	}
@@ -774,15 +731,15 @@ int eap_start(rlm_eap_t *inst, REQUEST *request)
 	 *	EAP-Identity, Notification, and NAK are all handled
 	 *	internally, so they never have handlers.
 	 */
-	if ((eap_msg->strvalue[4] >= PW_EAP_MD5) &&
+	if ((eap_msg->vp_octets[4] >= PW_EAP_MD5) &&
 	    inst->ignore_unknown_eap_types &&
-	    ((eap_msg->strvalue[4] == 0) ||
-	     (eap_msg->strvalue[4] > PW_EAP_MAX_TYPES) ||
-	     (inst->types[eap_msg->strvalue[4]] == NULL))) {
+	    ((eap_msg->vp_octets[4] == 0) ||
+	     (eap_msg->vp_octets[4] > PW_EAP_MAX_TYPES) ||
+	     (inst->types[eap_msg->vp_octets[4]] == NULL))) {
 		DEBUG2("  rlm_eap:  Ignoring Unknown EAP type");
 		return EAP_NOOP;
 	}
-		
+
 	/*
 	 *	They're NAKing the EAP type we wanted to use, and
 	 *	asking for one which we don't support.
@@ -798,16 +755,22 @@ int eap_start(rlm_eap_t *inst, REQUEST *request)
 	 *	returns NOOP, and another module may choose to proxy
 	 *	the request.
 	 */
-	if ((eap_msg->strvalue[4] == PW_EAP_NAK) &&
+	if ((eap_msg->vp_octets[4] == PW_EAP_NAK) &&
 	    (eap_msg->length >= (EAP_HEADER_LEN + 2)) &&
 	    inst->ignore_unknown_eap_types &&
-	    ((eap_msg->strvalue[5] == 0) ||
-	     (eap_msg->strvalue[5] > PW_EAP_MAX_TYPES) ||
-	     (inst->types[eap_msg->strvalue[5]] == NULL))) {
+	    ((eap_msg->vp_octets[5] == 0) ||
+	     (eap_msg->vp_octets[5] > PW_EAP_MAX_TYPES) ||
+	     (inst->types[eap_msg->vp_octets[5]] == NULL))) {
 		DEBUG2("  rlm_eap: Ignoring NAK with request for unknown EAP type");
 		return EAP_NOOP;
 	}
 
+	if ((eap_msg->vp_octets[4] == PW_EAP_TTLS) ||
+	    (eap_msg->vp_octets[4] == PW_EAP_PEAP)) {
+		DEBUG2("  rlm_eap: Continuing tunnel setup.");
+		return EAP_OK;
+	}
+
 	/*
 	 *	Later EAP messages are longer than the 'start'
 	 *	message, so if everything is OK, this function returns
@@ -1025,7 +988,7 @@ EAP_HANDLER *eap_handler(rlm_eap_t *inst, eap_packet_t **eap_packet_p,
                         *	request vps so that autz's work
                         *	correctly
 			*/
-                       radlog(L_INFO, "rlm_eap: Broken NAS did not set User-Name, setting from EAP Identity");
+                       DEBUG2("rlm_eap: Broken NAS did not set User-Name, setting from EAP Identity");
                        vp = pairmake("User-Name", handler->identity, T_OP_EQ);
                        if (vp == NULL) {
                                radlog(L_ERR, "rlm_eap: out of memory");
@@ -1046,7 +1009,7 @@ EAP_HANDLER *eap_handler(rlm_eap_t *inst, eap_packet_t **eap_packet_p,
                         *      request as the NAS is doing something
                         *      funny.
 			*/
-                       if (strncmp(handler->identity, vp->strvalue,
+                       if (strncmp(handler->identity, vp->vp_strvalue,
 				   MAX_STRING_LEN) != 0) {
                                radlog(L_ERR, "rlm_eap: Identity does not match User-Name.  Authentication failed.");
                                free(*eap_packet_p);
@@ -1084,7 +1047,7 @@ EAP_HANDLER *eap_handler(rlm_eap_t *inst, eap_packet_t **eap_packet_p,
                         *	request vps so that autz's work
                         *	correctly
 			*/
-                       radlog(L_INFO, "rlm_eap: WARNING NAS did not set User-Name.  Setting it locally from EAP Identity");
+                       DEBUG2("rlm_eap: WARNING NAS did not set User-Name.  Setting it locally from EAP Identity");
                        vp = pairmake("User-Name", handler->identity, T_OP_EQ);
                        if (vp == NULL) {
                                radlog(L_ERR, "rlm_eap: out of memory");
@@ -1102,7 +1065,7 @@ EAP_HANDLER *eap_handler(rlm_eap_t *inst, eap_packet_t **eap_packet_p,
                         *      identity, the NAS is doing something
                         *      funny, so reject the request.
 			*/
-                       if (strncmp(handler->identity, vp->strvalue,
+                       if (strncmp(handler->identity, vp->vp_strvalue,
 				   MAX_STRING_LEN) != 0) {
                                radlog(L_ERR, "rlm_eap: Identity does not match User-Name, setting from EAP Identity.");
                                free(*eap_packet_p);
