@@ -1,7 +1,7 @@
 /*
  * cb.c
  *
- * Version:     $Id$
+ * Version:     $Id: cb.c,v 1.8 2007/11/27 14:44:36 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -15,10 +15,15 @@
  *
  *   You should have received a copy of the GNU General Public License
  *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
  *
  * Copyright 2001  hereUare Communications, Inc. <raghud@hereuare.com>
+ * Copyright 2006  The FreeRADIUS server project
  */
+
+#include <freeradius-devel/ident.h>
+RCSID("$Id: cb.c,v 1.8 2007/11/27 14:44:36 aland Exp $")
+
 #include "eap_tls.h"
 
 #ifndef NO_OPENSSL
@@ -104,15 +109,4 @@ int cbtls_password(char *buf,
 	return(strlen((char *)userdata));
 }
 
-RSA *cbtls_rsa(SSL *s UNUSED, int is_export UNUSED, int keylength)
-{
-	static RSA *rsa_tmp=NULL;
-
-	if (rsa_tmp == NULL) {
-		DEBUG2("Generating temp (%d bit) RSA key...", keylength);
-		rsa_tmp=RSA_generate_key(keylength, RSA_F4, NULL, NULL);
-	}
-	return(rsa_tmp);
-}
-
 #endif /* !defined(NO_OPENSSL) */
