@@ -1,7 +1,7 @@
 /*
  * eap_tls.c
  *
- * Version:     $Id$
+ * Version:     $Id: eap_tls.c,v 1.10 2007/12/15 14:56:10 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -15,10 +15,11 @@
  *
  *   You should have received a copy of the GNU General Public License
  *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
  *
  * Copyright 2001  hereUare Communications, Inc. <raghud@hereuare.com>
  * Copyright 2003  Alan DeKok <aland@freeradius.org>
+ * Copyright 2006  The FreeRADIUS server project
  */
 
 /*
@@ -37,7 +38,10 @@
  *
  */
 
-#include "autoconf.h"
+#include <freeradius-devel/ident.h>
+RCSID("$Id: eap_tls.c,v 1.10 2007/12/15 14:56:10 aland Exp $")
+
+#include <freeradius-devel/autoconf.h>
 #include <assert.h>
 #include "eap_tls.h"
 
@@ -236,10 +240,11 @@ static eaptls_status_t eaptls_ack_handler(EAP_HANDLER *handler)
 		return EAPTLS_FAIL;
 
 	case handshake:
-		if (tls_session->info.handshake_type == finished) {
+		if ((tls_session->info.handshake_type == finished) &&
+		    (tls_session->dirty_out.used == 0)) {
 			DEBUG2("  rlm_eap_tls: ack handshake is finished");
 			return EAPTLS_SUCCESS;
-		}
+		} /* else more data to send */
 
 		DEBUG2("  rlm_eap_tls: ack handshake fragment handler");
 		/* Fragmentation handler, send next fragment */
@@ -332,8 +337,14 @@ static eaptls_status_t eaptls_verify(EAP_HANDLER *handler)
 	    ((eap_ds->response->length == EAP_HEADER_LEN + 2) &&
 	     ((eaptls_packet->flags & 0xc0) == 0x00))) {
 
-		if (prev_eap_ds &&
-		    (prev_eap_ds->request->id == eap_ds->response->id)) {
+#if 0
+		/*
+		 *	Un-comment this for TLS inside of TTLS/PEAP
+		 */
+		DEBUG2("rlm_eap_tls: Received EAP-TLS ACK message");
+		return eaptls_ack_handler(handler);
+#else
+		if (prev_eap_ds->request->id == eap_ds->response->id) {
 			/*
 			 *	Run the ACK handler directly from here.
 			 */
@@ -343,6 +354,7 @@ static eaptls_status_t eaptls_verify(EAP_HANDLER *handler)
 			radlog(L_ERR, "rlm_eap_tls: Received Invalid EAP-TLS ACK message");
 			return EAPTLS_INVALID;
 		}
+#endif
 	}
 
 	/*
@@ -367,6 +379,8 @@ static eaptls_status_t eaptls_verify(EAP_HANDLER *handler)
 	 *	from a fragment acknowledgement.
 	 */
 	if (TLS_LENGTH_INCLUDED(eaptls_packet->flags)) {
+		DEBUG2("  TLS Length %d",
+		       eaptls_packet->data[2] * 256 | eaptls_packet->data[3]);
 		if (TLS_MORE_FRAGMENTS(eaptls_packet->flags)) {
 			/*
 			 * FIRST_FRAGMENT is identified
@@ -387,7 +401,6 @@ static eaptls_status_t eaptls_verify(EAP_HANDLER *handler)
 				return EAPTLS_MORE_FRAGMENTS_WITH_LENGTH;
 			}
 		} else {
-
 			DEBUG2("rlm_eap_tls:  Length Included");
 			return EAPTLS_LENGTH_INCLUDED;
 		}
@@ -625,6 +638,8 @@ static void eaptls_operation(EAPTLS_PACKET *eaptls_packet UNUSED,
 		 */
 		eaptls_send_ack(handler->eap_ds, tls_session->peap_flag);
 	} else {
+		int rcode;
+
 		/*
 		 *	We have the complete TLS-data or TLS-message.
 		 *
@@ -634,16 +649,38 @@ static void eaptls_operation(EAPTLS_PACKET *eaptls_packet UNUSED,
 		 *	Success/Failure.
 		 *
 		 *	If more info
-		 *	is required then send another request.  */
-		if (tls_handshake_recv(tls_session)) {
+		 *	is required then send another request.
+		 */
+		rcode = tls_handshake_recv(tls_session);
+		if (rcode == 1) {
 			/*
 			 *	FIXME: return success/fail.
 			 *
 			 *	TLS proper can decide what to do, then.
 			 */
 			eaptls_request(handler->eap_ds, tls_session);
+
+			/*
+			 *	TLS returns 0 or 1.
+			 *	anything else is application-specific.
+			 *
+			 *	In our code, this means "session
+			 *	resumption was OK".
+			 */
+		} else if (rcode == 0xea) {
+			/*
+			 *	FIXME: hard-code key based on EAP type.
+			 *	Also, this code is duplicated all over
+			 *	the place...
+			 */
+			eaptls_success(handler->eap_ds, 0);
+			eaptls_gen_mppe_keys(&handler->request->reply->vps,
+					     tls_session->ssl,
+					     "client EAP encryption");
+
 		} else {
 			eaptls_fail(handler->eap_ds, tls_session->peap_flag);
+
 		}
 	}
 	return;
