@@ -1,7 +1,7 @@
 /*
  * eap_tls.h
  *
- * Version:     $Id$
+ * Version:     $Id: eap_tls.h,v 1.9 2007/11/25 14:02:08 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -15,14 +15,18 @@
  *
  *   You should have received a copy of the GNU General Public License
  *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
  *
  * Copyright 2001  hereUare Communications, Inc. <raghud@hereuare.com>
  * Copyright 2003  Alan DeKok <aland@freeradius.org>
+ * Copyright 2006  The FreeRADIUS server project
  */
 #ifndef _EAP_TLS_H
 #define _EAP_TLS_H
 
+#include <freeradius-devel/ident.h>
+RCSIDH(eap_tls_h, "$Id: eap_tls.h,v 1.9 2007/11/25 14:02:08 aland Exp $")
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -151,7 +155,7 @@ typedef struct _tls_session_t {
 				       unsigned int size);
 	unsigned int 	(*record_minus)(record_t *buf, void *ptr,
 					unsigned int size);
-	
+
 
 	/*
 	 * Framed-MTU attribute in RADIUS,
@@ -185,7 +189,7 @@ int 		eaptls_request(EAP_DS *eap_ds, tls_session_t *ssn);
 /* MPPE key generation */
 void            eaptls_gen_mppe_keys(VALUE_PAIR **reply_vps, SSL *s,
 				     const char *prf_label);
-void		eapttls_gen_challenge(SSL *s, char *buffer, int size);
+void		eapttls_gen_challenge(SSL *s, uint8_t *buffer, size_t size);
 
 #define BUFFER_SIZE 1024
 
@@ -349,7 +353,6 @@ int 		cbtls_password(char *buf, int num, int rwflag, void *userdata);
 void 		cbtls_info(const SSL *s, int where, int ret);
 void 		cbtls_msg(int write_p, int msg_version, int content_type,
 	       		const void *buf, size_t len, SSL *ssl, void *arg);
-RSA		*cbtls_rsa(SSL *s, int is_export, int keylength);
 
 /* TLS */
 tls_session_t 	*eaptls_new_session(SSL_CTX *ssl_ctx, int client_cert);
