@@ -1,7 +1,7 @@
 /*
  * mppe_keys.c
  *
- * Version:     $Id$
+ * Version:     $Id: mppe_keys.c,v 1.5 2007/11/25 14:02:08 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -15,12 +15,16 @@
  *
  *   You should have received a copy of the GNU General Public License
  *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
  *
  * Copyright 2002  Axis Communications AB
+ * Copyright 2006  The FreeRADIUS server project
  * Authors: Henrik Eriksson <henriken@axis.com> & Lars Viklund <larsv@axis.com>
  */
 
+#include <freeradius-devel/ident.h>
+RCSID("$Id: mppe_keys.c,v 1.5 2007/11/25 14:02:08 aland Exp $")
+
 #include <openssl/hmac.h>
 #include "eap_tls.h"
 
@@ -28,7 +32,7 @@
  * Add value pair to reply
  */
 static void add_reply(VALUE_PAIR** vp,
-		      const char* name, const char* value, int len)
+		      const char* name, const uint8_t * value, int len)
 {
 	VALUE_PAIR *reply_attr;
 	reply_attr = pairmake(name, "", T_OP_EQ);
@@ -39,7 +43,7 @@ static void add_reply(VALUE_PAIR** vp,
 		return;
 	}
 
-	memcpy(reply_attr->strvalue, value, len);
+	memcpy(reply_attr->vp_octets, value, len);
 	reply_attr->length = len;
 	pairadd(vp, reply_attr);
 }
@@ -158,11 +162,11 @@ void eaptls_gen_mppe_keys(VALUE_PAIR **reply_vps, SSL *s,
  *	It's in the TLS module simply because it's only a few lines
  *	of code, and it needs access to the TLS PRF functions.
  */
-void eapttls_gen_challenge(SSL *s, char *buffer, int size)
+void eapttls_gen_challenge(SSL *s, uint8_t *buffer, size_t size)
 {
-	unsigned char out[32], buf[32];
-	unsigned char seed[sizeof(EAPTLS_PRF_CHALLENGE)-1 + 2*SSL3_RANDOM_SIZE];
-	unsigned char *p = seed;
+	uint8_t out[32], buf[32];
+	uint8_t seed[sizeof(EAPTLS_PRF_CHALLENGE)-1 + 2*SSL3_RANDOM_SIZE];
+	uint8_t *p = seed;
 
 	memcpy(p, EAPTLS_PRF_CHALLENGE, sizeof(EAPTLS_PRF_CHALLENGE)-1);
 	p += sizeof(EAPTLS_PRF_CHALLENGE)-1;
