@@ -1,8 +1,28 @@
 #ifndef _EAP_MSCHAPV2_H
 #define _EAP_MSCHAPV2_H
 
+#include <freeradius-devel/ident.h>
+RCSIDH(eap_mschapv2_h, "$Id: eap_mschapv2.h,v 1.4 2006/11/14 21:22:10 fcusack Exp $")
+
 #include "eap.h"
 
+/*
+ *	draft-kamath-pppext-eap-mschapv2-00.txt says:
+ *
+ *	Supplicant		FreeRADIUS
+ *			<--	challenge
+ *	response	-->
+ *			<--	success
+ *	success		-->
+ *
+ *	But what we often see is:
+ *
+ *	Supplicant		FreeRADIUS
+ *			<--	challenge
+ *	response	-->
+ *			<--	success
+ *	ack		-->
+ */
 #define PW_EAP_MSCHAPV2_ACK		0
 #define PW_EAP_MSCHAPV2_CHALLENGE	1
 #define PW_EAP_MSCHAPV2_RESPONSE	2
