@@ -1,7 +1,7 @@
 /*
- * peap.c  contains the interfaces that are called from eap
+ * peap.c contains the interfaces that are called from eap
  *
- * Version:     $Id$
+ * Version:     $Id: peap.c,v 1.36 2008/03/07 09:53:26 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -15,10 +15,15 @@
  *
  *   You should have received a copy of the GNU General Public License
  *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
  *
  *   Copyright 2003 Alan DeKok <aland@freeradius.org>
+ *   Copyright 2006 The FreeRADIUS server project
  */
+
+#include <freeradius-devel/ident.h>
+RCSID("$Id: peap.c,v 1.36 2008/03/07 09:53:26 aland Exp $")
+
 #include "eap_peap.h"
 
 /*
@@ -50,7 +55,7 @@ static int eappeap_failure(EAP_HANDLER *handler, tls_session_t *tls_session)
 	 *	FIXME: Check the return code.
 	 */
 	tls_handshake_send(tls_session);
-	
+
 	return 1;
 }
 
@@ -96,7 +101,7 @@ static int eapmessage_verify(const uint8_t *data, unsigned int data_len)
 {
 	const eap_packet_t *eap_packet = (const eap_packet_t *) data;
 	uint8_t eap_type;
-	char identity[256];
+	char buffer[256];
 
 	if (!data || (data_len <= 1)) {
 		return 0;
@@ -105,9 +110,8 @@ static int eapmessage_verify(const uint8_t *data, unsigned int data_len)
 	eap_type = *data;
 	switch (eap_type) {
 	case PW_EAP_IDENTITY:
-		memcpy(identity, data + 1, data_len - 1);
-		identity[data_len - 1] = '\0';
-		DEBUG2("  rlm_eap_peap: Identity - %s", identity);
+		DEBUG2("  rlm_eap_peap: Identity - %*s",
+		       data_len - 1, data + 1);
 		return 1;
 		break;
 
@@ -133,7 +137,7 @@ static int eapmessage_verify(const uint8_t *data, unsigned int data_len)
 	default:
 		DEBUG2("  rlm_eap_peap: EAP type %s",
 		       eaptype_type2name(eap_type,
-					 identity, sizeof(identity)));
+					 buffer, sizeof(buffer)));
 		return 1;
 		break;
 	}
@@ -145,17 +149,12 @@ static int eapmessage_verify(const uint8_t *data, unsigned int data_len)
  *	Convert a pseudo-EAP packet to a list of VALUE_PAIR's.
  */
 static VALUE_PAIR *eap2vp(EAP_DS *eap_ds,
-			  const uint8_t *data, unsigned int data_len)
+			  const uint8_t *data, size_t data_len)
 {
-	VALUE_PAIR *vp = NULL;
+	size_t total;
+	VALUE_PAIR *vp = NULL, *head, **tail;
 
-	/*
-	 *	Sanity check this...
-	 */
-	if (data_len + EAP_HEADER_LEN > MAX_STRING_LEN) {
-		radlog(L_ERR, "rlm_eap_peap: EAP Response packet is too large.  Code must be fixed to handle this.");
-		return NULL;
-	}
+	if (data_len > 65535) return NULL; /* paranoia */
 
 	vp = paircreate(PW_EAP_MESSAGE, PW_TYPE_OCTETS);
 	if (!vp) {
@@ -163,18 +162,44 @@ static VALUE_PAIR *eap2vp(EAP_DS *eap_ds,
 		return NULL;
 	}
 
+	total = data_len;
+	if (total > 249) total = 249;
+
 	/*
 	 *	Hand-build an EAP packet from the crap in PEAP version 0.
 	 */
-	vp->strvalue[0] = PW_EAP_RESPONSE;
-	vp->strvalue[1] = eap_ds->response->id;
-	vp->strvalue[2] = 0;
-	vp->strvalue[3] = EAP_HEADER_LEN + data_len;
+	vp->vp_octets[0] = PW_EAP_RESPONSE;
+	vp->vp_octets[1] = eap_ds->response->id;
+	vp->vp_octets[2] = (data_len + EAP_HEADER_LEN) >> 8;
+	vp->vp_octets[3] = (data_len + EAP_HEADER_LEN) & 0xff;
 
-	memcpy(vp->strvalue + EAP_HEADER_LEN, data, data_len);
-	vp->length = EAP_HEADER_LEN + data_len;
+	memcpy(vp->vp_octets + EAP_HEADER_LEN, data, total);
+	vp->length = EAP_HEADER_LEN + total;
 
-	return vp;
+	head = vp;
+	tail = &(vp->next);
+	while (total < data_len) {
+		int vp_len;
+
+
+		vp = paircreate(PW_EAP_MESSAGE, PW_TYPE_OCTETS);
+		if (!vp) {
+			DEBUG2("  rlm_eap_peap: Failure in creating VP");
+			pairfree(&head);
+			return NULL;
+		}
+		vp_len = (data_len - total);
+		if (vp_len > 253) vp_len = 253;
+
+		memcpy(vp->vp_octets, data + total, vp_len);
+		vp->length = vp_len;
+		
+		total += vp_len;
+		*tail = vp;
+		tail = &(vp->next);
+	}
+
+	return head;
 }
 
 
@@ -184,40 +209,50 @@ static VALUE_PAIR *eap2vp(EAP_DS *eap_ds,
  */
 static int vp2eap(tls_session_t *tls_session, VALUE_PAIR *vp)
 {
-	if (vp->next != NULL) {
-		radlog(L_ERR, "rlm_eap_peap: EAP Request packet is too large.  Code must be fixed to handle this.");
-		return 0;
-	}
-
 	/*
 	 *	Skip the id, code, and length.  Just write the EAP
 	 *	type & data to the client.
 	 */
 #ifndef NDEBUG
-	if (debug_flag > 2) {
-		int i;
-		int total = vp->length - 4;
+	if ((debug_flag > 2) && fr_log_fp) {
+		size_t i, total;
+		VALUE_PAIR *this;
 
-		if (debug_flag > 0) for (i = 0; i < total; i++) {
-			if ((i & 0x0f) == 0) printf("  PEAP tunnel data out %04x: ", i);
+		total = 0;
 
-			printf("%02x ", vp->strvalue[i + 4]);
+		for (this = vp; this != NULL; this = this->next) {
+			int start = 0;
 
-			if ((i & 0x0f) == 0x0f) printf("\n");
+			if (this == vp) start = EAP_HEADER_LEN;
+			
+			for (i = start; i < vp->length; i++) {
+				if ((total & 0x0f) == 0) fprintf(fr_log_fp, "  PEAP tunnel data out %04x: ", total);
+
+				fprintf(fr_log_fp, "%02x ", vp->vp_octets[i]);
+				
+				if ((total & 0x0f) == 0x0f) fprintf(fr_log_fp, "\n");
+				total++;
+			}
 		}
-		if ((total & 0x0f) != 0) printf("\n");
+		if ((total & 0x0f) != 0) fprintf(fr_log_fp, "\n");
 	}
 #endif
 
 	/*
 	 *	Send the EAP data, WITHOUT the header.
 	 */
-#if 1
-	(tls_session->record_plus)(&tls_session->clean_in, vp->strvalue + EAP_HEADER_LEN,
-		vp->length - EAP_HEADER_LEN);
-#else
-	(tls_session->record_plus)(&tls_session->clean_in, vp->strvalue, vp->length);
-#endif
+	(tls_session->record_plus)(&tls_session->clean_in,
+				   vp->vp_octets + EAP_HEADER_LEN,
+				   vp->length - EAP_HEADER_LEN);
+	
+	/*
+	 *	Send the rest of the EAP data.
+	 */
+	for (vp = vp->next; vp != NULL; vp = vp->next) {
+		(tls_session->record_plus)(&tls_session->clean_in,
+					   vp->vp_octets, vp->length);
+	}
+
 	tls_handshake_send(tls_session);
 
 	return 1;
@@ -261,13 +296,11 @@ static int process_reply(EAP_HANDLER *handler, tls_session_t *tls_session,
 	peap_tunnel_t *t = tls_session->opaque;
 
 #ifndef NDEBUG
-	if (debug_flag > 0) {
-		printf("  PEAP: Processing from tunneled session code %p %d\n",
+	if ((debug_flag > 0) && fr_log_fp) {
+		fprintf(fr_log_fp, "  PEAP: Processing from tunneled session code %p %d\n",
 		       reply, reply->code);
-		
-		for (vp = reply->vps; vp != NULL; vp = vp->next) {
-			putchar('\t');vp_print(stdout, vp);putchar('\n');
-		}
+
+		debug_pair_list(reply->vps);
 	}
 #endif
 
@@ -341,7 +374,7 @@ static int process_reply(EAP_HANDLER *handler, tls_session_t *tls_session,
 			 */
 			pairdelete(&reply->vps, PW_PROXY_STATE);
 			pairdelete(&reply->vps, PW_MESSAGE_AUTHENTICATOR);
-			
+
 			t->accept_vps = reply->vps;
 			reply->vps = NULL;
 		}
@@ -386,12 +419,11 @@ static int eappeap_postproxy(EAP_HANDLER *handler, void *data)
 	fake = (REQUEST *) request_data_get(handler->request,
 					    handler->request->proxy,
 					    REQUEST_DATA_EAP_MSCHAP_TUNNEL_CALLBACK);
-	
+
 	/*
 	 *	Do the callback, if it exists, and if it was a success.
 	 */
 	if (fake && (handler->request->proxy_reply->code == PW_AUTHENTICATION_ACK)) {
-		VALUE_PAIR *vp;
 		REQUEST *request = handler->request;
 		peap_tunnel_t *t = tls_session->opaque;
 
@@ -402,6 +434,7 @@ static int eappeap_postproxy(EAP_HANDLER *handler, void *data)
 		 */
 		rad_assert(fake->packet == NULL);
 		fake->packet = request->proxy;
+		fake->packet->src_ipaddr = request->packet->src_ipaddr;
 		request->proxy = NULL;
 
 		rad_assert(fake->reply == NULL);
@@ -413,26 +446,21 @@ static int eappeap_postproxy(EAP_HANDLER *handler, void *data)
 		 *	handler, too...
 		 */
 		fake->options &= ~RAD_REQUEST_OPTION_PROXY_EAP;
-		DEBUG2("  PEAP: Passing reply back for EAP-MS-CHAP-V2 %p %d",
-		       fake, fake->reply->code);
+		DEBUG2("  PEAP: Passing reply back for EAP-MS-CHAP-V2");
 		rcode = module_post_proxy(0, fake);
 
 		/*
 		 *	FIXME: If rcode returns fail, do something
 		 *	intelligent...
 		 */
-		DEBUG2("  POST-PROXY %d", rcode);
 		rcode = rad_postauth(fake);
-		DEBUG2("  POST-AUTH %d", rcode);
 
 #ifndef NDEBUG
-		if (debug_flag > 0) {
-			printf("  PEAP: Final reply from tunneled session code %d\n",
+		if ((debug_flag > 0) && fr_log_fp) {
+			fprintf(fr_log_fp, "  PEAP: Final reply from tunneled session code %d\n",
 			       fake->reply->code);
-			
-			for (vp = fake->reply->vps; vp != NULL; vp = vp->next) {
-				putchar('\t');vp_print(stdout, vp);putchar('\n');
-			}
+
+			debug_pair_list(fake->reply->vps);
 		}
 #endif
 
@@ -454,12 +482,12 @@ static int eappeap_postproxy(EAP_HANDLER *handler, void *data)
 			eaptls_fail(handler->eap_ds, 0);
 			return 0;
 			break;
-			
+
                 default:  /* Don't Do Anything */
 			DEBUG2(" PEAP: Got reply %d",
 			       request->proxy_reply->code);
 			break;
-		}	
+		}
 	}
 	request_free(&fake);	/* robust if fake == NULL */
 
@@ -532,87 +560,53 @@ int eappeap_process(EAP_HANDLER *handler, tls_session_t *tls_session)
 	REQUEST *fake;
 	VALUE_PAIR *vp;
 	int rcode = RLM_MODULE_REJECT;
-	const uint8_t *data;
+	const uint8_t	*data;
 	unsigned int data_len;
-	unsigned char buffer[1024];
 #ifndef NDEBUG
-	int i;
+	size_t i;
 #endif
 
 	REQUEST *request = handler->request;
 	EAP_DS *eap_ds = handler->eap_ds;
 
 	/*
-	 *	Grab the dirty data, and copy it to our buffer.
-	 *
-	 *	I *really* don't like these 'record_t' things...
+	 *	FIXME: if the SSL session says "want read", or
+	 *	similar, leave the data in the clean_out buffer.  This
+	 *	lets the application data be sent across multiple
+	 *	fragments.
 	 */
-	data_len = (tls_session->record_minus)(&tls_session->dirty_in, buffer, sizeof(buffer));
-	data = buffer;
-
-	/*
-	 *	Write the data from the dirty buffer (i.e. packet
-	 *	data) into the buffer which we will give to SSL for
-	 *	decoding.
-	 *
-	 *	Some of this code COULD technically go into the TLS
-	 *	module, in eaptls_process(), where it returns EAPTLS_OK.
-	 *
-	 *	Similarly, the writing of data to the SSL context could
-	 *	go there, too...
-	 */
-	BIO_write(tls_session->into_ssl, buffer, data_len);
-	(tls_session->record_init)(&tls_session->clean_out);
-
-	/*
-	 *	Read (and decrypt) the tunneled data from the SSL session,
-	 *	and put it into the decrypted data buffer.
-	 */
-	err = SSL_read(tls_session->ssl, tls_session->clean_out.data,
-		       sizeof(tls_session->clean_out.data));
-	if (err < 0) {
-		/*
-		 *	FIXME: Call SSL_get_error() to see what went
-		 *	wrong.
-		 */
-		radlog(L_INFO, "rlm_eap_peap: SSL_read Error");
+	err = tls_handshake_recv(tls_session);
+	if (!err) {
+		DEBUG2(" rlm_eap_peap: Failed in SSL");
 		return RLM_MODULE_REJECT;
 	}
 
 	/*
-	 *	If there's no data, maybe this is an ACK to an
-	 *	MS-CHAP2-Success.
+	 *	Just look at the buffer directly, without doing
+	 *	record_minus.  This lets us avoid another data copy.
 	 */
-	if (err == 0) {
-		/*
-		 *	FIXME: Call SSL_get_error() to see what went
-		 *	wrong.
-		 */
-		radlog(L_INFO, "rlm_eap_peap: No data inside of the tunnel.");
-		return RLM_MODULE_REJECT;
-	}
-
-	data_len = tls_session->clean_out.used = err;
+	data_len = tls_session->clean_out.used;
+	tls_session->clean_out.used = 0;
 	data = tls_session->clean_out.data;
 
 #ifndef NDEBUG
-	if (debug_flag > 2) {
+	if ((debug_flag > 2) && fr_log_fp) {
 		for (i = 0; i < data_len; i++) {
-			if ((i & 0x0f) == 0) printf("  PEAP tunnel data in %04x: ", i);
+			if ((i & 0x0f) == 0) fprintf(fr_log_fp, "  PEAP tunnel data in %04x: ", i);
 
-			printf("%02x ", data[i]);
+			fprintf(fr_log_fp, "%02x ", data[i]);
 
-			if ((i & 0x0f) == 0x0f) printf("\n");
+			if ((i & 0x0f) == 0x0f) fprintf(fr_log_fp, "\n");
 		}
-		if ((data_len & 0x0f) != 0) printf("\n");
+		if ((data_len & 0x0f) != 0) fprintf(fr_log_fp, "\n");
 	}
 #endif
 
 	if (!eapmessage_verify(data, data_len)) {
+		DEBUG2("  rlm_eap_peap: Tunneled data is invalid.");
 		return RLM_MODULE_REJECT;
 	}
 
-	DEBUG2("  rlm_eap_peap: Tunneled data is valid.");
 
 	/*
 	 *	If we authenticated the user, then it's OK.
@@ -626,7 +620,7 @@ int eappeap_process(EAP_HANDLER *handler, tls_session_t *tls_session)
 		return RLM_MODULE_REJECT;
 
 	} else if (t->status == PEAP_STATUS_SENT_TLV_FAILURE) {
-		DEBUG2("  rlm_eap_peap:  Had sent TLV failure.  User was rejcted rejected earlier in this session.");
+		DEBUG2("  rlm_eap_peap:  Had sent TLV failure.  User was rejected earlier in this session.");
 		return RLM_MODULE_REJECT;
 	}
 
@@ -642,12 +636,10 @@ int eappeap_process(EAP_HANDLER *handler, tls_session_t *tls_session)
 	}
 
 #ifndef NDEBUG
-	if (debug_flag > 0) {
-	  printf("  PEAP: Got tunneled EAP-Message\n");
-
-	  for (vp = fake->packet->vps; vp != NULL; vp = vp->next) {
-	    putchar('\t');vp_print(stdout, vp);putchar('\n');
-	  }
+	if ((debug_flag > 0) && fr_log_fp) {
+		fprintf(fr_log_fp, "  PEAP: Got tunneled EAP-Message\n");
+		
+		debug_pair_list(fake->packet->vps);
 	}
 #endif
 
@@ -672,10 +664,10 @@ int eappeap_process(EAP_HANDLER *handler, tls_session_t *tls_session)
 			t->username = pairmake("User-Name", "", T_OP_EQ);
 			rad_assert(t->username != NULL);
 
-			memcpy(t->username->strvalue, data + 1, data_len - 1);
+			memcpy(t->username->vp_strvalue, data + 1, data_len - 1);
 			t->username->length = data_len - 1;
-			t->username->strvalue[t->username->length] = 0;
-			DEBUG2("  PEAP: Got tunneled identity of %s", t->username->strvalue);
+			t->username->vp_strvalue[t->username->length] = 0;
+			DEBUG2("  PEAP: Got tunneled identity of %s", t->username->vp_strvalue);
 
 			/*
 			 *	If there's a default EAP type,
@@ -684,7 +676,7 @@ int eappeap_process(EAP_HANDLER *handler, tls_session_t *tls_session)
 			if (t->default_eap_type != 0) {
 				DEBUG2("  PEAP: Setting default EAP type for tunneled EAP session.");
 				vp = pairmake("EAP-Type", "0", T_OP_EQ);
-				vp->lvalue = t->default_eap_type;
+				vp->vp_integer = t->default_eap_type;
 				pairadd(&fake->config_items, vp);
 			}
 		}
@@ -695,15 +687,13 @@ int eappeap_process(EAP_HANDLER *handler, tls_session_t *tls_session)
 		pairadd(&fake->packet->vps, vp);
 		fake->username = pairfind(fake->packet->vps, PW_USER_NAME);
 		DEBUG2("  PEAP: Setting User-Name to %s",
-		       fake->username->strvalue);
+		       fake->username->vp_strvalue);
 	}
 
 	/*
 	 *	Add the State attribute, too, if it exists.
 	 */
 	if (t->state) {
-		DEBUG2("  PEAP: Adding old state with %02x %02x",
-		       t->state->strvalue[0], t->state->strvalue[1]);
 		vp = paircopy(t->state);
 		if (vp) pairadd(&fake->packet->vps, vp);
 	}
@@ -779,13 +769,21 @@ int eappeap_process(EAP_HANDLER *handler, tls_session_t *tls_session)
 		}
 	}
 
-#ifndef NDEBUG
-	if (debug_flag > 0) {
-		printf("  PEAP: Sending tunneled request\n");
+	if ((vp = pairfind(request->config_items, PW_VIRTUAL_SERVER)) != NULL) {
+		fake->server = vp->vp_strvalue;
 
-		for (vp = fake->packet->vps; vp != NULL; vp = vp->next) {
-			putchar('\t');vp_print(stdout, vp);putchar('\n');
-		}
+	} else if (t->virtual_server) {
+		fake->server = t->virtual_server;
+
+	} /* else fake->server == request->server */
+
+#ifndef NDEBUG
+	if ((debug_flag > 0) && fr_log_fp) {
+		fprintf(fr_log_fp, "  PEAP: Sending tunneled request\n");
+
+		debug_pair_list(fake->packet->vps);
+
+		fprintf(fr_log_fp, "server %s {\n", fake->server);
 	}
 #endif
 
@@ -800,13 +798,13 @@ int eappeap_process(EAP_HANDLER *handler, tls_session_t *tls_session)
 	 *	attributes.
 	 */
 #ifndef NDEBUG
-	if (debug_flag > 0) {
-		printf("  PEAP: Got tunneled reply RADIUS code %d\n",
-		 fake->reply->code);
+	if ((debug_flag > 0) && fr_log_fp) {
+		fprintf(fr_log_fp, "} # server %s\n", fake->server);
 
-		for (vp = fake->reply->vps; vp != NULL; vp = vp->next) {
-			putchar('\t');vp_print(stdout, vp);putchar('\n');
-		}
+		fprintf(fr_log_fp, "  PEAP: Got tunneled reply RADIUS code %d\n",
+			fake->reply->code);
+		
+		debug_pair_list(fake->reply->vps);
 	}
 #endif
 
@@ -868,7 +866,7 @@ int eappeap_process(EAP_HANDLER *handler, tls_session_t *tls_session)
 				 *	done.  Handle it like normal.
 				 */
 				if ((fake->options & RAD_REQUEST_OPTION_PROXY_EAP) == 0) {
-					DEBUG2("    PEAP: Cancelling proxy to realm %s until the tunneled EAP session has been established", vp->strvalue);
+					DEBUG2("    PEAP: Cancelling proxy to realm %s until the tunneled EAP session has been established", vp->vp_strvalue);
 					goto do_process;
 				}
 
@@ -881,7 +879,7 @@ int eappeap_process(EAP_HANDLER *handler, tls_session_t *tls_session)
 					   PW_EAP_MESSAGE);
 			}
 
-			DEBUG2("  PEAP: Tunneled authentication will be proxied to %s", vp->strvalue);
+			DEBUG2("  PEAP: Tunneled authentication will be proxied to %s", vp->vp_strvalue);
 
 			/*
 			 *	Tell the original request that it's going
@@ -938,7 +936,7 @@ int eappeap_process(EAP_HANDLER *handler, tls_session_t *tls_session)
 							 REQUEST_DATA_EAP_MSCHAP_TUNNEL_CALLBACK,
 							 fake, my_request_free);
 				rad_assert(rcode == 0);
-				
+
 				/*
 				 *	Do NOT free the fake request!
 				 */
