@@ -1,7 +1,7 @@
 /*
  * rlm_eap_ttls.c  contains the interfaces that are called from eap
  *
- * Version:     $Id$
+ * Version:     $Id: rlm_eap_ttls.c,v 1.16 2007/11/27 16:01:53 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -15,12 +15,16 @@
  *
  *   You should have received a copy of the GNU General Public License
  *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
  *
  * Copyright 2003 Alan DeKok <aland@freeradius.org>
+ * Copyright 2006 The FreeRADIUS server project
  */
 
-#include "autoconf.h"
+#include <freeradius-devel/ident.h>
+RCSID("$Id: rlm_eap_ttls.c,v 1.16 2007/11/27 16:01:53 aland Exp $")
+
+#include <freeradius-devel/autoconf.h>
 #include "eap_ttls.h"
 
 
@@ -42,6 +46,11 @@ typedef struct rlm_eap_ttls_t {
 	 *	tunneled session in the tunneled request
 	 */
 	int	copy_request_to_tunnel;
+
+	/*
+	 *	Virtual server for inner tunnel session.
+	 */
+	char	*virtual_server;
 } rlm_eap_ttls_t;
 
 
@@ -55,6 +64,9 @@ static CONF_PARSER module_config[] = {
 	{ "use_tunneled_reply", PW_TYPE_BOOLEAN,
 	  offsetof(rlm_eap_ttls_t, use_tunneled_reply), NULL, "no" },
 
+	{ "virtual_server", PW_TYPE_STRING_PTR,
+	  offsetof(rlm_eap_ttls_t, virtual_server), NULL, NULL },
+
  	{ NULL, -1, 0, NULL, NULL }           /* end the list */
 };
 
@@ -65,7 +77,6 @@ static int eapttls_detach(void *arg)
 {
 	rlm_eap_ttls_t *inst = (rlm_eap_ttls_t *) arg;
 
-	if (inst->default_eap_type_name) free(inst->default_eap_type_name);
 
 	free(inst);
 
@@ -106,21 +117,6 @@ static int eapttls_attach(CONF_SECTION *cs, void **instance)
 		return -1;
 	}
 
-	/*
-	 *	Can't tunnel TLS inside of TLS, we don't like it.
-	 *
-	 *	More realistically, we haven't tested it, so we don't
-	 *	claim it works.
-	 */
-	if ((inst->default_eap_type == PW_EAP_TLS) ||
-	    (inst->default_eap_type == PW_EAP_TTLS) ||
-	    (inst->default_eap_type == PW_EAP_PEAP)) {
-		radlog(L_ERR, "rlm_eap_ttls: Cannot tunnel EAP-Type/%s inside of TTLS",
-		       inst->default_eap_type_name);
-		eapttls_detach(inst);
-		return -1;
-	}
-
 	*instance = inst;
 	return 0;
 }
@@ -137,7 +133,7 @@ static void ttls_free(void *p)
 
 	if (t->username) {
 		DEBUG2("  TTLS: Freeing handler for user %s",
-		       t->username->strvalue);
+		       t->username->vp_strvalue);
 	}
 
 	pairfree(&t->username);
@@ -160,6 +156,7 @@ static ttls_tunnel_t *ttls_alloc(rlm_eap_ttls_t *inst)
 	t->default_eap_type = inst->default_eap_type;
 	t->copy_request_to_tunnel = inst->copy_request_to_tunnel;
 	t->use_tunneled_reply = inst->use_tunneled_reply;
+	t->virtual_server = inst->virtual_server;
 	return t;
 }
 
@@ -191,7 +188,7 @@ static int eapttls_authenticate(void *arg, EAP_HANDLER *handler)
 		 *	an EAP-TLS-Success packet here.
 		 */
 	case EAPTLS_SUCCESS:
-		if (t->authenticated) {
+		if (t && t->authenticated) {
 			if (t->reply) {
 				pairmove(&handler->request->reply->vps,
 					 &t->reply);
