@@ -1,7 +1,7 @@
 /*
  * other.c
  *
- * Version:	$Id$
+ * Version:	$Id: other.c,v 1.5 2006/11/14 21:22:16 fcusack Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -15,12 +15,15 @@
  *
  *   You should have received a copy of the GNU General Public License
  *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
  *
- * Copyright 2000  The FreeRADIUS server project
+ * Copyright 2000,2006  The FreeRADIUS server project
  * Copyright 2000  your name <your address>
  */
 
+#include <freeradius-devel/ident.h>
+RCSID("$Id: other.c,v 1.5 2006/11/14 21:22:16 fcusack Exp $")
+
 #include <stdio.h>
 
 #include "other.h"
