@@ -1,7 +1,7 @@
 /*
  * rlm_mschap.c
  *
- * Version:	$Id$
+ * Version:	$Id: rlm_mschap.c,v 1.99 2008/01/09 13:20:56 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -15,9 +15,9 @@
  *
  *   You should have received a copy of the GNU General Public License
  *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
  *
- * Copyright 2000,2001  The FreeRADIUS server project
+ * Copyright 2000,2001,2006  The FreeRADIUS server project
  */
 
 
@@ -42,58 +42,21 @@
 
 /*  MPPE support from Takahiro Wagatsuma <waga@sic.shibaura-it.ac.jp> */
 
-#include	"autoconf.h"
-#include	"libradius.h"
+#include	<freeradius-devel/ident.h>
+RCSID("$Id: rlm_mschap.c,v 1.99 2008/01/09 13:20:56 aland Exp $")
+
+#include	<freeradius-devel/radiusd.h>
+#include	<freeradius-devel/modules.h>
+#include	<freeradius-devel/rad_assert.h>
+#include        <freeradius-devel/md5.h>
 
-#include	<stdio.h>
-#include	<stdlib.h>
-#include    	<string.h>
 #include 	<ctype.h>
 
-#include	"radiusd.h"
-#include	"modules.h"
-
-#include        "md4.h"
-#include        "md5.h"
-#include	"sha1.h"
-#include	"rad_assert.h"
-
 #include	"smbdes.h"
 
-static const char rcsid[] = "$Id$";
-
-static const char *letters = "0123456789ABCDEF";
-
-/*
- *	hex2bin converts hexadecimal strings into binary
- */
-static int hex2bin (const char *szHex, unsigned char* szBin, int len)
-{
-	char * c1, * c2;
-	int i;
-
-   	for (i = 0; i < len; i++) {
-		if( !(c1 = memchr(letters, toupper((int) szHex[i << 1]), 16)) ||
-		    !(c2 = memchr(letters, toupper((int) szHex[(i << 1) + 1]), 16)))
-		     break;
-                 szBin[i] = ((c1-letters)<<4) + (c2-letters);
-        }
-        return i;
-}
-
-/*
- *	bin2hex creates hexadecimal presentation
- *	of binary data
- */
-static void bin2hex (const unsigned char *szBin, char *szHex, int len)
-{
-	int i;
-	for (i = 0; i < len; i++) {
-		szHex[i<<1] = letters[szBin[i] >> 4];
-		szHex[(i<<1) + 1] = letters[szBin[i] & 0x0F];
-	}
-}
-
+#ifdef __APPLE__
+extern int od_mschap_auth(REQUEST *request, VALUE_PAIR *challenge, VALUE_PAIR * usernamepair);
+#endif
 
 /* Allowable account control bits */
 #define ACB_DISABLED   0x0001  /* 1 = User account disabled */
@@ -187,7 +150,7 @@ static int pdb_decode_acct_ctrl(const char *p)
  *	ntpwdhash converts Unicode password to 16-byte NT hash
  *	with MD4
  */
-static void ntpwdhash (unsigned char *szHash, const char *szPassword)
+static void ntpwdhash (uint8_t *szHash, const char *szPassword)
 {
 	char szUnicodePass[513];
 	int nPasswordLen;
@@ -204,7 +167,7 @@ static void ntpwdhash (unsigned char *szHash, const char *szPassword)
 	}
 
 	/* Encrypt Unicode password to a 16-byte MD4 hash */
-	md4_calc(szHash, szUnicodePass, (nPasswordLen<<1) );
+	fr_md4_calc(szHash, (uint8_t *) szUnicodePass, (nPasswordLen<<1) );
 }
 
 
@@ -213,18 +176,19 @@ static void ntpwdhash (unsigned char *szHash, const char *szPassword)
  *	implements RFC2759 ChallengeHash()
  *	generates 64 bit challenge
  */
-static void challenge_hash( const char *peer_challenge,
-			    const char *auth_challenge,
-			    const char *user_name, char *challenge )
+static void challenge_hash( const uint8_t *peer_challenge,
+			    const uint8_t *auth_challenge,
+			    const char *user_name, uint8_t *challenge )
 {
-	SHA1_CTX Context;
-	unsigned char hash[20];
+	fr_SHA1_CTX Context;
+	uint8_t hash[20];
 
-	SHA1Init(&Context);
-	SHA1Update(&Context, peer_challenge, 16);
-	SHA1Update(&Context, auth_challenge, 16);
-	SHA1Update(&Context, user_name, strlen(user_name));
-	SHA1Final(hash, &Context);
+	fr_SHA1Init(&Context);
+	fr_SHA1Update(&Context, peer_challenge, 16);
+	fr_SHA1Update(&Context, auth_challenge, 16);
+	fr_SHA1Update(&Context, (const uint8_t *) user_name,
+		      strlen(user_name));
+	fr_SHA1Final(hash, &Context);
 	memcpy(challenge, hash, 8);
 }
 
@@ -234,39 +198,42 @@ static void challenge_hash( const char *peer_challenge,
  *	returns 42-octet response string
  */
 static void auth_response(const char *username,
-			  const unsigned char *nt_hash_hash,
-			  unsigned char *ntresponse,
-			  char *peer_challenge, char *auth_challenge,
+			  const uint8_t *nt_hash_hash,
+			  uint8_t *ntresponse,
+			  uint8_t *peer_challenge, uint8_t *auth_challenge,
 			  char *response)
 {
-	SHA1_CTX Context;
-	const unsigned char magic1[39] =
+	fr_SHA1_CTX Context;
+	static const uint8_t magic1[39] =
 	{0x4D, 0x61, 0x67, 0x69, 0x63, 0x20, 0x73, 0x65, 0x72, 0x76,
 	 0x65, 0x72, 0x20, 0x74, 0x6F, 0x20, 0x63, 0x6C, 0x69, 0x65,
 	 0x6E, 0x74, 0x20, 0x73, 0x69, 0x67, 0x6E, 0x69, 0x6E, 0x67,
 	 0x20, 0x63, 0x6F, 0x6E, 0x73, 0x74, 0x61, 0x6E, 0x74};
 
-	const unsigned char magic2[41] =
+	static const uint8_t magic2[41] =
 	{0x50, 0x61, 0x64, 0x20, 0x74, 0x6F, 0x20, 0x6D, 0x61, 0x6B,
 	 0x65, 0x20, 0x69, 0x74, 0x20, 0x64, 0x6F, 0x20, 0x6D, 0x6F,
 	 0x72, 0x65, 0x20, 0x74, 0x68, 0x61, 0x6E, 0x20, 0x6F, 0x6E,
 	 0x65, 0x20, 0x69, 0x74, 0x65, 0x72, 0x61, 0x74, 0x69, 0x6F,
 	 0x6E};
 
-        char challenge[8];
-	unsigned char digest[20];
+	static const char hex[16] = "0123456789ABCDEF";
 
-	SHA1Init(&Context);
-	SHA1Update(&Context, nt_hash_hash, 16);
-	SHA1Update(&Context, ntresponse, 24);
-	SHA1Update(&Context, magic1, 39);
-	SHA1Final(digest, &Context);
+	size_t i;
+        uint8_t challenge[8];
+	uint8_t digest[20];
+
+	fr_SHA1Init(&Context);
+	fr_SHA1Update(&Context, nt_hash_hash, 16);
+	fr_SHA1Update(&Context, ntresponse, 24);
+	fr_SHA1Update(&Context, magic1, 39);
+	fr_SHA1Final(digest, &Context);
 	challenge_hash(peer_challenge, auth_challenge, username, challenge);
-	SHA1Init(&Context);
-	SHA1Update(&Context, digest, 20);
-	SHA1Update(&Context, challenge, 8);
-	SHA1Update(&Context, magic2, 41);
-	SHA1Final(digest, &Context);
+	fr_SHA1Init(&Context);
+	fr_SHA1Update(&Context, digest, 20);
+	fr_SHA1Update(&Context, challenge, 8);
+	fr_SHA1Update(&Context, magic2, 41);
+	fr_SHA1Final(digest, &Context);
 
 	/*
 	 *	Encode the value of 'Digest' as "S=" followed by
@@ -277,7 +244,14 @@ static void auth_response(const char *username,
 	 */
  	response[0] = 'S';
 	response[1] = '=';
-	bin2hex(digest, response + 2, 20);
+
+	/*
+	 *	The hexadecimal digits [A-F] MUST be uppercase.
+	 */
+	for (i = 0; i < sizeof(digest); i++) {
+		response[2 + (i * 2)] = hex[(digest[i] >> 4) & 0x0f];
+		response[3 + (i * 2)] = hex[digest[i] & 0x0f];
+	}
 }
 
 
@@ -287,9 +261,12 @@ typedef struct rlm_mschap_t {
         int require_strong;
         int with_ntdomain_hack;	/* this should be in another module */
 	char *passwd_file;
-	char *xlat_name;
+	const char *xlat_name;
 	char *ntlm_auth;
-	char *auth_type;
+	const char *auth_type;
+#ifdef __APPLE__
+	int  open_directory;
+#endif  
 } rlm_mschap_t;
 
 
@@ -299,11 +276,11 @@ typedef struct rlm_mschap_t {
  *	Pulls NT-Response, LM-Response, or Challenge from MSCHAP
  *	attributes.
  */
-static int mschap_xlat(void *instance, REQUEST *request,
+static size_t mschap_xlat(void *instance, REQUEST *request,
 		       char *fmt, char *out, size_t outlen,
 		       RADIUS_ESCAPE_STRING func)
 {
-	int		i, data_len;
+	size_t		i, data_len;
 	uint8_t		*data = NULL;
 	uint8_t		buffer[32];
 	VALUE_PAIR	*user_name;
@@ -318,7 +295,7 @@ static int mschap_xlat(void *instance, REQUEST *request,
 	 *	Challenge means MS-CHAPv1 challenge, or
 	 *	hash of MS-CHAPv2 challenge, and peer challenge.
 	 */
-	if (strcasecmp(fmt, "Challenge") == 0) {
+	if (strncasecmp(fmt, "Challenge", 9) == 0) {
 		chap_challenge = pairfind(request->packet->vps,
 					  PW_MSCHAP_CHALLENGE);
 		if (!chap_challenge) {
@@ -331,8 +308,9 @@ static int mschap_xlat(void *instance, REQUEST *request,
 		 *	for MS-CHAPv2
 		 */
 		if (chap_challenge->length == 8) {
-			DEBUG2(" mschap1: %02x", chap_challenge->strvalue[0]);
-			data = chap_challenge->strvalue;
+			DEBUG2(" mschap1: %02x",
+			       chap_challenge->vp_octets[0]);
+			data = chap_challenge->vp_octets;
 			data_len = 8;
 
 			/*
@@ -342,7 +320,7 @@ static int mschap_xlat(void *instance, REQUEST *request,
 		} else if (chap_challenge->length == 16) {
 			char *username_string;
 
-			DEBUG2(" mschap2: %02x", chap_challenge->strvalue[0]);
+			DEBUG2(" mschap2: %02x", chap_challenge->vp_octets[0]);
 			response = pairfind(request->packet->vps,
 					    PW_MSCHAP2_RESPONSE);
 			if (!response) {
@@ -368,15 +346,15 @@ static int mschap_xlat(void *instance, REQUEST *request,
 			/*
 			 *	with_ntdomain_hack moved here, too.
 			 */
-			if ((username_string = strchr(user_name->strvalue, '\\')) != NULL) {
+			if ((username_string = strchr(user_name->vp_strvalue, '\\')) != NULL) {
 				if (inst->with_ntdomain_hack) {
 					username_string++;
 				} else {
 					DEBUG2("  rlm_mschap: NT Domain delimeter found, should we have enabled with_ntdomain_hack?");
-					username_string = user_name->strvalue;
+					username_string = user_name->vp_strvalue;
 				}
 			} else {
-				username_string = user_name->strvalue;
+				username_string = user_name->vp_strvalue;
 			}
 
 			/*
@@ -384,8 +362,8 @@ static int mschap_xlat(void *instance, REQUEST *request,
 			 *	from the MS-CHAPv2 peer challenge,
 			 *	our challenge, and the user name.
 			 */
-			challenge_hash(response->strvalue + 2,
-				       chap_challenge->strvalue,
+			challenge_hash(response->vp_octets + 2,
+				       chap_challenge->vp_octets,
 				       username_string, buffer);
 			data = buffer;
 			data_len = 8;
@@ -393,12 +371,12 @@ static int mschap_xlat(void *instance, REQUEST *request,
 			DEBUG2("  rlm_mschap: Invalid MS-CHAP challenge length");
 			return 0;
 		}
-		
+
 		/*
 		 *	Get the MS-CHAPv1 response, or the MS-CHAPv2
 		 *	response.
 		 */
-	} else if (strcasecmp(fmt, "NT-Response") == 0) {
+	} else if (strncasecmp(fmt, "NT-Response", 11) == 0) {
 		response = pairfind(request->packet->vps,
 				    PW_MSCHAP_RESPONSE);
 		if (!response) response = pairfind(request->packet->vps,
@@ -413,7 +391,7 @@ static int mschap_xlat(void *instance, REQUEST *request,
 		 *	if the second octet says so.
 		 */
 		if ((response->attribute == PW_MSCHAP_RESPONSE) &&
-		    ((response->strvalue[1] & 0x01) == 0)) {
+		    ((response->vp_octets[1] & 0x01) == 0)) {
 			DEBUG2("  rlm_mschap: No NT-Response in MS-CHAP-Response");
 			return 0;
 		}
@@ -423,14 +401,14 @@ static int mschap_xlat(void *instance, REQUEST *request,
 		 *	the NT-Response at the same offset, and are
 		 *	the same length.
 		 */
-		data = response->strvalue + 26;
+		data = response->vp_octets + 26;
 		data_len = 24;
-		
+
 		/*
 		 *	LM-Response is deprecated, and exists only
 		 *	in MS-CHAPv1, and not often there.
 		 */
-	} else if (strcasecmp(fmt, "LM-Response") == 0) {
+	} else if (strncasecmp(fmt, "LM-Response", 11) == 0) {
 		response = pairfind(request->packet->vps,
 				    PW_MSCHAP_RESPONSE);
 		if (!response) {
@@ -442,17 +420,17 @@ static int mschap_xlat(void *instance, REQUEST *request,
 		 *	For MS-CHAPv1, the NT-Response exists only
 		 *	if the second octet says so.
 		 */
-		if ((response->strvalue[1] & 0x01) != 0) {
+		if ((response->vp_octets[1] & 0x01) != 0) {
 			DEBUG2("  rlm_mschap: No LM-Response in MS-CHAP-Response");
 			return 0;
 		}
-		data = response->strvalue + 2;
+		data = response->vp_octets + 2;
 		data_len = 24;
 
 		/*
 		 *	Pull the NT-Domain out of the User-Name, if it exists.
 		 */
-	} else if (strcasecmp(fmt, "NT-Domain") == 0) {
+	} else if (strncasecmp(fmt, "NT-Domain", 9) == 0) {
 		char *p, *q;
 
 		user_name = pairfind(request->packet->vps, PW_USER_NAME);
@@ -460,22 +438,22 @@ static int mschap_xlat(void *instance, REQUEST *request,
 			DEBUG2("  rlm_mschap: No User-Name was found in the request.");
 			return 0;
 		}
-		
+
 		/*
 		 *	First check to see if this is a host/ style User-Name
 		 *	(a la Kerberos host principal)
 		 */
-		if (strncmp(user_name->strvalue, "host/", 5) == 0) {
+		if (strncmp(user_name->vp_strvalue, "host/", 5) == 0) {
 			/*
 			 *	If we're getting a User-Name formatted in this way,
 			 *	it's likely due to PEAP.  The Windows Domain will be
 			 *	the first domain component following the hostname,
 			 *	or the machine name itself if only a hostname is supplied
 			 */
-			p = strchr(user_name->strvalue, '.');
+			p = strchr(user_name->vp_strvalue, '.');
 			if (!p) {
 				DEBUG2("  rlm_mschap: setting NT-Domain to same as machine name");
-				strNcpy(out, user_name->strvalue + 5, outlen);
+				strlcpy(out, user_name->vp_strvalue + 5, outlen);
 			} else {
 				p++;	/* skip the period */
 				q = strchr(p, '.');
@@ -484,11 +462,11 @@ static int mschap_xlat(void *instance, REQUEST *request,
 				 * only if another period was found
 				 */
 				if (q) *q = '\0';
-				strNcpy(out, p, outlen);
+				strlcpy(out, p, outlen);
 				if (q) *q = '.';
 			}
 		} else {
-			p = strchr(user_name->strvalue, '\\');
+			p = strchr(user_name->vp_strvalue, '\\');
 			if (!p) {
 				DEBUG2("  rlm_mschap: No NT-Domain was found in the User-Name.");
 				return 0;
@@ -498,7 +476,7 @@ static int mschap_xlat(void *instance, REQUEST *request,
 			 *	Hack.  This is simpler than the alternatives.
 			 */
 			*p = '\0';
-			strNcpy(out, user_name->strvalue, outlen);
+			strlcpy(out, user_name->vp_strvalue, outlen);
 			*p = '\\';
 		}
 
@@ -507,7 +485,7 @@ static int mschap_xlat(void *instance, REQUEST *request,
 		/*
 		 *	Pull the User-Name out of the User-Name...
 		 */
-	} else if (strcasecmp(fmt, "User-Name") == 0) {
+	} else if (strncasecmp(fmt, "User-Name", 9) == 0) {
 		char *p;
 
 		user_name = pairfind(request->packet->vps, PW_USER_NAME);
@@ -515,12 +493,12 @@ static int mschap_xlat(void *instance, REQUEST *request,
 			DEBUG2("  rlm_mschap: No User-Name was found in the request.");
 			return 0;
 		}
-		
+
 		/*
 		 *	First check to see if this is a host/ style User-Name
 		 *	(a la Kerberos host principal)
 		 */
-		if (strncmp(user_name->strvalue, "host/", 5) == 0) {
+		if (strncmp(user_name->vp_strvalue, "host/", 5) == 0) {
 			/*
 			 *	If we're getting a User-Name formatted in this way,
 			 *	it's likely due to PEAP.  When authenticating this against
@@ -531,22 +509,22 @@ static int mschap_xlat(void *instance, REQUEST *request,
 			 *	from that point to the first period into a string and appending
 			 * 	a $ to the end.
 			 */
-			p = strchr(user_name->strvalue, '.');
+			p = strchr(user_name->vp_strvalue, '.');
 			/*
 			 * use the same hack as above
 			 * only if a period was found
 			 */
 			if (p) *p = '\0';
-			snprintf(out, outlen, "%s$", user_name->strvalue + 5);
+			snprintf(out, outlen, "%s$", user_name->vp_strvalue + 5);
 			if (p) *p = '.';
 		} else {
-			p = strchr(user_name->strvalue, '\\');
+			p = strchr(user_name->vp_strvalue, '\\');
 			if (p) {
 				p++;	/* skip the backslash */
 			} else {
-				p = user_name->strvalue; /* use the whole User-Name */
+				p = user_name->vp_strvalue; /* use the whole User-Name */
 			}
-			strNcpy(out, p, outlen);
+			strlcpy(out, p, outlen);
 		}
 
 		return strlen(out);
@@ -563,7 +541,7 @@ static int mschap_xlat(void *instance, REQUEST *request,
 		DEBUG("rlm_mschap: NT-Hash: %s",p);
 		ntpwdhash(buffer,p);
 
-		lrad_bin2hex(buffer, out, 16);
+		fr_bin2hex(buffer, out, 16);
 		out[32] = '\0';
 		DEBUG("rlm_mschap: NT-Hash: Result: %s",out);
 		return 32;
@@ -577,10 +555,10 @@ static int mschap_xlat(void *instance, REQUEST *request,
 		p = fmt + 8;	/* 7 is the length of 'LM-Hash' */
 		if ((p == '\0') || (outlen <= 32))
 			return 0;
-			
+
 		DEBUG("rlm_mschap: LM-Hash: %s",p);
-		smbdes_lmpwdhash(p,buffer);
-		lrad_bin2hex(buffer, out, 16);
+		smbdes_lmpwdhash(p, buffer);
+		fr_bin2hex(buffer, out, 16);
 		out[32] = '\0';
 		DEBUG("rlm_mschap: LM-Hash: Result: %s",out);
 		return 32;
@@ -608,18 +586,18 @@ static int mschap_xlat(void *instance, REQUEST *request,
 	}
 
 	/*
-	 *	
+	 *
 	 */
 	for (i = 0; i < data_len; i++) {
 		sprintf(out + (2 * i), "%02x", data[i]);
 	}
 	out[data_len * 2] = '\0';
-	
+
 	return data_len * 2;
 }
 
 
-static CONF_PARSER module_config[] = {
+static const CONF_PARSER module_config[] = {
 	/*
 	 *	Cache the password by default.
 	 */
@@ -635,6 +613,10 @@ static CONF_PARSER module_config[] = {
 	  offsetof(rlm_mschap_t, passwd_file), NULL,  NULL },
 	{ "ntlm_auth",   PW_TYPE_STRING_PTR,
 	  offsetof(rlm_mschap_t, ntlm_auth), NULL,  NULL },
+#ifdef __APPLE__
+	{ "use_open_directory",    PW_TYPE_BOOLEAN,
+	  offsetof(rlm_mschap_t,open_directory), NULL, "yes" },
+#endif
 
 	{ NULL, -1, 0, NULL, NULL }		/* end the list */
 };
@@ -645,10 +627,9 @@ static CONF_PARSER module_config[] = {
  */
 static int mschap_detach(void *instance){
 #define inst ((rlm_mschap_t *)instance)
-	free(inst->passwd_file);
-	free(inst->ntlm_auth);
 	if (inst->xlat_name) {
 		xlat_unregister(inst->xlat_name, mschap_xlat);
+		free(inst->xlat_name);
 	}
 	free(instance);
 	return 0;
@@ -691,6 +672,7 @@ static int mschap_instantiate(CONF_SECTION *conf, void **instance)
 	 */
 	inst->xlat_name = cf_section_name2(conf);
 	if (!inst->xlat_name) inst->xlat_name = cf_section_name1(conf);
+	inst->xlat_name = strdup(inst->xlat_name);
 	xlat_register(inst->xlat_name, mschap_xlat, inst);
 
 	/*
@@ -709,7 +691,7 @@ static int mschap_instantiate(CONF_SECTION *conf, void **instance)
  *	add_reply() adds either MS-CHAP2-Success or MS-CHAP-Error
  *	attribute to reply packet
  */
-static void add_reply(VALUE_PAIR** vp, unsigned char ident,
+void mschap_add_reply(VALUE_PAIR** vp, unsigned char ident,
 		      const char* name, const char* value, int len)
 {
 	VALUE_PAIR *reply_attr;
@@ -719,8 +701,8 @@ static void add_reply(VALUE_PAIR** vp, unsigned char ident,
 		return;
 	}
 
-	reply_attr->strvalue[0] = ident;
-	memcpy(reply_attr->strvalue + 1, value, len);
+	reply_attr->vp_octets[0] = ident;
+	memcpy(reply_attr->vp_octets + 1, value, len);
 	reply_attr->length = len + 1;
 	pairadd(vp, reply_attr);
 }
@@ -728,19 +710,18 @@ static void add_reply(VALUE_PAIR** vp, unsigned char ident,
 /*
  *	Add MPPE attributes to the reply.
  */
-static void mppe_add_reply(VALUE_PAIR **vp,
-			   const char* name, const char* value, int len)
+static void mppe_add_reply(REQUEST *request,
+			   const char* name, const uint8_t * value, int len)
 {
-       VALUE_PAIR *reply_attr;
-       reply_attr = pairmake(name, "", T_OP_EQ);
-       if (!reply_attr) {
+       VALUE_PAIR *vp;
+       vp = radius_pairmake(request, &request->reply->vps, name, "", T_OP_EQ);
+       if (!vp) {
 	       DEBUG("rlm_mschap: mppe_add_reply failed to create attribute %s: %s\n", name, librad_errstr);
 	       return;
        }
 
-       memcpy(reply_attr->strvalue, value, len);
-       reply_attr->length = len;
-       pairadd(vp, reply_attr);
+       memcpy(vp->vp_octets, value, len);
+       vp->length = len;
 }
 
 
@@ -772,7 +753,7 @@ static int do_mschap(rlm_mschap_t *inst,
 	 */
 	vp = pairfind(request->config_items,
 		      PW_MS_CHAP_USE_NTLM_AUTH);
-	if (vp) do_ntlm_auth = vp->lvalue;
+	if (vp) do_ntlm_auth = vp->vp_integer;
 
 	/*
 	 *	No ntlm_auth configured, attribute to tell us to
@@ -795,19 +776,19 @@ static int do_mschap(rlm_mschap_t *inst,
 			DEBUG2("  rlm_mschap: FAILED: No NT/LM-Password.  Cannot perform authentication.");
 			return -1;
 		}
-		
-		smbdes_mschap(password->strvalue, challenge, calculated);
+
+		smbdes_mschap(password->vp_strvalue, challenge, calculated);
 		if (memcmp(response, calculated, 24) != 0) {
 			return -1;
 		}
-		
+
 		/*
 		 *	If the password exists, and is an NT-Password,
 		 *	then calculate the hash of the NT hash.  Doing this
 		 *	here minimizes work for later.
 		 */
 		if (password && (password->attribute == PW_NT_PASSWORD)) {
-			md4_calc(nthashhash, password->strvalue, 16);
+			fr_md4_calc(nthashhash, password->vp_octets, 16);
 		} else {
 			memset(nthashhash, 0, 16);
 		}
@@ -818,29 +799,14 @@ static int do_mschap(rlm_mschap_t *inst,
 		memset(nthashhash, 0, 16);
 
 		/*
-		 *	Run the program, and expect that we get 16 
+		 *	Run the program, and expect that we get 16
 		 */
 		result = radius_exec_program(inst->ntlm_auth, request,
 					     TRUE, /* wait */
 					     buffer, sizeof(buffer),
-					     NULL, NULL);
+					     NULL, NULL, 1);
 		if (result != 0) {
-			char *p;
-			
 			DEBUG2("  rlm_mschap: External script failed.");
-
-			vp = pairmake("Module-Failure-Message", "", T_OP_EQ);
-			if (!vp) {
-				radlog(L_ERR, "No memory");
-				return -1;
-			}
-
-			p = strchr(buffer, '\n');
-			if (p) *p = '\0';
-			snprintf(vp->strvalue, sizeof(vp->strvalue), 
-				 "rlm_mschap: %s", buffer);
-			vp->length = strlen(vp->strvalue);
-			pairadd(&request->packet->vps, vp);
 			return -1;
 		}
 
@@ -867,7 +833,7 @@ static int do_mschap(rlm_mschap_t *inst,
 		/*
 		 *	Update the NT hash hash, from the NT key.
 		 */
-		if (hex2bin(buffer + 8, nthashhash, 16) != 16) {
+		if (fr_hex2bin(buffer + 8, nthashhash, 16) != 16) {
 			DEBUG2("  rlm_mschap: Invalid output from ntlm_auth: NT_KEY has non-hex values");
 			return -1;
 		}
@@ -924,13 +890,13 @@ static void mppe_GetMasterKey(uint8_t *nt_hashhash,uint8_t *nt_response,
 			      uint8_t *masterkey)
 {
        uint8_t digest[20];
-       SHA1_CTX Context;
+       fr_SHA1_CTX Context;
 
-       SHA1Init(&Context);
-       SHA1Update(&Context,nt_hashhash,16);
-       SHA1Update(&Context,nt_response,24);
-       SHA1Update(&Context,magic1,27);
-       SHA1Final(digest,&Context);
+       fr_SHA1Init(&Context);
+       fr_SHA1Update(&Context,nt_hashhash,16);
+       fr_SHA1Update(&Context,nt_response,24);
+       fr_SHA1Update(&Context,magic1,27);
+       fr_SHA1Final(digest,&Context);
 
        memcpy(masterkey,digest,16);
 }
@@ -941,7 +907,7 @@ static void mppe_GetAsymmetricStartKey(uint8_t *masterkey,uint8_t *sesskey,
 {
        uint8_t digest[20];
        const uint8_t *s;
-       SHA1_CTX Context;
+       fr_SHA1_CTX Context;
 
        memset(digest,0,20);
 
@@ -951,12 +917,12 @@ static void mppe_GetAsymmetricStartKey(uint8_t *masterkey,uint8_t *sesskey,
                s = magic2;
        }
 
-       SHA1Init(&Context);
-       SHA1Update(&Context,masterkey,16);
-       SHA1Update(&Context,SHSpad1,40);
-       SHA1Update(&Context,s,84);
-       SHA1Update(&Context,SHSpad2,40);
-       SHA1Final(digest,&Context);
+       fr_SHA1Init(&Context);
+       fr_SHA1Update(&Context,masterkey,16);
+       fr_SHA1Update(&Context,SHSpad1,40);
+       fr_SHA1Update(&Context,s,84);
+       fr_SHA1Update(&Context,SHSpad2,40);
+       fr_SHA1Final(digest,&Context);
 
        memcpy(sesskey,digest,keylen);
 }
@@ -1003,9 +969,7 @@ static void mppe_chap2_gen_keys128(uint8_t *nt_hashhash,uint8_t *response,
 static int mschap_authorize(void * instance, REQUEST *request)
 {
 #define inst ((rlm_mschap_t *)instance)
-	VALUE_PAIR *challenge = NULL;
-	VALUE_PAIR *response = NULL;
-	VALUE_PAIR *vp;
+	VALUE_PAIR *challenge = NULL, *response = NULL;
 
 	challenge = pairfind(request->packet->vps, PW_MSCHAP_CHALLENGE);
 	if (!challenge) {
@@ -1024,6 +988,11 @@ static int mschap_authorize(void * instance, REQUEST *request)
 		return RLM_MODULE_NOOP;
 	}
 
+	if (pairfind(request->config_items, PW_AUTH_TYPE)) {
+		DEBUG2("  rlm_mschap: Found existing Auth-Type.  Not changing it.");
+		return RLM_MODULE_NOOP;
+	}
+
 	DEBUG2("  rlm_mschap: Found MS-CHAP attributes.  Setting 'Auth-Type  = %s'", inst->xlat_name);
 
 	/*
@@ -1031,10 +1000,10 @@ static int mschap_authorize(void * instance, REQUEST *request)
 	 *	will take care of turning clear-text passwords into
 	 *	NT/LM passwords.
 	 */
-	vp = pairmake("Auth-Type", inst->auth_type, T_OP_EQ);
-	if (!vp) return RLM_MODULE_FAIL;
-	pairmove(&request->config_items, &vp);
-	pairfree(&vp);		/* may be NULL */
+	if (!radius_pairmake(request, &request->config_items,
+			     "Auth-Type", inst->auth_type, T_OP_EQ)) {
+		return RLM_MODULE_FAIL;
+	}
 
 	return RLM_MODULE_OK;
 #undef inst
@@ -1049,7 +1018,7 @@ static int mschap_authorize(void * instance, REQUEST *request)
  *
  *	If PW_SMB_ACCOUNT_CTRL is not set to ACB_PWNOTREQ we must have
  *	one of:
- *		PAP:      PW_PASSWORD or
+ *		PAP:      PW_USER_PASSWORD or
  *		MS-CHAP:  PW_MSCHAP_CHALLENGE and PW_MSCHAP_RESPONSE or
  *		MS-CHAP2: PW_MSCHAP_CHALLENGE and PW_MSCHAP2_RESPONSE
  *	In case of password mismatch or locked account we MAY return
@@ -1065,9 +1034,8 @@ static int mschap_authenticate(void * instance, REQUEST *request)
 	VALUE_PAIR *password = NULL;
 	VALUE_PAIR *lm_password, *nt_password, *smb_ctrl;
 	VALUE_PAIR *username;
-	VALUE_PAIR *reply_attr;
 	uint8_t nthashhash[16];
-	uint8_t msch2resp[42];
+	char msch2resp[42];
 	char *username_string;
 	int chap = 0;
 
@@ -1080,9 +1048,13 @@ static int mschap_authenticate(void * instance, REQUEST *request)
 		password = pairfind(request->config_items,
 				    PW_SMB_ACCOUNT_CTRL_TEXT);
 		if (password) {
-			smb_ctrl = pairmake("SMB-Account-CTRL", "0", T_OP_SET);
-			pairadd(&request->config_items, smb_ctrl);
-			smb_ctrl->lvalue = pdb_decode_acct_ctrl(password->strvalue);
+			smb_ctrl = radius_pairmake(request,
+						   &request->config_items,
+						   "SMB-Account-CTRL", "0",
+						   T_OP_SET);
+			if (smb_ctrl) {
+				smb_ctrl->vp_integer = pdb_decode_acct_ctrl(password->vp_strvalue);
+			}
 		}
 	}
 
@@ -1094,7 +1066,7 @@ static int mschap_authenticate(void * instance, REQUEST *request)
 		/*
 		 *	Password is not required.
 		 */
-		if ((smb_ctrl->lvalue & ACB_PWNOTREQ) != 0) {
+		if ((smb_ctrl->vp_integer & ACB_PWNOTREQ) != 0) {
 			DEBUG2("  rlm_mschap: SMB-Account-Ctrl says no password is required.");
 			return RLM_MODULE_OK;
 		}
@@ -1103,7 +1075,7 @@ static int mschap_authenticate(void * instance, REQUEST *request)
 	/*
 	 *	Decide how to get the passwords.
 	 */
-	password = pairfind(request->config_items, PW_PASSWORD);
+	password = pairfind(request->config_items, PW_CLEARTEXT_PASSWORD);
 
 	/*
 	 *	We need an LM-Password.
@@ -1115,8 +1087,8 @@ static int mschap_authenticate(void * instance, REQUEST *request)
 		 */
 		if ((lm_password->length == 16) ||
 		    ((lm_password->length == 32) &&
-		     (hex2bin(lm_password->strvalue,
-			      lm_password->strvalue, 16) == 16))) {
+		     (fr_hex2bin(lm_password->vp_strvalue,
+				 lm_password->vp_octets, 16) == 16))) {
 			DEBUG2("  rlm_mschap: Found LM-Password");
 			lm_password->length = 16;
 
@@ -1126,17 +1098,17 @@ static int mschap_authenticate(void * instance, REQUEST *request)
 		}
 
 	} else if (!password) {
-		DEBUG2("  rlm_mschap: No User-Password configured.  Cannot create LM-Password.");
+		DEBUG2("  rlm_mschap: No Cleartext-Password configured.  Cannot create LM-Password.");
 
-	} else {		/* there is a configured User-Password */
-		lm_password = pairmake("LM-Password", "", T_OP_EQ);
+	} else {		/* there is a configured Cleartext-Password */
+		lm_password = radius_pairmake(request, &request->config_items,
+					      "LM-Password", "", T_OP_EQ);
 		if (!lm_password) {
 			radlog(L_ERR, "No memory");
 		} else {
-			smbdes_lmpwdhash(password->strvalue,
-				       lm_password->strvalue);
+			smbdes_lmpwdhash(password->vp_strvalue,
+					 lm_password->vp_octets);
 			lm_password->length = 16;
-			pairadd(&request->config_items, lm_password);
 		}
 	}
 
@@ -1147,8 +1119,8 @@ static int mschap_authenticate(void * instance, REQUEST *request)
 	if (nt_password) {
 		if ((nt_password->length == 16) ||
 		    ((nt_password->length == 32) &&
-		     (hex2bin(nt_password->strvalue,
-			      nt_password->strvalue, 16) == 16))) {
+		     (fr_hex2bin(nt_password->vp_strvalue,
+				 nt_password->vp_octets, 16) == 16))) {
 			DEBUG2("  rlm_mschap: Found NT-Password");
 			nt_password->length = 16;
 
@@ -1157,17 +1129,18 @@ static int mschap_authenticate(void * instance, REQUEST *request)
 			nt_password = NULL;
 		}
 	} else if (!password) {
-		DEBUG2("  rlm_mschap: No User-Password configured.  Cannot create NT-Password.");
+		DEBUG2("  rlm_mschap: No Cleartext-Password configured.  Cannot create NT-Password.");
 
-	} else {		/* there is a configured User-Password */
-		nt_password = pairmake("NT-Password", "", T_OP_EQ);
+	} else {		/* there is a configured Cleartext-Password */
+		nt_password = radius_pairmake(request, &request->config_items,
+					      "NT-Password", "", T_OP_EQ);
 		if (!nt_password) {
 			radlog(L_ERR, "No memory");
 			return RLM_MODULE_FAIL;
 		} else {
-			ntpwdhash(nt_password->strvalue, password->strvalue);
+			ntpwdhash(nt_password->vp_octets,
+				  password->vp_strvalue);
 			nt_password->length = 16;
-			pairadd(&request->config_items, nt_password);
 		}
 	}
 
@@ -1208,7 +1181,7 @@ static int mschap_authenticate(void * instance, REQUEST *request)
 		 *	We are doing MS-CHAP.  Calculate the MS-CHAP
 		 *	response
 		 */
-		if (response->strvalue[1] & 0x01) {
+		if (response->vp_octets[1] & 0x01) {
 			DEBUG2("  rlm_mschap: Told to do MS-CHAPv1 with NT-Password");
 			password = nt_password;
 			offset = 26;
@@ -1221,11 +1194,12 @@ static int mschap_authenticate(void * instance, REQUEST *request)
 		/*
 		 *	Do the MS-CHAP authentication.
 		 */
-		if (do_mschap(inst, request, password, challenge->strvalue,
-			      response->strvalue + offset, nthashhash) < 0) {
+		if (do_mschap(inst, request, password, challenge->vp_octets,
+			      response->vp_octets + offset, nthashhash) < 0) {
 			DEBUG2("  rlm_mschap: MS-CHAP-Response is incorrect.");
-			add_reply(&request->reply->vps, *response->strvalue,
-				  "MS-CHAP-Error", "E=691 R=1", 9);
+			mschap_add_reply(&request->reply->vps,
+					 *response->vp_octets,
+					 "MS-CHAP-Error", "E=691 R=1", 9);
 			return RLM_MODULE_REJECT;
 		}
 
@@ -1263,17 +1237,27 @@ static int mschap_authenticate(void * instance, REQUEST *request)
 		/*
 		 *	with_ntdomain_hack moved here
 		 */
-		if ((username_string = strchr(username->strvalue, '\\')) != NULL) {
+		if ((username_string = strchr(username->vp_strvalue, '\\')) != NULL) {
 		        if (inst->with_ntdomain_hack) {
 			        username_string++;
 			} else {
 				DEBUG2("  rlm_mschap: NT Domain delimeter found, should we have enabled with_ntdomain_hack?");
-				username_string = username->strvalue;
+				username_string = username->vp_strvalue;
 			}
 		} else {
-		        username_string = username->strvalue;
+		        username_string = username->vp_strvalue;
 		}
 
+#ifdef __APPLE__
+		/*
+		 *	No "known good" NT-Password attribute.  Try to do
+		 *	OpenDirectory authentication.
+		 */
+		if (!nt_password && inst->open_directory) {
+			DEBUG2("  rlm_mschap: No NT-Password configured. Trying DirectoryService Authentication.");
+			return od_mschap_auth(request, challenge, username);
+		}
+#endif
 		/*
 		 *	The old "mschapv2" function has been moved to
 		 *	here.
@@ -1281,19 +1265,20 @@ static int mschap_authenticate(void * instance, REQUEST *request)
 		 *	MS-CHAPv2 takes some additional data to create an
 		 *	MS-CHAPv1 challenge, and then does MS-CHAPv1.
 		 */
-		challenge_hash(response->strvalue + 2, /* peer challenge */
-			       challenge->strvalue, /* our challenge */
+		challenge_hash(response->vp_octets + 2, /* peer challenge */
+			       challenge->vp_octets, /* our challenge */
 			       username_string,	/* user name */
 			       mschapv1_challenge); /* resulting challenge */
-		
+
 		DEBUG2("  rlm_mschap: Told to do MS-CHAPv2 for %s with NT-Password",
 		       username_string);
 
 		if (do_mschap(inst, request, nt_password, mschapv1_challenge,
-			      response->strvalue + 26, nthashhash) < 0) {
+			      response->vp_octets + 26, nthashhash) < 0) {
 			DEBUG2("  rlm_mschap: FAILED: MS-CHAP2-Response is incorrect");
-			add_reply(&request->reply->vps, *response->strvalue,
-				  "MS-CHAP-Error", "E=691 R=1", 9);
+			mschap_add_reply(&request->reply->vps,
+					 *response->vp_octets,
+					 "MS-CHAP-Error", "E=691 R=1", 9);
 			return RLM_MODULE_REJECT;
 		}
 
@@ -1305,12 +1290,12 @@ static int mschap_authenticate(void * instance, REQUEST *request)
 
 		auth_response(username_string, /* without the domain */
 			      nthashhash, /* nt-hash-hash */
-			      response->strvalue + 26, /* peer response */
-			      response->strvalue + 2, /* peer challenge */
-			      challenge->strvalue, /* our challenge */
+			      response->vp_octets + 26, /* peer response */
+			      response->vp_octets + 2, /* peer challenge */
+			      challenge->vp_octets, /* our challenge */
 			      msch2resp); /* calculated MPPE key */
-		add_reply( &request->reply->vps, *response->strvalue,
-			   "MS-CHAP2-Success", msch2resp, 42);
+		mschap_add_reply(&request->reply->vps, *response->vp_octets,
+				 "MS-CHAP2-Success", msch2resp, 42);
 		chap = 2;
 
 	} else {		/* Neither CHAPv1 or CHAPv2 response: die */
@@ -1330,21 +1315,23 @@ static int mschap_authenticate(void * instance, REQUEST *request)
 		 *	They're found, but they don't exist, so we
 		 *	return 'not found'.
 		 */
-		if (((smb_ctrl->lvalue & ACB_DISABLED) != 0) ||
-		    ((smb_ctrl->lvalue & ACB_NORMAL) == 0)) {
+		if (((smb_ctrl->vp_integer & ACB_DISABLED) != 0) ||
+		    ((smb_ctrl->vp_integer & ACB_NORMAL) == 0)) {
 			DEBUG2("  rlm_mschap: SMB-Account-Ctrl says that the account is disabled, or is not a normal account.");
-			add_reply( &request->reply->vps, *response->strvalue,
-				   "MS-CHAP-Error", "E=691 R=1", 9);
+			mschap_add_reply( &request->reply->vps,
+					  *response->vp_octets,
+					  "MS-CHAP-Error", "E=691 R=1", 9);
 			return RLM_MODULE_NOTFOUND;
 		}
 
 		/*
 		 *	User is locked out.
 		 */
-		if ((smb_ctrl->lvalue & ACB_AUTOLOCK) != 0) {
+		if ((smb_ctrl->vp_integer & ACB_AUTOLOCK) != 0) {
 			DEBUG2("  rlm_mschap: SMB-Account-Ctrl says that the account is locked out.");
-			add_reply( &request->reply->vps, *response->strvalue,
-				   "MS-CHAP-Error", "E=647 R=0", 9);
+			mschap_add_reply( &request->reply->vps,
+					  *response->vp_octets,
+					  "MS-CHAP-Error", "E=647 R=0", 9);
 			return RLM_MODULE_USERLOCK;
 		}
 	}
@@ -1358,7 +1345,7 @@ static int mschap_authenticate(void * instance, REQUEST *request)
 			DEBUG2("rlm_mschap: adding MS-CHAPv1 MPPE keys");
 			memset(mppe_sendkey, 0, 32);
 			if (lm_password) {
-				memcpy(mppe_sendkey, lm_password->strvalue, 8);
+				memcpy(mppe_sendkey, lm_password->vp_octets, 8);
 			}
 
 			/*
@@ -1375,34 +1362,31 @@ static int mschap_authenticate(void * instance, REQUEST *request)
 			 */
 			memcpy(mppe_sendkey + 8,
 			       nthashhash, 16);
-			mppe_add_reply(&request->reply->vps,
+			mppe_add_reply(request,
 				       "MS-CHAP-MPPE-Keys",
 				       mppe_sendkey, 32);
 		} else if (chap == 2) {
 			DEBUG2("rlm_mschap: adding MS-CHAPv2 MPPE keys");
 			mppe_chap2_gen_keys128(nthashhash,
-					       response->strvalue + 26,
+					       response->vp_octets + 26,
 					       mppe_sendkey, mppe_recvkey);
-			
-			mppe_add_reply(&request->reply->vps,
+
+			mppe_add_reply(request,
 				       "MS-MPPE-Recv-Key",
 				       mppe_recvkey, 16);
-			mppe_add_reply(&request->reply->vps,
+			mppe_add_reply(request,
 				       "MS-MPPE-Send-Key",
 				       mppe_sendkey, 16);
 
 		}
-		reply_attr = pairmake("MS-MPPE-Encryption-Policy",
-				      (inst->require_encryption)? "0x00000002":"0x00000001",
-				      T_OP_EQ);
-		rad_assert(reply_attr != NULL);
-		pairadd(&request->reply->vps, reply_attr);
-		reply_attr = pairmake("MS-MPPE-Encryption-Types",
-				      (inst->require_strong)? "0x00000004":"0x00000006",
-				      T_OP_EQ);
-		rad_assert(reply_attr != NULL);
-		pairadd(&request->reply->vps, reply_attr);
-
+		radius_pairmake(request, &request->reply->vps,
+				"MS-MPPE-Encryption-Policy",
+				(inst->require_encryption)? "0x00000002":"0x00000001",
+				T_OP_EQ);
+		radius_pairmake(request, &request->reply->vps,
+				"MS-MPPE-Encryption-Types",
+				(inst->require_strong)? "0x00000004":"0x00000006",
+				T_OP_EQ);
 	} /* else we weren't asked to use MPPE */
 
 	return RLM_MODULE_OK;
@@ -1410,20 +1394,19 @@ static int mschap_authenticate(void * instance, REQUEST *request)
 }
 
 module_t rlm_mschap = {
-  "MS-CHAP",
-  RLM_TYPE_THREAD_SAFE,		/* type */
-  NULL,				/* initialize */
-  mschap_instantiate,		/* instantiation */
-  {
-	  mschap_authenticate,	/* authenticate */
-	  mschap_authorize,	/* authorize */
-	  NULL,			/* pre-accounting */
-	  NULL,			/* accounting */
-	  NULL,			/* checksimul */
-	  NULL,			/* pre-proxy */
-	  NULL,			/* post-proxy */
-	  NULL			/* post-auth */
-  },
-  mschap_detach,		/* detach */
-  NULL,				/* destroy */
+	RLM_MODULE_INIT,
+	"MS-CHAP",
+	RLM_TYPE_THREAD_SAFE,		/* type */
+	mschap_instantiate,		/* instantiation */
+	mschap_detach,		/* detach */
+	{
+		mschap_authenticate,	/* authenticate */
+		mschap_authorize,	/* authorize */
+		NULL,			/* pre-accounting */
+		NULL,			/* accounting */
+		NULL,			/* checksimul */
+		NULL,			/* pre-proxy */
+		NULL,			/* post-proxy */
+		NULL			/* post-auth */
+	},
 };
