@@ -1,7 +1,7 @@
 /*
  * rlm_pap.c
  *
- * Version:  $Id$
+ * Version:  $Id: rlm_pap.c,v 1.63 2008/03/29 21:30:33 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -15,23 +15,20 @@
  *
  *   You should have received a copy of the GNU General Public License
  *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
  *
- * Copyright 2001  The FreeRADIUS server project
+ * Copyright 2001,2006  The FreeRADIUS server project
  * Copyright 2001  Kostas Kalevras <kkalev@noc.ntua.gr>
  */
 
-#include "autoconf.h"
-#include "libradius.h"
+#include <freeradius-devel/ident.h>
+RCSID("$Id: rlm_pap.c,v 1.63 2008/03/29 21:30:33 aland Exp $")
 
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
-#include <unistd.h>
+#include <freeradius-devel/radiusd.h>
+#include <freeradius-devel/modules.h>
+
+#include <ctype.h>
 
-#include "radiusd.h"
-#include "modules.h"
-#include "conffile.h"
 #include "../../include/md5.h"
 #include "../../include/sha1.h"
 
@@ -49,8 +46,6 @@
 #define PAP_MAX_ENC		9
 
 
-static const char rcsid[] = "$Id$";
-
 /*
  *      Define a structure for our module configuration.
  *
@@ -64,6 +59,7 @@ typedef struct rlm_pap_t {
 	int sch;
 	char norm_passwd;
 	int auto_header;
+	int auth_type;
 } rlm_pap_t;
 
 /*
@@ -75,13 +71,13 @@ typedef struct rlm_pap_t {
  *      to the strdup'd string into 'config.string'.  This gets around
  *      buffer over-flows.
  */
-static CONF_PARSER module_config[] = {
-  { "encryption_scheme", PW_TYPE_STRING_PTR, offsetof(rlm_pap_t,scheme), NULL, "crypt" },
+static const CONF_PARSER module_config[] = {
+	{ "encryption_scheme", PW_TYPE_STRING_PTR, offsetof(rlm_pap_t,scheme), NULL, "auto" },
 	{ "auto_header", PW_TYPE_BOOLEAN, offsetof(rlm_pap_t,auto_header), NULL, "no" },
-  { NULL, -1, 0, NULL, NULL }
+	{ NULL, -1, 0, NULL, NULL }
 };
 
-static const LRAD_NAME_NUMBER schemes[] = {
+static const FR_NAME_NUMBER schemes[] = {
 	{ "clear", PAP_ENC_CLEAR },
 	{ "crypt", PAP_ENC_CRYPT },
 	{ "md5", PAP_ENC_MD5 },
@@ -98,7 +94,7 @@ static const LRAD_NAME_NUMBER schemes[] = {
 /*
  *	For auto-header discovery.
  */
-static const LRAD_NAME_NUMBER header_names[] = {
+static const FR_NAME_NUMBER header_names[] = {
 	{ "{clear}",	PW_CLEARTEXT_PASSWORD },
 	{ "{cleartext}", PW_CLEARTEXT_PASSWORD },
 	{ "{md5}",	PW_MD5_PASSWORD },
@@ -109,6 +105,8 @@ static const LRAD_NAME_NUMBER header_names[] = {
 	{ "{nt}",	PW_NT_PASSWORD },
 	{ "{x-nthash}",	PW_NT_PASSWORD },
 	{ "{ns-mta-md5}", PW_NS_MTA_MD5_PASSWORD },
+	{ "{x- orcllmv}", PW_LM_PASSWORD },
+	{ "{X- ORCLNTV}", PW_NT_PASSWORD },
 	{ NULL, 0 }
 };
 
@@ -117,7 +115,6 @@ static int pap_detach(void *instance)
 {
 	rlm_pap_t *inst = (rlm_pap_t *) instance;
 
-	free((char *)inst->scheme);
 	free(inst);
 
 	return 0;
@@ -127,6 +124,7 @@ static int pap_detach(void *instance)
 static int pap_instantiate(CONF_SECTION *conf, void **instance)
 {
         rlm_pap_t *inst;
+	DICT_VALUE *dval;
 
         /*
          *      Set up a storage area for instance data
@@ -142,29 +140,36 @@ static int pap_instantiate(CONF_SECTION *conf, void **instance)
          *      fail.
          */
         if (cf_section_parse(conf, inst, module_config) < 0) {
-                pap_detach(inst);
+		pap_detach(inst);
                 return -1;
         }
-
 	if (inst->scheme == NULL || strlen(inst->scheme) == 0){
 		radlog(L_ERR, "rlm_pap: No scheme defined");
 		pap_detach(inst);
 		return -1;
 	}
 
-	inst->sch = lrad_str2int(schemes, inst->scheme, PAP_ENC_INVALID);
+	inst->sch = fr_str2int(schemes, inst->scheme, PAP_ENC_INVALID);
 	if (inst->sch == PAP_ENC_INVALID) {
 		radlog(L_ERR, "rlm_pap: Unknown scheme \"%s\"", inst->scheme);
 		pap_detach(inst);
 		return -1;
 	}
 
-        *instance = inst;
 	inst->name = cf_section_name2(conf);
 	if (!inst->name) {
 		inst->name = cf_section_name1(conf);
 	}
 
+	dval = dict_valbyname(PW_AUTH_TYPE, inst->name);
+	if (dval) {
+		inst->auth_type = dval->value;
+	} else {
+		inst->auth_type = 0;
+	}
+
+        *instance = inst;
+
         return 0;
 }
 
@@ -182,7 +187,7 @@ static int decode_it(const char *src, uint8_t *dst)
 			x = (x << 6) + (unsigned int)(src[i] - 'A' + 0);
 		else if (src[i] >= 'a' && src[i] <= 'z')
 			 x = (x << 6) + (unsigned int)(src[i] - 'a' + 26);
-		else if(src[i] >= '0' && src[i] <= '9') 
+		else if(src[i] >= '0' && src[i] <= '9')
 			 x = (x << 6) + (unsigned int)(src[i] - '0' + 52);
 		else if(src[i] == '+')
 			x = (x << 6) + 62;
@@ -192,11 +197,11 @@ static int decode_it(const char *src, uint8_t *dst)
 			x = (x << 6);
 		else return 0;
 	}
-	
+
 	dst[2] = (unsigned char)(x & 255); x >>= 8;
 	dst[1] = (unsigned char)(x & 255); x >>= 8;
 	dst[0] = (unsigned char)(x & 255); x >>= 8;
-	
+
 	return 1;
 }
 
@@ -216,7 +221,7 @@ static int base64_decode (const char *src, uint8_t *dst)
 	while (src[length + equals] == '=') equals++;
 
 	num = (length + equals) / 4;
-	
+
 	for (i = 0; i < num - 1; i++) {
 		if (!decode_it(src, dst)) return 0;
 		src += 4;
@@ -235,21 +240,21 @@ static int base64_decode (const char *src, uint8_t *dst)
 /*
  *	Hex or base64 or bin auto-discovery.
  */
-static void normify(VALUE_PAIR *vp, int min_length)
+static void normify(VALUE_PAIR *vp, size_t min_length)
 {
-	int decoded;
-	char buffer[64];
-	
-	if ((size_t) min_length >= sizeof(buffer)) return; /* paranoia */
+	size_t decoded;
+	uint8_t buffer[64];
+
+	if (min_length >= sizeof(buffer)) return; /* paranoia */
 
 	/*
 	 *	Hex encoding.
 	 */
 	if (vp->length >= (2 * min_length)) {
-		decoded = lrad_hex2bin(vp->strvalue, buffer, vp->length >> 1);
+		decoded = fr_hex2bin(vp->vp_strvalue, buffer, vp->length >> 1);
 		if (decoded == (vp->length >> 1)) {
 			DEBUG2("rlm_pap: Normalizing %s from hex encoding", vp->name);
-			memcpy(vp->strvalue, buffer, decoded);
+			memcpy(vp->vp_octets, buffer, decoded);
 			vp->length = decoded;
 			return;
 		}
@@ -260,10 +265,10 @@ static void normify(VALUE_PAIR *vp, int min_length)
 	 *	and we want to avoid division...
 	 */
 	if ((vp->length * 3) >= ((min_length * 4))) {
-		decoded = base64_decode(vp->strvalue, buffer);
+		decoded = base64_decode(vp->vp_strvalue, buffer);
 		if (decoded >= min_length) {
 			DEBUG2("rlm_pap: Normalizing %s from base64 encoding", vp->name);
-			memcpy(vp->strvalue, buffer, decoded);
+			memcpy(vp->vp_octets, buffer, decoded);
 			vp->length = decoded;
 			return;
 		}
@@ -286,23 +291,20 @@ static int pap_authorize(void *instance, REQUEST *request)
 	rlm_pap_t *inst = instance;
 	int auth_type = FALSE;
 	int found_pw = FALSE;
-	int user_pw = FALSE;
 	VALUE_PAIR *vp, *next;
-	VALUE_PAIR *cleartext_pw = NULL;
 
 	for (vp = request->config_items; vp != NULL; vp = next) {
 		next = vp->next;
 
 		switch (vp->attribute) {
 		case PW_USER_PASSWORD: /* deprecated */
-			user_pw = TRUE;
 			found_pw = TRUE;
 
 			/*
 			 *	Look for '{foo}', and use them
 			 */
 			if (!inst->auto_header ||
-			    (vp->strvalue[0] != '{')) {
+			    (vp->vp_strvalue[0] != '{')) {
 				break;
 			}
 			/* FALL-THROUGH */
@@ -310,12 +312,12 @@ static int pap_authorize(void *instance, REQUEST *request)
 		case PW_PASSWORD_WITH_HEADER: /* preferred */
 		{
 			int attr;
-			uint8_t *p, *q;
+			char *p, *q;
 			char buffer[64];
 			VALUE_PAIR *new_vp;
-			
+
 			found_pw = TRUE;
-			q = vp->strvalue;
+			q = vp->vp_strvalue;
 			p = strchr(q + 1, '}');
 			if (!p) {
 				/*
@@ -326,24 +328,23 @@ static int pap_authorize(void *instance, REQUEST *request)
 				 */
 				break;
 			}
-			
+
 			if ((size_t) (p - q) > sizeof(buffer)) break;
-			
+
 			memcpy(buffer, q, p - q + 1);
 			buffer[p - q + 1] = '\0';
-			
-			attr = lrad_str2int(header_names, buffer, 0);
+
+			attr = fr_str2int(header_names, buffer, 0);
 			if (!attr) {
 				DEBUG2("rlm_pap: Found unknown header {%s}: Not doing anything", buffer);
 				break;
 			}
-			
-			new_vp = paircreate(attr, PW_TYPE_STRING);
-			if (!new_vp) break; /* OOM */
-			
-			strcpy(new_vp->strvalue, p + 1);/* bounds OK */
-			new_vp->length = strlen(new_vp->strvalue);
-			pairadd(&request->config_items, new_vp);
+
+			new_vp = radius_paircreate(request,
+						   &request->config_items,
+						   attr, PW_TYPE_STRING);
+			strcpy(new_vp->vp_strvalue, p + 1);/* bounds OK */
+			new_vp->length = strlen(new_vp->vp_strvalue);
 
 			/*
 			 *	May be old-style User-Password with header.
@@ -356,8 +357,6 @@ static int pap_authorize(void *instance, REQUEST *request)
 			break;
 
 		case PW_CLEARTEXT_PASSWORD:
-			cleartext_pw = vp;
-
 		case PW_CRYPT_PASSWORD:
 		case PW_NS_MTA_MD5_PASSWORD:
 			found_pw = TRUE;
@@ -383,9 +382,8 @@ static int pap_authorize(void *instance, REQUEST *request)
 			 */
 		case PW_PROXY_TO_REALM:
 		{
-			REALM *realm = realm_find(vp->strvalue, 0);
-			if (realm &&
-			    (realm->ipaddr != htonl(INADDR_NONE))) {
+			REALM *realm = realm_find(vp->vp_strvalue);
+			if (realm && realm->auth_pool) {
 				return RLM_MODULE_NOOP;
 			}
 			break;
@@ -398,15 +396,15 @@ static int pap_authorize(void *instance, REQUEST *request)
 			 *	Auth-Type := Accept
 			 *	Auth-Type := Reject
 			 */
-			if ((vp->lvalue == 254) ||
-			    (vp->lvalue == 4)) {
+			if ((vp->vp_integer == 254) ||
+			    (vp->vp_integer == 4)) {
 			    found_pw = 1;
 			}
 			break;
 
 		default:
 			break;	/* ignore it */
-			
+
 		}
 	}
 
@@ -414,30 +412,37 @@ static int pap_authorize(void *instance, REQUEST *request)
 	 *	Print helpful warnings if there was no password.
 	 */
 	if (!found_pw) {
+		/*
+		 *	Likely going to be proxied.  Avoid printing
+		 *	warning message.
+		 */
+		if (pairfind(request->config_items, PW_REALM) ||
+		    (pairfind(request->config_items, PW_PROXY_TO_REALM))) {
+			return RLM_MODULE_NOOP;
+		}
+
+		/*
+		 *	The TLS types don't need passwords.
+		 */
+		vp = pairfind(request->packet->vps, PW_EAP_TYPE);
+		if (vp &&
+		    ((vp->vp_integer == 13) || /* EAP-TLS */
+		     (vp->vp_integer == 21) || /* EAP-TTLS */
+		     (vp->vp_integer == 25))) {	/* PEAP */
+			return RLM_MODULE_NOOP;
+		}
+
 		DEBUG("rlm_pap: WARNING! No \"known good\" password found for the user.  Authentication may fail because of this.");
 		return RLM_MODULE_NOOP;
 	}
 
-	/*
-	 *	For backwards compatibility with all of the other
-	 *	modules, copy Cleartext-Password to User-Password
-	 */
-	if (cleartext_pw && !user_pw) {
-		vp = paircreate(PW_USER_PASSWORD, PW_TYPE_STRING);
-		if (!vp) return RLM_MODULE_FAIL;
-
-		memcpy(vp, cleartext_pw, sizeof(*vp));
-		vp->next = NULL;
-		pairadd(&request->config_items, vp);
-	}
-
 	/*
 	 *	Don't touch existing Auth-Types.
 	 */
 	if (auth_type) {
 		DEBUG2("rlm_pap: Found existing Auth-Type, not changing it.");
 		return RLM_MODULE_NOOP;
-	}	
+	}
 
 	/*
 	 *	Can't do PAP if there's no password.
@@ -456,11 +461,11 @@ static int pap_authorize(void *instance, REQUEST *request)
 		return RLM_MODULE_NOOP;
 	}
 
-	vp = paircreate(PW_AUTH_TYPE, PW_TYPE_INTEGER);
-	if (!vp) return RLM_MODULE_FAIL;
-	pairparsevalue(vp, inst->name);
-
-	pairadd(&request->config_items, vp);
+	if (inst->auth_type) {
+		vp = radius_paircreate(request, &request->config_items,
+				       PW_AUTH_TYPE, PW_TYPE_INTEGER);
+		vp->vp_integer = inst->auth_type;
+	}
 
 	return RLM_MODULE_UPDATED;
 }
@@ -475,8 +480,8 @@ static int pap_authenticate(void *instance, REQUEST *request)
 	VALUE_PAIR *vp;
 	VALUE_PAIR *module_fmsg_vp;
 	char module_fmsg[MAX_STRING_LEN];
-	MD5_CTX md5_context;
-	SHA1_CTX sha1_context;
+	FR_MD5_CTX md5_context;
+	fr_SHA1_CTX sha1_context;
 	uint8_t digest[40];
 	char buff[MAX_STRING_LEN];
 	char buff2[MAX_STRING_LEN + 50];
@@ -505,29 +510,29 @@ static int pap_authenticate(void *instance, REQUEST *request)
 		return RLM_MODULE_INVALID;
 	}
 
-	DEBUG("rlm_pap: login attempt with password %s",
-	      request->password->strvalue);
+	DEBUG("rlm_pap: login attempt with password \"%s\"",
+	      request->password->vp_strvalue);
 
 	/*
 	 *	First, auto-detect passwords, by attribute in the
 	 *	config items.
 	 */
-	if ((inst->sch == PAP_ENC_AUTO) || inst->auto_header) {
+	if (inst->sch == PAP_ENC_AUTO) {
 		for (vp = request->config_items; vp != NULL; vp = vp->next) {
 			switch (vp->attribute) {
 			case PW_USER_PASSWORD: /* deprecated */
 			case PW_CLEARTEXT_PASSWORD: /* preferred */
 				goto do_clear;
-				
+
 			case PW_CRYPT_PASSWORD:
 				goto do_crypt;
-				
+
 			case PW_MD5_PASSWORD:
 				goto do_md5;
-				
+
 			case PW_SHA_PASSWORD:
 				goto do_sha;
-				
+
 			case PW_NT_PASSWORD:
 				goto do_nt;
 
@@ -545,7 +550,7 @@ static int pap_authenticate(void *instance, REQUEST *request)
 
 			default:
 				break;	/* ignore it */
-				
+
 			}
 		}
 
@@ -555,7 +560,7 @@ static int pap_authenticate(void *instance, REQUEST *request)
 
 	} else {
 		vp = NULL;
-		
+
 		if (inst->sch == PAP_ENC_CRYPT) {
 			vp = pairfind(request->config_items, PW_CRYPT_PASSWORD);
 		}
@@ -567,7 +572,6 @@ static int pap_authenticate(void *instance, REQUEST *request)
 			vp = pairfind(request->config_items, PW_USER_PASSWORD);
 			if (!vp) goto fail;
 		}
-		scheme = inst->sch;
 	}
 
 	/*
@@ -576,9 +580,10 @@ static int pap_authenticate(void *instance, REQUEST *request)
 	switch (scheme) {
 	case PAP_ENC_CLEAR:
 	do_clear:
-		DEBUG("rlm_pap: Using clear text password \"%s\".", vp->strvalue);
-		if (strcmp((char *) vp->strvalue,
-			   (char *) request->password->strvalue) != 0){
+		DEBUG("rlm_pap: Using clear text password \"%s\"",
+		      vp->vp_strvalue);
+		if (strcmp((char *) vp->vp_strvalue,
+			   (char *) request->password->vp_strvalue) != 0){
 			snprintf(module_fmsg,sizeof(module_fmsg),"rlm_pap: CLEAR TEXT password check failed");
 			goto make_msg;
 		}
@@ -586,18 +591,18 @@ static int pap_authenticate(void *instance, REQUEST *request)
 		DEBUG("rlm_pap: User authenticated successfully");
 		return RLM_MODULE_OK;
 		break;
-		
+
 	case PAP_ENC_CRYPT:
 	do_crypt:
 		DEBUG("rlm_pap: Using CRYPT encryption.");
-		if (lrad_crypt_check((char *) request->password->strvalue,
-				     (char *) vp->strvalue) != 0) {
+		if (fr_crypt_check((char *) request->password->vp_strvalue,
+				     (char *) vp->vp_strvalue) != 0) {
 			snprintf(module_fmsg,sizeof(module_fmsg),"rlm_pap: CRYPT password check failed");
 			goto make_msg;
 		}
 		goto done;
 		break;
-		
+
 	case PW_MD5_PASSWORD:
 	do_md5:
 		DEBUG("rlm_pap: Using MD5 encryption.");
@@ -608,18 +613,18 @@ static int pap_authenticate(void *instance, REQUEST *request)
 			snprintf(module_fmsg,sizeof(module_fmsg),"rlm_pap: Configured MD5 password has incorrect length");
 			goto make_msg;
 		}
-		
-		MD5Init(&md5_context);
-		MD5Update(&md5_context, request->password->strvalue,
-			  request->password->length);
-		MD5Final(digest, &md5_context);
-		if (memcmp(digest, vp->strvalue, vp->length) != 0) {
+
+		fr_MD5Init(&md5_context);
+		fr_MD5Update(&md5_context, request->password->vp_octets,
+			     request->password->length);
+		fr_MD5Final(digest, &md5_context);
+		if (memcmp(digest, vp->vp_octets, vp->length) != 0) {
 			snprintf(module_fmsg,sizeof(module_fmsg),"rlm_pap: MD5 password check failed");
 			goto make_msg;
 		}
 		goto done;
 		break;
-		
+
 	case PW_SMD5_PASSWORD:
 	do_smd5:
 		DEBUG("rlm_pap: Using SMD5 encryption.");
@@ -630,49 +635,49 @@ static int pap_authenticate(void *instance, REQUEST *request)
 			snprintf(module_fmsg,sizeof(module_fmsg),"rlm_pap: Configured SMD5 password has incorrect length");
 			goto make_msg;
 		}
-		
-		MD5Init(&md5_context);
-		MD5Update(&md5_context, request->password->strvalue,
-			  request->password->length);
-		MD5Update(&md5_context, &vp->strvalue[16], vp->length - 16);
-		MD5Final(digest, &md5_context);
+
+		fr_MD5Init(&md5_context);
+		fr_MD5Update(&md5_context, request->password->vp_octets,
+			     request->password->length);
+		fr_MD5Update(&md5_context, &vp->vp_octets[16], vp->length - 16);
+		fr_MD5Final(digest, &md5_context);
 
 		/*
 		 *	Compare only the MD5 hash results, not the salt.
 		 */
-		if (memcmp(digest, vp->strvalue, 16) != 0) {
+		if (memcmp(digest, vp->vp_octets, 16) != 0) {
 			snprintf(module_fmsg,sizeof(module_fmsg),"rlm_pap: SMD5 password check failed");
 			goto make_msg;
 		}
 		goto done;
 		break;
-		
+
 	case PW_SHA_PASSWORD:
 	do_sha:
 		DEBUG("rlm_pap: Using SHA1 encryption.");
-		
+
 		normify(vp, 20);
 		if (vp->length != 20) {
 			DEBUG("rlm_pap: Configured SHA1 password has incorrect length");
 			snprintf(module_fmsg,sizeof(module_fmsg),"rlm_pap: Configured SHA1 password has incorrect length");
 			goto make_msg;
 		}
-		
-		SHA1Init(&sha1_context);
-		SHA1Update(&sha1_context, request->password->strvalue,
-			   request->password->length);
-		SHA1Final(digest,&sha1_context);
-		if (memcmp(digest, vp->strvalue, vp->length) != 0) {
+
+		fr_SHA1Init(&sha1_context);
+		fr_SHA1Update(&sha1_context, request->password->vp_octets,
+			      request->password->length);
+		fr_SHA1Final(digest,&sha1_context);
+		if (memcmp(digest, vp->vp_octets, vp->length) != 0) {
 			snprintf(module_fmsg,sizeof(module_fmsg),"rlm_pap: SHA1 password check failed");
 			goto make_msg;
 		}
 		goto done;
 		break;
-		
+
 	case PW_SSHA_PASSWORD:
 	do_ssha:
 		DEBUG("rlm_pap: Using SSHA encryption.");
-		
+
 		normify(vp, 20);
 		if (vp->length <= 20) {
 			DEBUG("rlm_pap: Configured SSHA password has incorrect length");
@@ -680,19 +685,19 @@ static int pap_authenticate(void *instance, REQUEST *request)
 			goto make_msg;
 		}
 
-		
-		SHA1Init(&sha1_context);
-		SHA1Update(&sha1_context, request->password->strvalue,
+
+		fr_SHA1Init(&sha1_context);
+		fr_SHA1Update(&sha1_context, request->password->vp_octets,
 			   request->password->length);
-		SHA1Update(&sha1_context, &vp->strvalue[20], vp->length - 20);
-		SHA1Final(digest,&sha1_context);
-		if (memcmp(digest, vp->strvalue, 20) != 0) {
+		fr_SHA1Update(&sha1_context, &vp->vp_octets[20], vp->length - 20);
+		fr_SHA1Final(digest,&sha1_context);
+		if (memcmp(digest, vp->vp_octets, 20) != 0) {
 			snprintf(module_fmsg,sizeof(module_fmsg),"rlm_pap: SSHA password check failed");
 			goto make_msg;
 		}
 		goto done;
 		break;
-		
+
 	case PW_NT_PASSWORD:
 	do_nt:
 		DEBUG("rlm_pap: Using NT encryption.");
@@ -703,41 +708,41 @@ static int pap_authenticate(void *instance, REQUEST *request)
 			snprintf(module_fmsg,sizeof(module_fmsg),"rlm_pap: Configured NT-Password has incorrect length");
 			goto make_msg;
 		}
-		
-		sprintf(buff2,"%%{mschap:NT-Hash %s}",
-			request->password->strvalue);
-		if (!radius_xlat(digest,sizeof(digest),buff2,request,NULL)){
+
+		snprintf(buff2, sizeof(buff2), "%%{mschap:NT-Hash %s}",
+			request->password->vp_strvalue);
+		if (!radius_xlat(digest, sizeof(digest),buff2,request,NULL)){
 			DEBUG("rlm_pap: mschap xlat failed");
 			snprintf(module_fmsg,sizeof(module_fmsg),"rlm_pap: mschap xlat failed");
 			goto make_msg;
 		}
-		if ((lrad_hex2bin(digest, digest, 16) != vp->length) ||
-		    (memcmp(digest, vp->strvalue, vp->length) != 0)) {
+		if ((fr_hex2bin(digest, digest, 16) != vp->length) ||
+		    (memcmp(digest, vp->vp_octets, vp->length) != 0)) {
 			snprintf(module_fmsg,sizeof(module_fmsg),"rlm_pap: NT password check failed");
 			goto make_msg;
 		}
 		goto done;
 		break;
-		
+
 	case PW_LM_PASSWORD:
 	do_lm:
 		DEBUG("rlm_pap: Using LM encryption.");
-		
+
 		normify(vp, 16);
 		if (vp->length != 16) {
 			DEBUG("rlm_pap: Configured LM-Password has incorrect length");
 			snprintf(module_fmsg,sizeof(module_fmsg),"rlm_pap: Configured LM-Password has incorrect length");
 			goto make_msg;
 		}
-		sprintf(buff2,"%%{mschap:LM-Hash %s}",
-			request->password->strvalue);
+		snprintf(buff2, sizeof(buff2), "%%{mschap:LM-Hash %s}",
+			request->password->vp_strvalue);
 		if (!radius_xlat(digest,sizeof(digest),buff2,request,NULL)){
 			DEBUG("rlm_pap: mschap xlat failed");
 			snprintf(module_fmsg,sizeof(module_fmsg),"rlm_pap: mschap xlat failed");
 			goto make_msg;
 		}
-		if ((lrad_hex2bin(digest, digest, 16) != vp->length) ||
-		    (memcmp(digest, vp->strvalue, vp->length) != 0)) {
+		if ((fr_hex2bin(digest, digest, 16) != vp->length) ||
+		    (memcmp(digest, vp->vp_octets, vp->length) != 0)) {
 			snprintf(module_fmsg,sizeof(module_fmsg),"rlm_pap: LM password check failed");
 		make_msg:
 			DEBUG("rlm_pap: Passwords don't match");
@@ -762,7 +767,7 @@ static int pap_authenticate(void *instance, REQUEST *request)
 		/*
 		 *	Sanity check the value of NS-MTA-MD5-Password
 		 */
-		if (lrad_hex2bin(vp->strvalue, buff, 32) != 16) {
+		if (fr_hex2bin(vp->vp_strvalue, buff, 32) != 16) {
 			DEBUG("rlm_pap: Configured NS-MTA-MD5-Password has invalid value");
 			snprintf(module_fmsg,sizeof(module_fmsg),"rlm_pap: Configured NS-MTA-MD5-Password has invalid value");
 			goto make_msg;
@@ -773,7 +778,7 @@ static int pap_authenticate(void *instance, REQUEST *request)
 		 *
 		 *	This really: sizeof(buff) - 2 - 2*32 - strlen(passwd)
 		 */
-		if (strlen(request->password->strvalue) >= (sizeof(buff2) - 2 - 2 * 32)) {
+		if (strlen(request->password->vp_strvalue) >= (sizeof(buff2) - 2 - 2 * 32)) {
 			DEBUG("rlm_pap: Configured password is too long");
 			snprintf(module_fmsg,sizeof(module_fmsg),"rlm_pap: password is too long");
 			goto make_msg;
@@ -785,18 +790,19 @@ static int pap_authenticate(void *instance, REQUEST *request)
 		{
 			char *p = buff2;
 
-			memcpy(p, &vp->strvalue[32], 32);
+			memcpy(p, &vp->vp_octets[32], 32);
 			p += 32;
 			*(p++) = 89;
-			strcpy(p, request->password->strvalue);
+			strcpy(p, request->password->vp_strvalue);
 			p += strlen(p);
 			*(p++) = 247;
-			memcpy(p, &vp->strvalue[32], 32);
+			memcpy(p, &vp->vp_octets[32], 32);
 			p += 32;
 
-			MD5Init(&md5_context);
-			MD5Update(&md5_context, buff2, p - buff2);
-			MD5Final(digest, &md5_context);
+			fr_MD5Init(&md5_context);
+			fr_MD5Update(&md5_context, (uint8_t *) buff2,
+				     p - buff2);
+			fr_MD5Final(digest, &md5_context);
 		}
 		if (memcmp(digest, buff, 16) != 0) {
 			snprintf(module_fmsg,sizeof(module_fmsg),"rlm_pap: NS-MTA-MD5 password check failed");
@@ -823,13 +829,14 @@ static int pap_authenticate(void *instance, REQUEST *request)
  *	is single-threaded.
  */
 module_t rlm_pap = {
+	RLM_MODULE_INIT,
 	"PAP",
-	0,				/* type */
-	NULL,				/* initialization */
+	RLM_TYPE_CHECK_CONFIG_SAFE | RLM_TYPE_HUP_SAFE,   	/* type */
 	pap_instantiate,		/* instantiation */
+	pap_detach,			/* detach */
 	{
 		pap_authenticate,	/* authentication */
-		pap_authorize,	       	/* authorization */
+		pap_authorize,		/* authorization */
 		NULL,			/* preaccounting */
 		NULL,			/* accounting */
 		NULL,			/* checksimul */
@@ -837,6 +844,4 @@ module_t rlm_pap = {
 		NULL,			/* post-proxy */
 		NULL			/* post-auth */
 	},
-	pap_detach,			/* detach */
-	NULL,				/* destroy */
 };
