@@ -1,7 +1,7 @@
-/*
+ /*
  * rlm_perl.c
  *
- * Version:    $Id$
+ * Version:    $Id: rlm_perl.c,v 1.53 2008/01/06 03:03:09 nbk Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -15,22 +15,17 @@
  *
  *   You should have received a copy of the GNU General Public License
  *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
  *
- * Copyright 2002  The FreeRADIUS server project
+ * Copyright 2002,2006  The FreeRADIUS server project
  * Copyright 2002  Boian Jordanov <bjordanov@orbitel.bg>
  */
 
-#include "autoconf.h"
-#include "libradius.h"
+#include <freeradius-devel/ident.h>
+RCSID("$Id: rlm_perl.c,v 1.53 2008/01/06 03:03:09 nbk Exp $")
 
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
-
-#include "radiusd.h"
-#include "modules.h"
-#include "conffile.h"
+#include <freeradius-devel/radiusd.h>
+#include <freeradius-devel/modules.h>
 
 #ifdef DEBUG
 #undef DEBUG
@@ -50,8 +45,6 @@
 extern char **environ;
 #endif
 
-static const char rcsid[] = "$Id$";
-
 #ifdef USE_ITHREADS
 
 /*
@@ -127,7 +120,7 @@ typedef struct perl_inst {
  *	buffer over-flows.
  */
 static const CONF_PARSER module_config[] = {
-	{ "module",  PW_TYPE_STRING_PTR,
+	{ "module",  PW_TYPE_FILENAME,
 	  offsetof(PERL_INST,module), NULL,  "module"},
 	{ "func_authorize", PW_TYPE_STRING_PTR,
 	  offsetof(PERL_INST,func_authorize), NULL, "authorize"},
@@ -170,13 +163,13 @@ EXTERN_C void boot_DynaLoader(pTHX_ CV* cv);
  *	We clone it for every instance if we have perl
  *	with -Duseithreads compiled in
  */
-static PerlInterpreter	*interp;
+static PerlInterpreter	*interp = NULL;
 
 static const CONF_PARSER pool_conf[] = {
 	{ "max_clones", PW_TYPE_INTEGER, offsetof(PERL_POOL, max_clones), NULL,		"32"},
-	{ "start_clones",PW_TYPE_INTEGER, offsetof(PERL_POOL, start_clones), NULL,		"5"},
-	{ "min_spare_clones",PW_TYPE_INTEGER, offsetof(PERL_POOL, min_spare_clones),NULL,	"3"},
-	{ "max_spare_clones",PW_TYPE_INTEGER, offsetof(PERL_POOL,max_spare_clones),NULL,	"3"},
+	{ "start_clones",PW_TYPE_INTEGER, offsetof(PERL_POOL, start_clones), NULL,		"32"},
+	{ "min_spare_clones",PW_TYPE_INTEGER, offsetof(PERL_POOL, min_spare_clones),NULL,	"0"},
+	{ "max_spare_clones",PW_TYPE_INTEGER, offsetof(PERL_POOL,max_spare_clones),NULL,	"32"},
 	{ "cleanup_delay",PW_TYPE_INTEGER, offsetof(PERL_POOL,cleanup_delay),NULL,		"5"},
 	{ "max_request_per_clone",PW_TYPE_INTEGER, offsetof(PERL_POOL,max_request_per_clone),NULL,	"0"},
 	{ NULL, -1, 0, NULL, NULL }		/* end the list */
@@ -257,7 +250,7 @@ static void rlm_perl_close_handles(void **handles)
 static PerlInterpreter *rlm_perl_clone(PerlInterpreter *perl)
 {
 	PerlInterpreter *clone;
-	UV	clone_flags = 0;
+	UV clone_flags = 0;
 
 	PERL_SET_CONTEXT(perl);
 
@@ -554,6 +547,7 @@ static int init_pool (CONF_SECTION *conf, PERL_INST *inst) {
 	int t;
 	PERL_POOL	*pool;
 
+
 	pool = rad_malloc(sizeof(PERL_POOL));
 	memset(pool,0,sizeof(PERL_POOL));
 
@@ -580,20 +574,6 @@ static int init_pool (CONF_SECTION *conf, PERL_INST *inst) {
 	return 1;
 }
 #endif
-/*
- *	Do any per-module initialization.  e.g. set up connections
- *	to external databases, read configuration files, set up
- *	dictionary entries, etc.
- *
- *	Try to avoid putting too much stuff in here - it's better to
- *	do it in instantiate() where it is not global.
- *	I use one global interpetator to make things more fastest for
- *	Threading env I clone new perl from this interp.
- */
-static int perl_init(void)
-{
-	return 0;
-}
 
 static void xs_init(pTHX)
 {
@@ -634,14 +614,15 @@ static XS(XS_radiusd_radlog)
 /*
  * The xlat function
  */
-static int perl_xlat(void *instance, REQUEST *request, char *fmt, char * out,
-		     size_t freespace, RADIUS_ESCAPE_STRING func)
+static size_t perl_xlat(void *instance, REQUEST *request, char *fmt, char *out,
+			size_t freespace, RADIUS_ESCAPE_STRING func)
 {
 
 	PERL_INST	*inst= (PERL_INST *) instance;
 	PerlInterpreter *perl;
 	char		params[1024], *ptr, *tmp;
-	int		count, ret=0;
+	int		count;
+	size_t		ret = 0;
 	STRLEN		n_a;
 
 	/*
@@ -693,7 +674,7 @@ static int perl_xlat(void *instance, REQUEST *request, char *fmt, char * out,
 		POPs ;
 	} else if (count > 0) {
 		tmp = POPp;
-		strNcpy(out,tmp,freespace);
+		strlcpy(out, tmp, freespace);
 		ret = strlen(out);
 
 		radlog(L_DBG,"rlm_perl: Len is %d , out is %s freespace is %d",
@@ -730,12 +711,14 @@ static int perl_instantiate(CONF_SECTION *conf, void **instance)
 	PERL_INST       *inst = (PERL_INST *) instance;
 	HV		*rad_reply_hv;
 	HV		*rad_check_hv;
+	HV		*rad_config_hv;
 	HV		*rad_request_hv;
 	HV		*rad_request_proxy_hv;
 	HV		*rad_request_proxy_reply_hv;
 	AV		*end_AV;
 
-	char *embed[4], *xlat_name;
+	char *embed[4];
+	const char *xlat_name;
 	int exitstatus = 0, argc=0;
 
 	/*
@@ -768,7 +751,7 @@ static int perl_instantiate(CONF_SECTION *conf, void **instance)
 
 #ifdef USE_ITHREADS
 	inst->perl = interp;
-	
+
 	if ((inst->perl = perl_alloc()) == NULL) {
 		radlog(L_DBG, "rlm_perl: No memory for allocating new perl !");
 		return (-1);
@@ -812,12 +795,14 @@ static int perl_instantiate(CONF_SECTION *conf, void **instance)
 
 	rad_reply_hv = newHV();
 	rad_check_hv = newHV();
+	rad_config_hv = newHV();
 	rad_request_hv = newHV();
 	rad_request_proxy_hv = newHV();
 	rad_request_proxy_reply_hv = newHV();
 
 	rad_reply_hv = get_hv("RAD_REPLY",1);
         rad_check_hv = get_hv("RAD_CHECK",1);
+	rad_config_hv = get_hv("RAD_CONFIG",1);
         rad_request_hv = get_hv("RAD_REQUEST",1);
 	rad_request_proxy_hv = get_hv("RAD_REQUEST_PROXY",1);
 	rad_request_proxy_reply_hv = get_hv("RAD_REQUEST_PROXY_REPLY",1);
@@ -926,6 +911,7 @@ static int get_hv_content(HV *my_hv, VALUE_PAIR **vp)
        I32		key_len, len, i, j;
        int		ret=0;
 
+       *vp = NULL;
        for (i = hv_iterinit(my_hv); i > 0; i--) {
                res_sv = hv_iternextsv(my_hv,&key,&key_len);
                if (SvROK(res_sv) && (SvTYPE(SvRV(res_sv)) == SVt_PVAV)) {
@@ -956,6 +942,7 @@ static int rlmperl_call(void *instance, REQUEST *request, char *function_name)
 
 	HV		*rad_reply_hv;
 	HV		*rad_check_hv;
+	HV		*rad_config_hv;
 	HV		*rad_request_hv;
 	HV		*rad_request_proxy_hv;
 	HV		*rad_request_proxy_reply_hv;
@@ -993,6 +980,7 @@ static int rlmperl_call(void *instance, REQUEST *request, char *function_name)
 
 	rad_reply_hv = get_hv("RAD_REPLY",1);
 	rad_check_hv = get_hv("RAD_CHECK",1);
+	rad_config_hv = get_hv("RAD_CONFIG",1);
 	rad_request_hv = get_hv("RAD_REQUEST",1);
 	rad_request_proxy_hv = get_hv("RAD_REQUEST_PROXY",1);
 	rad_request_proxy_reply_hv = get_hv("RAD_REQUEST_PROXY_REPLY",1);
@@ -1001,7 +989,8 @@ static int rlmperl_call(void *instance, REQUEST *request, char *function_name)
 	perl_store_vps(request->reply->vps, rad_reply_hv);
 	perl_store_vps(request->config_items, rad_check_hv);
 	perl_store_vps(request->packet->vps, rad_request_hv);
-	
+	perl_store_vps(request->config_items, rad_config_hv);
+
 	if (request->proxy != NULL) {
 		perl_store_vps(request->proxy->vps, rad_request_proxy_hv);
 	} else {
@@ -1012,10 +1001,7 @@ static int rlmperl_call(void *instance, REQUEST *request, char *function_name)
 		perl_store_vps(request->proxy_reply->vps, rad_request_proxy_reply_hv);
 	} else {
 		hv_undef(rad_request_proxy_reply_hv);
-	}	
-	
-	vp = NULL;
-
+	}
 
 	PUSHMARK(SP);
 	/*
@@ -1043,28 +1029,56 @@ static int rlmperl_call(void *instance, REQUEST *request, char *function_name)
 			exitstatus = RLM_MODULE_FAIL;
 		}
 	}
-	
+
 
 	PUTBACK;
 	FREETMPS;
 	LEAVE;
 
+	vp = NULL;
+	if ((get_hv_content(rad_request_hv, &vp)) > 0 ) {
+		pairfree(&request->packet->vps);
+		request->packet->vps = vp;
+		vp = NULL;
+
+		/*
+		 *	Update cached copies
+		 */
+		request->username = pairfind(request->packet->vps,
+					     PW_USER_NAME);
+		request->password = pairfind(request->packet->vps,
+					     PW_USER_PASSWORD);
+		if (!request->password)
+			request->password = pairfind(request->packet->vps,
+						     PW_CHAP_PASSWORD);
+	}
 
 	if ((get_hv_content(rad_reply_hv, &vp)) > 0 ) {
-		pairmove(&request->reply->vps, &vp);
-		pairfree(&vp);
+		pairfree(&request->reply->vps);
+		request->reply->vps = vp;
+		vp = NULL;
 	}
 
 	if ((get_hv_content(rad_check_hv, &vp)) > 0 ) {
-		pairmove(&request->config_items, &vp);
-		pairfree(&vp);
+		pairfree(&request->config_items);
+		request->config_items = vp;
+		vp = NULL;
 	}
-	
-	if ((get_hv_content(rad_request_proxy_reply_hv, &vp)) > 0 && request->proxy_reply != NULL) {
+
+	if (request->proxy &&
+	    (get_hv_content(rad_request_proxy_hv, &vp) > 0)) {
+		pairfree(&request->proxy->vps);
+		request->proxy->vps = vp;
+		vp = NULL;
+	}
+
+	if (request->proxy_reply &&
+	    (get_hv_content(rad_request_proxy_reply_hv, &vp) > 0)) {
 		pairfree(&request->proxy_reply->vps);
-		pairmove(&request->proxy_reply->vps, &vp);
-		pairfree(&vp);
+		request->proxy_reply->vps = vp;
+		vp = NULL;
 	}
+
 	}
 #ifdef USE_ITHREADS
 	pool_release(handle,instance);
@@ -1111,7 +1125,7 @@ static int perl_accounting(void *instance, REQUEST *request)
 	int 		acctstatustype=0;
 
 	if ((pair = pairfind(request->packet->vps, PW_ACCT_STATUS_TYPE)) != NULL) {
-                acctstatustype = pair->lvalue;
+		acctstatustype = pair->vp_integer;
         } else {
                 radlog(L_ERR, "Invalid Accounting Packet");
                 return RLM_MODULE_INVALID;
@@ -1184,7 +1198,7 @@ static int perl_post_auth(void *instance, REQUEST *request)
 static int perl_detach(void *instance)
 {
 	PERL_INST	*inst = (PERL_INST *) instance;
-	int 		exitstatus=0,count=0;
+	int 		exitstatus = 0, count = 0;
 
 #ifdef USE_ITHREADS
 	POOL_HANDLE	*handle, *tmp, *tmp2;
@@ -1270,15 +1284,6 @@ static int perl_detach(void *instance)
 	xlat_unregister(inst->xlat_name, perl_xlat);
 	free(inst->xlat_name);
 
-	if (inst->func_authorize) free(inst->func_authorize);
-	if (inst->func_authenticate) free(inst->func_authenticate);
-	if (inst->func_accounting) free(inst->func_accounting);
-	if (inst->func_preacct) free(inst->func_preacct);
-	if (inst->func_checksimul) free(inst->func_checksimul);
-	if (inst->func_pre_proxy) free(inst->func_pre_proxy);
-	if (inst->func_post_proxy) free(inst->func_post_proxy);
-	if (inst->func_post_auth) free(inst->func_post_auth);
-	if (inst->func_detach) free(inst->func_detach);
 
 #ifdef USE_ITHREADS
 	free(inst->perl_pool->head);
@@ -1304,24 +1309,23 @@ static int perl_detach(void *instance)
  *	is single-threaded.
  */
 module_t rlm_perl = {
+	RLM_MODULE_INIT,
 	"perl",				/* Name */
 #ifdef USE_ITHREADS
 	RLM_TYPE_THREAD_SAFE,		/* type */
 #else
 	RLM_TYPE_THREAD_UNSAFE,
 #endif
-	perl_init,			/* initialization */
 	perl_instantiate,		/* instantiation */
-	{
-		perl_authenticate,
-		perl_authorize,
-		perl_preacct,
-		perl_accounting,
-		perl_checksimul,      	/* check simul */
-		perl_pre_proxy,	 /* pre-proxy */
-		perl_post_proxy,	/* post-proxy */
-		perl_post_auth	  /* post-auth */
-	},
 	perl_detach,			/* detach */
-	NULL,				/* destroy */
+	{
+		perl_authenticate,	/* authenticate */
+		perl_authorize,		/* authorize */
+		perl_preacct,		/* preacct */
+		perl_accounting,	/* accounting */
+		perl_checksimul,      	/* check simul */
+		perl_pre_proxy,		/* pre-proxy */
+		perl_post_proxy,	/* post-proxy */
+		perl_post_auth		/* post-auth */
+	},
 };
