@@ -1,7 +1,7 @@
 /*
  * rlm_realm.c
  *
- * Version:	$Id$
+ * Version:	$Id: rlm_realm.c,v 1.72 2008/04/17 08:11:23 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -15,31 +15,17 @@
  *
  *   You should have received a copy of the GNU General Public License
  *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
  *
- * Copyright 2000  The FreeRADIUS server project
+ * Copyright 2000,2006  The FreeRADIUS server project
  * FIXME add copyrights
  */
 
-#include "autoconf.h"
+#include <freeradius-devel/ident.h>
+RCSID("$Id: rlm_realm.c,v 1.72 2008/04/17 08:11:23 aland Exp $")
 
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
-
-#ifdef HAVE_SYS_TYPES_H
-#include <sys/types.h>
-#endif
-
-#ifdef HAVE_NETINET_IN_H
-#include <netinet/in.h>
-#endif
-
-#include "libradius.h"
-#include "radiusd.h"
-#include "modules.h"
-
-static const char rcsid[] = "$Id$";
+#include <freeradius-devel/radiusd.h>
+#include <freeradius-devel/modules.h>
 
 #define  REALM_FORMAT_PREFIX   0
 #define  REALM_FORMAT_SUFFIX   1
@@ -114,7 +100,7 @@ static int check_for_realm(void *instance, REQUEST *request, REALM **returnrealm
 	 *	We will be modifing this later, so we want our own copy
 	 *	of it.
 	 */
-	strNcpy(namebuf, (char *)request->username->strvalue, sizeof(namebuf));
+	strlcpy(namebuf, (char *)request->username->vp_strvalue, sizeof(namebuf));
 	username = namebuf;
 
 	switch(inst->format)
@@ -155,34 +141,33 @@ static int check_for_realm(void *instance, REQUEST *request, REALM **returnrealm
 	 */
 	if (realmname) {
 		DEBUG2("    rlm_realm: Looking up realm \"%s\" for User-Name = \"%s\"",
-		       realmname, request->username->strvalue);
+		       realmname, request->username->vp_strvalue);
 	} else {
 		if( inst->ignore_null ) {
 			DEBUG2("    rlm_realm: No '%c' in User-Name = \"%s\", skipping NULL due to config.",
-			inst->delim[0], request->username->strvalue);
+			inst->delim[0], request->username->vp_strvalue);
 			return 0;
 		}
 		DEBUG2("    rlm_realm: No '%c' in User-Name = \"%s\", looking up realm NULL",
-		       inst->delim[0], request->username->strvalue);
+		       inst->delim[0], request->username->vp_strvalue);
 	}
 
 	/*
 	 *	Allow DEFAULT realms unless told not to.
 	 */
-	realm = realm_find(realmname, (request->packet->code == PW_ACCOUNTING_REQUEST));
+	realm = realm_find(realmname);
 	if (!realm) {
 		DEBUG2("    rlm_realm: No such realm \"%s\"",
 		       (realmname == NULL) ? "NULL" : realmname);
 		return 0;
 	}
 	if( inst->ignore_default &&
-	    (strcmp(realm->realm, "DEFAULT")) == 0) {
+	    (strcmp(realm->name, "DEFAULT")) == 0) {
 		DEBUG2("    rlm_realm: Found DEFAULT, but skipping due to config.");
 		return 0;
 	}
 
-
-	DEBUG2("    rlm_realm: Found realm \"%s\"", realm->realm);
+	DEBUG2("    rlm_realm: Found realm \"%s\"", realm->name);
 
 	/*
 	 *	If we've been told to strip the realm off, then do so.
@@ -194,32 +179,26 @@ static int check_for_realm(void *instance, REQUEST *request, REALM **returnrealm
 		 *
 		 */
 		if (request->username->attribute != PW_STRIPPED_USER_NAME) {
-			vp = paircreate(PW_STRIPPED_USER_NAME, PW_TYPE_STRING);
-			if (!vp) {
-				radlog(L_ERR|L_CONS, "no memory");
-				return -1;
-			}
-			pairadd(&request->packet->vps, vp);
+			vp = radius_paircreate(request, &request->packet->vps,
+					       PW_STRIPPED_USER_NAME,
+					       PW_TYPE_STRING);
 			DEBUG2("    rlm_realm: Adding Stripped-User-Name = \"%s\"", username);
 		} else {
 			vp = request->username;
 			DEBUG2("    rlm_realm: Setting Stripped-User-Name = \"%s\"", username);
 		}
 
-		strcpy(vp->strvalue, username);
-		vp->length = strlen((char *)vp->strvalue);
+		strcpy(vp->vp_strvalue, username);
+		vp->length = strlen((char *)vp->vp_strvalue);
 		request->username = vp;
 	}
 
-	DEBUG2("    rlm_realm: Proxying request from user %s to realm %s",
-	       username, realm->realm);
-
 	/*
 	 *	Add the realm name to the request.
 	 */
-	pairadd(&request->packet->vps, pairmake("Realm", realm->realm,
+	pairadd(&request->packet->vps, pairmake("Realm", realm->name,
 						T_OP_EQ));
-	DEBUG2("    rlm_realm: Adding Realm = \"%s\"", realm->realm);
+	DEBUG2("    rlm_realm: Adding Realm = \"%s\"", realm->name);
 
 	/*
 	 *	Figure out what to do with the request.
@@ -234,52 +213,97 @@ static int check_for_realm(void *instance, REQUEST *request, REALM **returnrealm
 		 *	Perhaps accounting proxying was turned off.
 		 */
 	case PW_ACCOUNTING_REQUEST:
-		if (realm->acct_ipaddr == htonl(INADDR_NONE)) {
+		if (!realm->acct_pool) {
 			DEBUG2("    rlm_realm: Accounting realm is LOCAL.");
 			return 0;
 		}
-
-		if (realm->acct_port == 0) {
-			DEBUG2("    rlm_realm: acct_port is not set.  Proxy cancelled.");
-			return 0;
-		}
 		break;
 
 		/*
 		 *	Perhaps authentication proxying was turned off.
 		 */
 	case PW_AUTHENTICATION_REQUEST:
-		if (realm->ipaddr == htonl(INADDR_NONE)) {
+		if (!realm->auth_pool) {
 			DEBUG2("    rlm_realm: Authentication realm is LOCAL.");
 			return 0;
 		}
-
-		if (realm->auth_port == 0) {
-			DEBUG2("    rlm_realm: auth_port is not set.  Proxy cancelled.");
-			return 0;
-		}
 		break;
 	}
 
+	DEBUG2("    rlm_realm: Proxying request from user %s to realm %s",
+	       username, realm->name);
+
+	/*
+	 *	Skip additional checks if it's not an accounting
+	 *	request.
+	 */
+	if (request->packet->code != PW_ACCOUNTING_REQUEST) {
+		*returnrealm = realm;
+		return 0;
+	}
+
+	/*
+	 *	FIXME: Each server should have a unique server key,
+	 *	and put it in the accounting packet.  Every server
+	 *	should know about the keys, and NOT proxy requests to
+	 *	a server with key X if the packet already contains key
+	 *	X.
+	 */
+
 	/*
 	 *      If this request has arrived from another freeradius server
 	 *      that has already proxied the request, we don't need to do
 	 *      it again.
 	 */
-	for (vp = request->packet->vps; vp; vp = vp->next) {
-		if (vp->attribute == PW_FREERADIUS_PROXIED_TO) {
-			if (request->packet->code == PW_AUTHENTICATION_REQUEST &&
-			    vp->lvalue == realm->ipaddr) {
-				DEBUG2("    rlm_realm: Request not proxied due to Freeradius-Proxied-To");
-				return 0;
-			}
-			if (request->packet->code == PW_ACCOUNTING_REQUEST &&
-			    vp->lvalue == realm->acct_ipaddr) {
-				DEBUG2("    rlm_realm: Request not proxied due to Freeradius-Proxied-To");
+	vp = pairfind(request->packet->vps, PW_FREERADIUS_PROXIED_TO);
+	if (vp && (request->packet->src_ipaddr.af == AF_INET)) {
+		int i;
+		fr_ipaddr_t my_ipaddr;
+
+		my_ipaddr.af = AF_INET;
+		my_ipaddr.ipaddr.ip4addr.s_addr = vp->vp_ipaddr;
+
+		/*
+		 *	Loop over the home accounting servers for this
+		 *	realm.  If one of them has the same IP as the
+		 *	FreeRADIUS-Proxied-To attribute, then the
+		 *	packet has already been sent there.  Don't
+		 *	send it there again.
+		 */
+		for (i = 0; i < realm->acct_pool->num_home_servers; i++) {
+			if (fr_ipaddr_cmp(&realm->acct_pool->servers[i]->ipaddr,
+					    &my_ipaddr) == 0) {
+				DEBUG2("Suppressing proxy due to FreeRADIUS-Proxied-To");
 				return 0;
 			}
 		}
-        }
+
+		/*
+		 *	See detail_recv() in src/main/listen.c for the
+		 *	additional checks.
+		 */
+	} else if ((request->listener->type == RAD_LISTEN_DETAIL) &&
+		   ((request->packet->src_ipaddr.af == AF_INET6) ||
+		    (request->packet->src_ipaddr.ipaddr.ip4addr.s_addr != htonl(INADDR_NONE)))) {
+		int i;
+
+		/*
+		 *	Loop over the home accounting servers for this
+		 *	realm.  If one of them has the same IP as the
+		 *	FreeRADIUS-Proxied-To attribute, then the
+		 *	packet has already been sent there.  Don't
+		 *	send it there again.
+		 */
+		for (i = 0; i < realm->acct_pool->num_home_servers; i++) {
+			if ((fr_ipaddr_cmp(&realm->acct_pool->servers[i]->ipaddr,
+					     &request->packet->src_ipaddr) == 0) &&
+			    (realm->acct_pool->servers[i]->port == request->packet->src_port)) {
+				DEBUG2("Suppressing proxy because packet was already sent to a server in that realm");
+				return 0;
+			}
+		}
+
+	}
 
 	/*
 	 *	We got this far, which means we have a realm, set returnrealm
@@ -299,7 +323,7 @@ static void add_proxy_to_realm(VALUE_PAIR **vps, REALM *realm)
 	 *	Tell the server to proxy this request to another
 	 *	realm.
 	 */
-	vp = pairmake("Proxy-To-Realm", realm->realm, T_OP_EQ);
+	vp = pairmake("Proxy-To-Realm", realm->name, T_OP_EQ);
 	if (!vp) {
 		radlog(L_ERR|L_CONS, "no memory");
 		exit(1);
@@ -341,7 +365,6 @@ static int realm_instantiate(CONF_SECTION *conf, void **instance)
 	     free(inst);
 	     return -1;
 	}
-	free(inst->formatstring);
 	if(strlen(inst->delim) != 1) {
 	     radlog(L_ERR, "Bad value \"%s\" for realm delimiter value", inst->delim);
 	     free(inst);
@@ -384,7 +407,7 @@ static int realm_authorize(void *instance, REQUEST *request)
 	 *	Maybe add a Proxy-To-Realm attribute to the request.
 	 */
 	DEBUG2("    rlm_realm: Preparing to proxy authentication request to realm \"%s\"\n",
-	       realm->realm);
+	       realm->name);
 	add_proxy_to_realm(&request->config_items, realm);
 
 	return RLM_MODULE_UPDATED; /* try the next module */
@@ -396,7 +419,7 @@ static int realm_authorize(void *instance, REQUEST *request)
  */
 static int realm_preacct(void *instance, REQUEST *request)
 {
-	const char *name = (char *)request->username->strvalue;
+	const char *name = (char *)request->username->vp_strvalue;
 	REALM *realm;
 
 	if (!name)
@@ -420,7 +443,7 @@ static int realm_preacct(void *instance, REQUEST *request)
 	 *	Maybe add a Proxy-To-Realm attribute to the request.
 	 */
 	DEBUG2("    rlm_realm: Preparing to proxy accounting request to realm \"%s\"\n",
-	       realm->realm);
+	       realm->name);
 	add_proxy_to_realm(&request->config_items, realm);
 
 	return RLM_MODULE_UPDATED; /* try the next module */
@@ -428,28 +451,25 @@ static int realm_preacct(void *instance, REQUEST *request)
 
 static int realm_detach(void *instance)
 {
-	struct realm_config_t *inst = instance;
-	free(inst->delim);
 	free(instance);
 	return 0;
 }
 
 /* globally exported name */
 module_t rlm_realm = {
-  "realm",
-  0,				/* type: reserved */
-  NULL,				/* initialization */
-  realm_instantiate,	       	/* instantiation */
-  {
-	  NULL,			/* authentication */
-	  realm_authorize,	/* authorization */
-	  realm_preacct,	/* preaccounting */
-	  NULL,			/* accounting */
-	  NULL,			/* checksimul */
-	  NULL,			/* pre-proxy */
-	  NULL,			/* post-proxy */
-	  NULL			/* post-auth */
-  },
-  realm_detach,			/* detach */
-  NULL,				/* destroy */
+	RLM_MODULE_INIT,
+	"realm",
+	RLM_TYPE_CHECK_CONFIG_SAFE | RLM_TYPE_HUP_SAFE,   	/* type */
+	realm_instantiate,	       	/* instantiation */
+	realm_detach,			/* detach */
+	{
+		NULL,			/* authentication */
+		realm_authorize,	/* authorization */
+		realm_preacct,	/* preaccounting */
+		NULL,			/* accounting */
+		NULL,			/* checksimul */
+		NULL,			/* pre-proxy */
+		NULL,			/* post-proxy */
+		NULL			/* post-auth */
+	},
 };
