@@ -1,7 +1,7 @@
 /*
  * sql_postgresql.c		Postgresql rlm_sql driver
  *
- * Version:	$Id$
+ * Version:	$Id: sql_postgresql.c,v 1.59 2008/04/11 12:00:42 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -15,9 +15,9 @@
  *
  *   You should have received a copy of the GNU General Public License
  *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
  *
- * Copyright 2000  The FreeRADIUS server project
+ * Copyright 2000,2006  The FreeRADIUS server project
  * Copyright 2000  Mike Machado <mike@innercite.com>
  * Copyright 2000  Alan DeKok <aland@ox.org>
  */
@@ -37,40 +37,37 @@
  * Bernhard Herzog <bh@intevation.de>
  */
 
-/* Modification of rlm_sql_mysql to handle postgres */
+#include <freeradius-devel/ident.h>
+RCSID("$Id: sql_postgresql.c,v 1.59 2008/04/11 12:00:42 aland Exp $")
+
+#include <freeradius-devel/radiusd.h>
 
-#include <stdio.h>
 #include <sys/stat.h>
-#include <stdlib.h>
-#include <string.h>
 
-#include "radiusd.h"
-
-#include        <libpq-fe.h>
-#include	"rlm_sql.h"
+#include <libpq-fe.h>
+#include "rlm_sql.h"
+#include "sql_postgresql.h"
 
 typedef struct rlm_sql_postgres_sock {
    PGconn          *conn;
    PGresult        *result;
    int             cur_row;
    int             num_fields;
-    int		   affected_rows;
+   int		   affected_rows;
    char            **row;
 } rlm_sql_postgres_sock;
 
 /* Prototypes */
-static int sql_store_result(SQLSOCK * sqlsocket, SQL_CONFIG *config);
-static int sql_num_fields(SQLSOCK * sqlsocket, SQL_CONFIG *config);
-static int sql_close(SQLSOCK * sqlsocket, SQL_CONFIG *config);
+static int sql_close(SQLSOCK *sqlsocket, SQL_CONFIG *config);
 
 /* Internal function. Return true if the postgresql status value
  * indicates successful completion of the query. Return false otherwise
- */
 static int
 status_is_ok(ExecStatusType status)
 {
 	return status == PGRES_COMMAND_OK || status == PGRES_TUPLES_OK;
 }
+*/
 
 
 /* Internal function. Return the number of affected rows of the result
@@ -90,52 +87,51 @@ free_result_row(rlm_sql_postgres_sock * pg_sock)
 	if (pg_sock->row != NULL) {
 		for (i = pg_sock->num_fields-1; i >= 0; i--) {
 			if (pg_sock->row[i] != NULL) {
-				xfree(pg_sock->row[i]);
+				free(pg_sock->row[i]);
 			}
 		}
-		xfree((char*)pg_sock->row);
+		free((char*)pg_sock->row);
 		pg_sock->row = NULL;
 		pg_sock->num_fields = 0;
 	}
 }
 
+
 /*************************************************************************
- *
- *      Function: sql_check_error
- *
- *      Purpose: check the error to see if the server is down
- *
- *	Note: It is possible that something other than a connection error
- *	could cause PGRES_FATAL_ERROR. If that happens a reconnect will
- *	occur anyway. Not optimal, but I couldn't find a way to check it.
- *		 		Peter Nixon <codemonkey@peternixon.net>
- *
+*	Function: check_fatal_error
+*
+*	Purpose:  Check error type and behave accordingly
+*
+*************************************************************************/
 
-************************************************************************/
-static int sql_check_error(int error) {
-        switch(error) {
-        case PGRES_FATAL_ERROR:
-        case -1:
-                radlog(L_DBG, "rlm_sql_postgresql: Postgresql check_error: %s, returning SQL_DOWN", PQresStatus(error));
-                return SQL_DOWN;
-                break;
+static int check_fatal_error (char *errorcode)
+{
+	int x = 0;
 
-        case PGRES_COMMAND_OK:
-        case PGRES_TUPLES_OK:
-        case 0:
-                return 0;
-                break;
+	/*
+	Check the error code to see if we should reconnect or not
+	Error Code table taken from
+	http://www.postgresql.org/docs/8.1/interactive/errcodes-appendix.html
+	*/
 
-        case PGRES_NONFATAL_ERROR:
-        case PGRES_BAD_RESPONSE:
-        default:
-                radlog(L_DBG, "rlm_sql_postgresql: Postgresql check_error: %s received", PQresStatus(error));
-                return -1;
-                break;
-        }
+	while(errorcodes[x].errorcode != NULL){
+		if (strcmp(errorcodes[x].errorcode, errorcode) == 0){
+			radlog(L_DBG, "rlm_sql_postgresql: Postgresql Fatal Error: [%s: %s] Occurred!!", errorcode, errorcodes[x].meaning);
+			if (errorcodes[x].shouldreconnect == 1)
+				return SQL_DOWN;
+			else
+				return -1;
+		}
+		x++;
+	}
+
+	radlog(L_DBG, "rlm_sql_postgresql: Postgresql Fatal Error: [%s] Occurred!!", errorcode);
+	/*	We don't seem to have a matching error class/code */
+	return -1;
 }
 
 
+
 /*************************************************************************
  *
  *	Function: sql_create_socket
@@ -145,7 +141,7 @@ static int sql_check_error(int error) {
  *************************************************************************/
 static int sql_init_socket(SQLSOCK *sqlsocket, SQL_CONFIG *config) {
 	char connstring[2048];
-	char *port, *host;
+	const char *port, *host;
 	rlm_sql_postgres_sock *pg_sock;
 
 	if (config->sql_server[0] != '\0') {
@@ -179,9 +175,9 @@ static int sql_init_socket(SQLSOCK *sqlsocket, SQL_CONFIG *config) {
 	pg_sock->result=NULL;
 	pg_sock->conn=PQconnectdb(connstring);
 
-	if (PQstatus(pg_sock->conn) == CONNECTION_BAD) {
+	if (PQstatus(pg_sock->conn) != CONNECTION_OK) {
 		radlog(L_ERR, "rlm_sql_postgresql: Couldn't connect socket to PostgreSQL server %s@%s:%s", config->sql_login, config->sql_server, config->sql_db);
-		radlog(L_ERR, "rlm_sql_postgresql: Postgresql error '%s'", PQerrorMessage(pg_sock->conn));
+		/*radlog(L_ERR, "rlm_sql_postgresql: Postgresql error '%s'", PQerrorMessage(pg_sock->conn));*/
 		sql_close(sqlsocket, config);
 		return SQL_DOWN;
 	}
@@ -199,6 +195,9 @@ static int sql_init_socket(SQLSOCK *sqlsocket, SQL_CONFIG *config) {
 static int sql_query(SQLSOCK * sqlsocket, SQL_CONFIG *config, char *querystr) {
 
 	rlm_sql_postgres_sock *pg_sock = sqlsocket->conn;
+	int numfields = 0;
+	char *errorcode;
+	char *errormsg;
 
 	if (config->sqltrace)
 		radlog(L_DBG,"rlm_sql_postgresql: query:\n%s", querystr);
@@ -209,35 +208,99 @@ static int sql_query(SQLSOCK * sqlsocket, SQL_CONFIG *config, char *querystr) {
 	}
 
 	pg_sock->result = PQexec(pg_sock->conn, querystr);
-		/* Returns a result pointer or possibly a NULL pointer.
-		 * A non-NULL pointer will generally be returned except in
+		/*
+		 * Returns a PGresult pointer or possibly a null pointer.
+		 * A non-null pointer will generally be returned except in
 		 * out-of-memory conditions or serious errors such as inability
-		 * to send the command to the backend. If a NULL is returned,
-		 *  it should be treated like a PGRES_FATAL_ERROR result.
-		 * Use PQerrorMessage to get more information about the error.
+		 * to send the command to the server. If a null pointer is
+		 * returned, it should be treated like a PGRES_FATAL_ERROR
+		 * result.
 		 */
 	if (!pg_sock->result)
 	{
 		radlog(L_ERR, "rlm_sql_postgresql: PostgreSQL Query failed Error: %s",
 				PQerrorMessage(pg_sock->conn));
+		/* As this error COULD be a connection error OR an out-of-memory
+		 * condition return value WILL be wrong SOME of the time regardless!
+		 * Pick your poison....
+		 */
 		return  SQL_DOWN;
 	} else {
 		ExecStatusType status = PQresultStatus(pg_sock->result);
-
 		radlog(L_DBG, "rlm_sql_postgresql: Status: %s", PQresStatus(status));
 
-		radlog(L_DBG, "rlm_sql_postgresql: affected rows = %s",
-				PQcmdTuples(pg_sock->result));
+		switch (status){
 
-		if (!status_is_ok(status))
-			return sql_check_error(status);
+			case PGRES_COMMAND_OK:
+				/*Successful completion of a command returning no data.*/
 
-		if (strncasecmp("select", querystr, 6) != 0) {
-			/* store the number of affected rows because the sql module
-			 * calls finish_query before it retrieves the number of affected
-			 * rows from the driver */
-			pg_sock->affected_rows = affected_rows(pg_sock->result);
-			return 0;
+				/*affected_rows function only returns
+				the number of affected rows of a command
+				returning no data...
+				*/
+				pg_sock->affected_rows	= affected_rows(pg_sock->result);
+				radlog(L_DBG, "rlm_sql_postgresql: query affected rows = %i", pg_sock->affected_rows);
+				return 0;
+
+			break;
+
+			case PGRES_TUPLES_OK:
+				/*Successful completion of a command returning data (such as a SELECT or SHOW).*/
+
+				pg_sock->cur_row = 0;
+ 				pg_sock->affected_rows = PQntuples(pg_sock->result);
+				numfields = PQnfields(pg_sock->result); /*Check row storing functions..*/
+				radlog(L_DBG, "rlm_sql_postgresql: query affected rows = %i , fields = %i", pg_sock->affected_rows, numfields);
+				return 0;
+
+			break;
+
+			case PGRES_BAD_RESPONSE:
+				/*The server's response was not understood.*/
+				radlog(L_DBG, "rlm_sql_postgresql: Bad Response From Server!!");
+				return -1;
+
+			break;
+
+			case PGRES_NONFATAL_ERROR:
+				/*A nonfatal error (a notice or warning) occurred. Possibly never returns*/
+
+				return -1;
+
+			break;
+
+			case PGRES_FATAL_ERROR:
+#if defined(PG_DIAG_SQLSTATE) && defined(PG_DIAG_MESSAGE_PRIMARY)
+				/*A fatal error occurred.*/
+
+				errorcode = PQresultErrorField(pg_sock->result, PG_DIAG_SQLSTATE);
+				errormsg  = PQresultErrorField(pg_sock->result, PG_DIAG_MESSAGE_PRIMARY);
+				radlog(L_DBG, "rlm_sql_postgresql: Error %s", errormsg);
+				return check_fatal_error(errorcode);
+#endif
+
+			break;
+
+			default:
+				/* FIXME: An unhandled error occurred.*/
+
+				/* PGRES_EMPTY_QUERY PGRES_COPY_OUT PGRES_COPY_IN */
+
+				return -1;
+
+			break;
+
+
+		}
+
+		/*
+			Note to self ... sql_store_result returns 0 anyway
+			after setting the sqlsocket->affected_rows..
+			sql_num_fields returns 0 at worst case which means the check below
+			has a really small chance to return false..
+			lets remove it then .. yuck!!
+		*/
+		/*
 		} else {
 			if ((sql_store_result(sqlsocket, config) == 0)
 					&& (sql_num_fields(sqlsocket, config) >= 0))
@@ -245,7 +308,9 @@ static int sql_query(SQLSOCK * sqlsocket, SQL_CONFIG *config, char *querystr) {
 			else
 				return -1;
 		}
+		*/
 	}
+	return -1;
 }
 
 
@@ -261,23 +326,6 @@ static int sql_select_query(SQLSOCK * sqlsocket, SQL_CONFIG *config, char *query
 }
 
 
-/*************************************************************************
- *
- *	Function: sql_store_result
- *
- *	Purpose: database specific store_result function. Returns a result
- *               set for the query.
- *
- *************************************************************************/
-static int sql_store_result(SQLSOCK * sqlsocket, SQL_CONFIG *config) {
-	rlm_sql_postgres_sock *pg_sock = sqlsocket->conn;
-
-	pg_sock->cur_row = 0;
-	pg_sock->affected_rows = PQntuples(pg_sock->result);
-	return 0;
-}
-
-
 /*************************************************************************
  *
  *      Function: sql_destroy_socket
@@ -285,44 +333,23 @@ static int sql_store_result(SQLSOCK * sqlsocket, SQL_CONFIG *config) {
  *      Purpose: Free socket and private connection data
  *
  *************************************************************************/
-static int sql_destroy_socket(SQLSOCK *sqlsocket, SQL_CONFIG *config)
+static int sql_destroy_socket(SQLSOCK *sqlsocket, UNUSED SQL_CONFIG *config)
 {
         free(sqlsocket->conn);
 	sqlsocket->conn = NULL;
         return 0;
 }
 
-/*************************************************************************
- *
- *	Function: sql_num_fields
- *
- *	Purpose: database specific num_fields function. Returns number
- *               of columns from query
- *
- *************************************************************************/
-static int sql_num_fields(SQLSOCK * sqlsocket, SQL_CONFIG *config) {
-
-	int num = 0;
-	rlm_sql_postgres_sock *pg_sock = sqlsocket->conn;
-
-	if (!(num = PQnfields(pg_sock->result))) {
-		radlog(L_ERR, "rlm_sql_postgresql: PostgreSQL Error: Cannot get result");
-		radlog(L_ERR, "rlm_sql_postgresql: PostgreSQL error: %s", PQerrorMessage(pg_sock->conn));
-	}
-	return num;
-}
-
-
 /*************************************************************************
  *
  *	Function: sql_fetch_row
  *
  *	Purpose: database specific fetch_row. Returns a SQL_ROW struct
- *               with all the data for the query in 'sqlsocket->row'. Returns
- *		 0 on success, -1 on failure, SQL_DOWN if 'database is down'.
+ *	with all the data for the query in 'sqlsocket->row'. Returns
+ *	0 on success, -1 on failure, SQL_DOWN if 'database is down'.
  *
  *************************************************************************/
-static int sql_fetch_row(SQLSOCK * sqlsocket, SQL_CONFIG *config) {
+static int sql_fetch_row(SQLSOCK * sqlsocket, UNUSED SQL_CONFIG *config) {
 
 	int records, i, len;
 	rlm_sql_postgres_sock *pg_sock = sqlsocket->conn;
@@ -345,14 +372,13 @@ static int sql_fetch_row(SQLSOCK * sqlsocket, SQL_CONFIG *config) {
 			len = PQgetlength(pg_sock->result, pg_sock->cur_row, i);
 			pg_sock->row[i] = (char *)rad_malloc(len+1);
 			memset(pg_sock->row[i], '\0', len+1);
-			strncpy(pg_sock->row[i], PQgetvalue(pg_sock->result, pg_sock->cur_row,i),len);
+			strlcpy(pg_sock->row[i], PQgetvalue(pg_sock->result, pg_sock->cur_row,i),len + 1);
 		}
 		pg_sock->cur_row++;
 		sqlsocket->row = pg_sock->row;
-		return 0;
-	} else {
-		return 0;
 	}
+
+	return 0;
 }
 
 
@@ -365,7 +391,7 @@ static int sql_fetch_row(SQLSOCK * sqlsocket, SQL_CONFIG *config) {
  *               for a result set
  *
  *************************************************************************/
-static int sql_free_result(SQLSOCK * sqlsocket, SQL_CONFIG *config) {
+static int sql_free_result(SQLSOCK * sqlsocket, UNUSED SQL_CONFIG *config) {
 
 	rlm_sql_postgres_sock *pg_sock = sqlsocket->conn;
 
@@ -373,12 +399,8 @@ static int sql_free_result(SQLSOCK * sqlsocket, SQL_CONFIG *config) {
 		PQclear(pg_sock->result);
 		pg_sock->result = NULL;
 	}
-#if 0
-	/*
-	 *  Commented out because it appears to free memory too early.
-	 */
+
 	free_result_row(pg_sock);
-#endif
 
 	return 0;
 }
@@ -393,7 +415,7 @@ static int sql_free_result(SQLSOCK * sqlsocket, SQL_CONFIG *config) {
  *               connection
  *
  *************************************************************************/
-static char *sql_error(SQLSOCK * sqlsocket, SQL_CONFIG *config) {
+static const char *sql_error(SQLSOCK * sqlsocket, UNUSED SQL_CONFIG *config) {
 
 	rlm_sql_postgres_sock *pg_sock = sqlsocket->conn;
 
@@ -409,7 +431,7 @@ static char *sql_error(SQLSOCK * sqlsocket, SQL_CONFIG *config) {
  *               connection
  *
  *************************************************************************/
-static int sql_close(SQLSOCK * sqlsocket, SQL_CONFIG *config) {
+static int sql_close(SQLSOCK * sqlsocket, UNUSED SQL_CONFIG *config) {
 
 	rlm_sql_postgres_sock *pg_sock = sqlsocket->conn;
 
@@ -457,7 +479,7 @@ static int sql_finish_select_query(SQLSOCK * sqlsocket, SQL_CONFIG *config) {
  *	Purpose: Return the number of rows affected by the last query.
  *
  *************************************************************************/
-static int sql_affected_rows(SQLSOCK * sqlsocket, SQL_CONFIG *config) {
+static int sql_affected_rows(SQLSOCK * sqlsocket, UNUSED SQL_CONFIG *config) {
 	rlm_sql_postgres_sock *pg_sock = sqlsocket->conn;
 
 	return pg_sock->affected_rows;
@@ -465,7 +487,7 @@ static int sql_affected_rows(SQLSOCK * sqlsocket, SQL_CONFIG *config) {
 
 
 static int NEVER_RETURNS
-not_implemented(SQLSOCK * sqlsocket, SQL_CONFIG *config)
+not_implemented(UNUSED SQLSOCK * sqlsocket, UNUSED SQL_CONFIG *config)
 {
 	radlog(L_ERR, "sql_postgresql: calling unimplemented function");
 	exit(1);
