@@ -2,11 +2,12 @@
  *  rlm_sql_log.c	Append the SQL queries in a log file which
  *			is read later by the radsqlrelay program
  *
- *  Version:    $Id$
+ *  Version:    $Id: rlm_sql_log.c,v 1.12 2007/12/15 21:26:51 aland Exp $
  *
  *  Author:     Nicolas Baradakis <nicolas.baradakis@cegetel.net>
  *
  *  Copyright (C) 2005 Cegetel
+ *  Copyright 2006 The FreeRADIUS server project
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -20,28 +21,37 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
  */
 
-#include "autoconf.h"
+#include <freeradius-devel/ident.h>
+RCSID("$Id: rlm_sql_log.c,v 1.12 2007/12/15 21:26:51 aland Exp $")
+
+#include <freeradius-devel/radiusd.h>
+#include <freeradius-devel/modules.h>
 
 #include <fcntl.h>
-#include <stdio.h>
-#include <stdlib.h>
 #include <sys/stat.h>
 
-#include "libradius.h"
-#include "radiusd.h"
-#include "modules.h"
-#include "conffile.h"
-
-static const char rcsid[] = "$Id$";
-
 static int sql_log_instantiate(CONF_SECTION *conf, void **instance);
 static int sql_log_detach(void *instance);
 static int sql_log_accounting(void *instance, REQUEST *request);
 static int sql_log_postauth(void *instance, REQUEST *request);
 
+#ifndef HAVE_PTHREAD_H
+/*
+ *      This is a lot simpler than putting ifdef's around
+ *      every use of the pthread functions.
+ */
+#define pthread_mutex_lock(a)
+#define pthread_mutex_trylock(a) (0)
+#define pthread_mutex_unlock(a)
+#define pthread_mutex_init(a,b)
+#define pthread_mutex_destroy(a)
+#else
+#include	<pthread.h>
+#endif
+
 #define MAX_QUERY_LEN 4096
 
 /*
@@ -54,6 +64,9 @@ typedef struct rlm_sql_log_t {
 	char		*sql_user_name;
 	char		*allowed_chars;
 	CONF_SECTION	*conf_section;
+#ifdef HAVE_PTHREAD_H
+	pthread_mutex_t	mutex;
+#endif
 } rlm_sql_log_t;
 
 /*
@@ -169,7 +182,7 @@ static int sql_log_detach(void *instance)
 /*
  *	Translate the SQL queries.
  */
-static int sql_escape_func(char *out, int outlen, const char *in)
+static size_t sql_escape_func(char *out, size_t outlen, const char *in)
 {
 	int len = 0;
 
@@ -230,7 +243,7 @@ static int sql_set_user(rlm_sql_log_t *inst, REQUEST *request, char *sqlusername
 	pairdelete(&request->packet->vps, PW_SQL_USER_NAME);
 
 	if (username != NULL) {
-		strNcpy(tmpuser, username, MAX_STRING_LEN);
+		strlcpy(tmpuser, username, MAX_STRING_LEN);
 	} else if (inst->sql_user_name[0] != '\0') {
 		radius_xlat(tmpuser, sizeof(tmpuser), inst->sql_user_name,
 			    request, NULL);
@@ -239,7 +252,7 @@ static int sql_set_user(rlm_sql_log_t *inst, REQUEST *request, char *sqlusername
 	}
 
 	if (tmpuser[0] != '\0') {
-		strNcpy(sqlusername, tmpuser, sizeof(tmpuser));
+		strlcpy(sqlusername, tmpuser, sizeof(tmpuser));
 		DEBUG2("rlm_sql_log (%s): sql_set_user escaped user --> '%s'",
 		       inst->name, sqlusername);
 		vp = pairmake("SQL-User-Name", sqlusername, 0);
@@ -311,8 +324,9 @@ static int sql_log_write(rlm_sql_log_t *inst, REQUEST *request, const char *line
 
 	path[0] = '\0';
 	radius_xlat(path, sizeof(path), inst->path, request, NULL);
-	if (path[0] == '\0')
+	if (path[0] == '\0') {
 		return RLM_MODULE_FAIL;
+	}
 
 	while (!locked) {
 		if ((fd = open(path, O_WRONLY | O_APPEND | O_CREAT, 0666)) < 0) {
@@ -360,7 +374,7 @@ static int sql_log_accounting(void *instance, REQUEST *request)
 {
 	int		ret;
 	char		querystr[MAX_QUERY_LEN];
-	char		*cfquery;
+	const char	*cfquery;
 	rlm_sql_log_t	*inst = (rlm_sql_log_t *)instance;
 	VALUE_PAIR	*pair;
 	DICT_VALUE	*dval;
@@ -376,9 +390,9 @@ static int sql_log_accounting(void *instance, REQUEST *request)
 	}
 
 	/* Search the query in conf section of the module */
-	if ((dval = dict_valbyattr(PW_ACCT_STATUS_TYPE, pair->lvalue)) == NULL) {
+	if ((dval = dict_valbyattr(PW_ACCT_STATUS_TYPE, pair->vp_integer)) == NULL) {
 		radlog(L_ERR, "rlm_sql_log (%s): Unsupported Acct-Status-Type = %d",
-		       inst->name, pair->lvalue);
+		       inst->name, pair->vp_integer);
 		return RLM_MODULE_NOOP;
 	}
 	if ((cp = cf_pair_find(inst->conf_section, dval->name)) == NULL) {
@@ -428,10 +442,11 @@ static int sql_log_postauth(void *instance, REQUEST *request)
  *	is single-threaded.
  */
 module_t rlm_sql_log = {
+	RLM_MODULE_INIT,
 	"sql_log",
-	RLM_TYPE_THREAD_SAFE,		/* type */
-	NULL,				/* initialization */
+	RLM_TYPE_THREAD_UNSAFE | RLM_TYPE_CHECK_CONFIG_SAFE | RLM_TYPE_HUP_SAFE,		/* type */
 	sql_log_instantiate,		/* instantiation */
+	sql_log_detach,			/* detach */
 	{
 		NULL,			/* authentication */
 		NULL,			/* authorization */
@@ -442,6 +457,4 @@ module_t rlm_sql_log = {
 		NULL,			/* post-proxy */
 		sql_log_postauth	/* post-auth */
 	},
-	sql_log_detach,			/* detach */
-	NULL,				/* destroy */
 };
