@@ -2,7 +2,7 @@
  * rlm_sqlhpwippool.c
  * Chooses an IPv4 address from pools defined in ASN Netvim
  *
- * Version:	$Id$
+ * Version:	$Id: rlm_sqlhpwippool.c,v 1.7 2007/04/20 14:31:31 aland Exp $
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
@@ -34,20 +34,11 @@
  */
 
 #include "config.h"
-#include "autoconf.h"
-#include "libradius.h"
+#include <freeradius-devel/radiusd.h>
+#include <freeradius-devel/modules.h>
+#include <freeradius-devel/modpriv.h>
 
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
 #include <ctype.h>
-#include <stdarg.h>
-#include <netinet/in.h>
-
-#include "radiusd.h"
-#include "modules.h"
-#include "conffile.h"
-#include "modpriv.h"
 
 #include "rlm_sql.h"
 
@@ -59,7 +50,7 @@
 #define RLM_NETVIM_MAX_ROWS 1000000
 #define RLM_NETVIM_TMP_PREFIX "auth-tmp-"
 
-static const char rcsid[] = "$Id$";
+static const char rcsid[] = "$Id: rlm_sqlhpwippool.c,v 1.7 2007/04/20 14:31:31 aland Exp $";
 
 typedef struct rlm_sqlhpwippool_t {
 	const char *myname;         /* name of this instance */
@@ -330,7 +321,7 @@ static int sqlhpwippool_instantiate(CONF_SECTION *conf, void **instance)
 
 	data->sincesync = 0;
 
-	modinst = find_module_instance(data->sqlinst_name);
+	modinst = find_module_instance(cf_section_find("modules"), (data->sqlinst_name) );
 	if (!modinst) {
 		nvp_log(__LINE__, data, L_ERR,
 		        "sqlhpwippool_instantiate(): cannot find module instance "
@@ -388,7 +379,7 @@ static int sqlhpwippool_postauth(void *instance, REQUEST *request)
 	/* if no pool name, we don't need to do anything */
 	vp = pairfind(request->reply->vps, PW_ASN_IP_POOL_NAME);
 	if (vp) {
-		pname = vp->strvalue;
+		pname = vp->vp_strvalue;
 		nvp_log(__LINE__, data, L_DBG,
 		        "sqlhpwippool_postauth(): pool name = '%s'",
 		        pname);
@@ -402,7 +393,7 @@ static int sqlhpwippool_postauth(void *instance, REQUEST *request)
 	/* if no NAS IP address, assign 0 */
 	vp = pairfind(request->packet->vps, PW_NAS_IP_ADDRESS);
 	if (vp) {
-		nasip = ntohl(vp->lvalue);
+		nasip = ntohl(vp->vp_ipaddr);
 	}
 	else {
 		nasip = 0;
@@ -669,19 +660,9 @@ end_gid:
 	}
 
 	/* add IP address to reply packet */
-	vp = paircreate(PW_FRAMED_IP_ADDRESS, PW_TYPE_IPADDR);
-	if (!vp) {
-		nvp_log(__LINE__, data, L_ERR,
-		        "sqlhpwippool_postauth(): couldn't save chosen IP - no memory - "
-		        "exiting with error");
-
-		/* don't free chosen IP, because we have no memory (now adress will need to
-		 * wait for timeout, what's not so bad, BTW) */
-		return RLM_MODULE_FAIL;
-	}
-
-	vp->lvalue = ip.s_addr;
-	pairadd(&request->reply->vps, vp);
+	vp = radius_paircreate(request, &request->reply->vps,
+			       PW_FRAMED_IP_ADDRESS, PW_TYPE_IPADDR);
+	vp->vp_ipaddr = ip.s_addr;
 
 	nvp_log(__LINE__, data, L_DBG, "sqlhpwippool_postauth(): returning %s",
 	        inet_ntoa(ip));
@@ -703,7 +684,7 @@ static int sqlhpwippool_accounting(void *instance, REQUEST *request)
 	/* if no unique session ID, don't even try */
 	vp = pairfind(request->packet->vps, PW_ACCT_UNIQUE_SESSION_ID);
 	if (vp) {
-		sessid = vp->strvalue;
+		sessid = vp->vp_strvalue;
 	}
 	else {
 		nvp_log(__LINE__, data, L_ERR,
@@ -713,7 +694,7 @@ static int sqlhpwippool_accounting(void *instance, REQUEST *request)
 
 	vp = pairfind(request->packet->vps, PW_ACCT_STATUS_TYPE);
 	if (vp) {
-		acct_type = vp->lvalue;
+		acct_type = vp->vp_integer;
 	}
 	else {
 		nvp_log(__LINE__, data, L_ERR, "sqlhpwippool_accounting(): "
@@ -748,7 +729,7 @@ static int sqlhpwippool_accounting(void *instance, REQUEST *request)
 				return RLM_MODULE_FAIL;
 			}
 
-			framedip = ntohl(vp->lvalue);
+			framedip = ntohl(vp->vp_ipaddr);
 
 			if (!nvp_query(__LINE__, data, sqlsock,
 			    "UPDATE `%s`.`ips` "
@@ -788,7 +769,7 @@ static int sqlhpwippool_accounting(void *instance, REQUEST *request)
 				return RLM_MODULE_FAIL;
 			}
 
-			nasip.s_addr = vp->lvalue;
+			nasip.s_addr = vp->vp_ipaddr;
 			strncpy(nasipstr, inet_ntoa(nasip), sizeof(nasipstr) - 1);
 			nasipstr[sizeof(nasipstr)] = 0;
 
@@ -812,20 +793,19 @@ static int sqlhpwippool_accounting(void *instance, REQUEST *request)
 }
 
 module_t rlm_sqlhpwippool = {
-	"sqlhpwippool",             /* name */
-	RLM_TYPE_THREAD_SAFE,      /* type */
-	NULL,
-	sqlhpwippool_instantiate,   /* instantiation */
+	RLM_MODULE_INIT,
+	"sqlhpwippool",			/* name */
+	RLM_TYPE_THREAD_SAFE,		/* type */
+	sqlhpwippool_instantiate,	/* instantiation */
+	sqlhpwippool_detach,		/* detach */
 	{
-		NULL,                    /* authentication */
-		NULL,                    /* authorization */
-		NULL,                    /* preaccounting */
-		sqlhpwippool_accounting,  /* accounting */
-		NULL,                    /* checksimul */
-		NULL,                    /* pre-proxy */
-		NULL,                    /* post-proxy */
-		sqlhpwippool_postauth     /* post-auth */
+		NULL,			/* authentication */
+		NULL,			/* authorization */
+		NULL,			/* preaccounting */
+		sqlhpwippool_accounting,/* accounting */
+		NULL,			/* checksimul */
+		NULL,			/* pre-proxy */
+		NULL,			/* post-proxy */
+		sqlhpwippool_postauth	/* post-auth */
 	},
-	sqlhpwippool_detach,        /* detach */
-	NULL
 };
